/* Copyright (C) 2003, 2004 MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

   
#include "myadmin.h"

#include "MAdministrator.h"
#include "MDataInterface.h"
#include "MACatalogsPanel.h"

#include "myg_utils.h"


MACatalogsPanel::MACatalogsPanel(MAdministrator *app, MDataInterface *data)
    : MAPanel(app, data), _tables(0), _table_status(0)
{
}


MACatalogsPanel::~MACatalogsPanel()
{
  _schema_browser->unreference();

  if (_table_status)
    myx_free_schema_table_status(_table_status);
  
  delete _check_dlg_xml;
  delete _repair_dlg_xml;
}


void MACatalogsPanel::setup_trees()
{
  Gtk::TreeView *tree;
  Gtk::TreeView::Column *column;
  
  // table listing
  _table_store= Gtk::ListStore::create(_table_columns);
  tree= get_tree("table_tree");
  tree->set_model(_table_store);
  tree->get_selection()->set_mode(Gtk::SELECTION_MULTIPLE);
  
  column= new Gtk::TreeView::Column(_("Table Name"));
  column->pack_start(_table_columns._icon, false);
  column->pack_start(_table_columns._name);
  tree->append_column(*Gtk::manage(column));
  tree->append_column(_("Type"), _table_columns._type);
  tree->append_column(_("Row Format"), _table_columns._format);
  tree->append_column(_("Rows"), _table_columns._rows);
  tree->append_column(_("Data Length"), _table_columns._length);
  tree->append_column(_("Update Time"), _table_columns._time);
  for (int i= 0; i < 6; i++)
    tree->get_column(i)->set_resizable(true);
  tree->get_selection()->signal_changed().connect(SigC::slot(*this,&MACatalogsPanel::selected_table));

  // column listing
  _column_store= Gtk::TreeStore::create(_column_columns);
  tree= get_tree("column_tree");
  tree->set_model(_column_store);
  
  column= new Gtk::TreeView::Column(_("Column Name"));
  column->pack_start(_column_columns._icon, false);
  column->pack_start(_column_columns._name);
  tree->append_column(*Gtk::manage(column));
  tree->append_column(_("Type"), _column_columns._type);
  tree->append_column(_("Default"), _column_columns._default);
  tree->append_column(_("Primary"), _column_columns._key);
  tree->append_column(_("Null Allowed"), _column_columns._null);
  tree->append_column(_("Other"), _column_columns._extra);
  for (int i= 0; i < 6; i++)
    tree->get_column(i)->set_resizable(true);

  // table index listing
  _tindex_store= Gtk::TreeStore::create(_index_columns);
  tree= get_tree("tindex_tree");
  tree->set_model(_tindex_store);

  column= new Gtk::TreeView::Column(_("Key Name/Columns"));
  column->pack_start(_index_columns._icon, false);
  column->pack_start(_index_columns._name);
  tree->append_column(*Gtk::manage(column));
  tree->append_column(_("Index Type"), _index_columns._type);
  tree->append_column(_("Unique"), _index_columns._unique);
  tree->append_column(_("Null Allowed"), _index_columns._null);
  tree->append_column(_("Seq. in Index"), _index_columns._seq);
  tree->append_column(_("Collation"), _index_columns._coll);
  for (int i= 0; i < 6; i++)
    tree->get_column(i)->set_resizable(true);

  // index listing
  _index_store= Gtk::TreeStore::create(_index_columns);
  tree= get_tree("index_tree");
  tree->set_model(_index_store);
  
  column= new Gtk::TreeView::Column(_("Table Name/Column"));
  column->pack_start(_index_columns._icon, false);
  column->pack_start(_index_columns._name);
  tree->append_column(*Gtk::manage(column));
  tree->append_column(_("Key Name"), _index_columns._key);
  tree->append_column(_("Index Type"), _index_columns._type);
  tree->append_column(_("Unique"), _index_columns._unique);
  tree->append_column(_("Null Allowed"), _index_columns._null);
  tree->append_column(_("Seq. in Index"), _index_columns._seq);
  tree->append_column(_("Collation"), _index_columns._coll);
  for (int i= 0; i < 7; i++)
    tree->get_column(i)->set_resizable(true);

  // view listing
  _view_store= Gtk::ListStore::create(_view_columns);
  tree= get_tree("view_tree");
  tree->set_model(_view_store);
  
  column= new Gtk::TreeView::Column(_("View Name"));
  column->pack_start(_view_columns._icon, false);
  column->pack_start(_view_columns._name);
  tree->append_column(*Gtk::manage(column));
  tree->append_column(_("Type"), _view_columns._type);
  tree->append_column(_("Tables"), _view_columns._tables);
  tree->append_column(_("Time"), _view_columns._time);
  for (int i= 0; i < 4; i++)
    tree->get_column(i)->set_resizable(true);

  
  // user listing
  _user_store= Gtk::ListStore::create(_user_columns);
  tree= get_tree("user_tree");
  tree->set_model(_user_store);
  
  column= new Gtk::TreeView::Column(_("User"));
  column->pack_start(_user_columns._icon, false);
  column->pack_start(_user_columns._name);
  tree->append_column(*Gtk::manage(column));
  tree->append_column(_("Full Name"), _user_columns._fullname);
  tree->append_column(_("Privileges"), _user_columns._privileges);
  for (int i= 0; i < 3; i++)
    tree->get_column(i)->set_resizable(true);

  // stored procedures
  _procedure_store= Gtk::ListStore::create(_procedure_columns);
  tree= get_tree("procedure_tree");
  tree->set_model(_procedure_store);
  
  column= new Gtk::TreeView::Column(_("Function Name"));
  column->pack_start(_procedure_columns._icon, false);
  column->pack_start(_procedure_columns._name);
  tree->append_column(*Gtk::manage(column));
  tree->append_column(_("Parameters"), _procedure_columns._parameter);
  tree->append_column(_("Create Time"), _procedure_columns._ctime);
  tree->append_column(_("Update Time"), _procedure_columns._utime);
  for (int i= 0; i < 4; i++)
    tree->get_column(i)->set_resizable(true);

  // events
  _event_store= Gtk::ListStore::create(_event_columns);
  tree= get_tree("event_tree");
  tree->set_model(_event_store);
  
  column= new Gtk::TreeView::Column(_("Event Name"));
  column->pack_start(_event_columns._icon, false);
  column->pack_start(_event_columns._name);
  tree->append_column(*Gtk::manage(column));
  tree->append_column(_("Type"), _event_columns._type);
  tree->append_column(_("Create Time"), _event_columns._ctime);
  tree->append_column(_("Update Time"), _event_columns._utime);
  for (int i= 0; i < 4; i++)
    tree->get_column(i)->set_resizable(true);

  // doc listing
  _doc_store= Gtk::TreeStore::create(_doc_columns);
  tree= get_tree("doc_tree");
  tree->set_model(_doc_store);
  
  column= new Gtk::TreeView::Column(_("Developer Documents"));
  column->pack_start(_doc_columns._icon, false);
  column->pack_start(_doc_columns._name);
  tree->append_column(*Gtk::manage(column));
  tree->append_column(_("Type"), _doc_columns._type);
  tree->append_column(_("Version"), _doc_columns._version);
  tree->append_column(_("User"), _doc_columns._user);
  tree->append_column(_("Checked Out"), _doc_columns._checked_out);
  tree->append_column(_("Description"), _doc_columns._description);
  for (int i= 0; i < 6; i++)
    tree->get_column(i)->set_resizable(true);
}


bool MACatalogsPanel::init()
{
  if (_xml)
    return true;

  if (!MAPanel::init_from_glade(GLADE_DATADIR"/"GLADE_CATALOGS_FILE, "panel_frame"))
    return false;

  _check_dlg_xml= new MGGladeXML(GLADE_DATADIR"/"GLADE_CATALOGS_FILE, "check_dialog");
  _repair_dlg_xml= new MGGladeXML(GLADE_DATADIR"/"GLADE_CATALOGS_FILE, "repair_dialog");
  
  _schema_icon= _app->load_pixbuf("16x16_Database.png");
  _sys_schema_icon= _schema_icon;
  _table_icon= _app->load_pixbuf("16x16_Table.png");
  _column_icon= _app->load_pixbuf("16x16_Field.png");
  _key_column_icon= _app->load_pixbuf("16x16_KeyColumn.png");
  _index_icon= _app->load_pixbuf("16x16_Index.png");
  _doc_icon= _app->load_pixbuf("16x16_DBModel.png");

  setup_trees();

  _schema_browser= new MGSchemaBrowserList(_("Schemata"), true);
  _schema_browser->signal_selected().connect(SigC::slot(*this,&MACatalogsPanel::schema_selected));
//  _schema_browser->set_mark_delegate(schema_mark_delegate, this);
  _schema_browser->set_icons(_schema_icon, _sys_schema_icon, _table_icon, _column_icon);
  _schema_browser->show();

  get_button("optimize_button")->signal_clicked().connect(SigC::slot(*this,&MACatalogsPanel::optimize_table));
  get_button("check_button")->signal_clicked().connect(SigC::slot(*this,&MACatalogsPanel::check_table));
  get_button("repair_button")->signal_clicked().connect(SigC::slot(*this,&MACatalogsPanel::repair_table));

  return true;
}


void MACatalogsPanel::show()
{
  MAPanel::show();

  _schema_browser->set_catalogs(_data->get_catalogs());
}


bool MACatalogsPanel::before_show()
{
  _app->add_side_panel(_schema_browser);
  return true;
}


bool MACatalogsPanel::before_hide()
{
  _app->remove_side_panel(_schema_browser);
  return true;
}


void MACatalogsPanel::refresh_table_info(const Glib::ustring &schema)
{
  Gtk::TreeIter iter;
  Gtk::TreeModel::Row row;

  get_label("table_label")->set_markup(ufmt(_("<b>%s</b>\n"
                                              "All tables of the '%s' schema"),
                                            schema.c_str(),
                                            schema.c_str()));

  _table_store->clear();

  if (_table_status)
  {
    // fill table list
    for (unsigned int i= 0; i < _table_status->schema_tables_num; ++i)
    {
      iter= _table_store->append();
      row= *iter;

      row[_table_columns._icon]= _table_icon;
      row[_table_columns._name]= (char*)_table_status->schema_tables[i].table_name;
      row[_table_columns._type]= (char*)_table_status->schema_tables[i].table_type;
      row[_table_columns._format]= (char*)_table_status->schema_tables[i].row_format;
      row[_table_columns._rows]= (char*)_table_status->schema_tables[i].rows;
      row[_table_columns._length]= format_value(atol((char*)_table_status->schema_tables[i].data_length));
      row[_table_columns._time]= (char*)_table_status->schema_tables[i].update_time?:"";
      
      row[_table_columns._status]= _table_status->schema_tables+i;
      
      // find the table data for this one
      for (unsigned int j= 0; j < _tables->schema_tables_num; ++j)
      {
        if (strcmp((char*)_tables->schema_tables[j].table_name, 
                   (char*)_table_status->schema_tables[i].table_name)==0)
        {
          row[_table_columns._table]= _tables->schema_tables+j;
          break;
        }
      }
    }
  }
}


void MACatalogsPanel::refresh_index_info(const Glib::ustring &schema)
{
  get_label("index_label")->set_markup(ufmt(_("<b>%s</b>\n"
                                              "All indices in schema '%s'"),
                                            schema.c_str(),
                                            schema.c_str()));
  
  _index_store->clear();

  if (_table_status)
  {
    // fill table list
    for (unsigned int i= 0; i < _table_status->schema_tables_num; ++i)
    {
      for (unsigned int j= 0; j < _table_status->schema_tables[i].indexes_num; j++)
      {
        Gtk::TreeIter iter;
        Gtk::TreeModel::Row row;

        MYX_TABLE_INDEX *index= _table_status->schema_tables[i].indexes+j;

        iter= _index_store->append();
        row= *iter;

        row[_index_columns._icon]= _index_icon;
        row[_index_columns._name]= (char*)_table_status->schema_tables[i].table_name;
        row[_index_columns._key]= index->key_name;
        row[_index_columns._type]= index->index_type;
        row[_index_columns._unique]= index->is_non_unique?"":_("UNIQUE");
        row[_index_columns._null]= index->is_null_allowed?_("Yes"):"";
        
        for (unsigned int k= 0; k < index->index_columns_num; k++)
        {
          Gtk::TreeIter kter= _index_store->append(row.children());
          Gtk::TreeModel::Row crow= *kter;
        
          crow[_index_columns._icon]= _column_icon;
          crow[_index_columns._name]= index->index_columns[k].column_name;
          crow[_index_columns._seq]= index->index_columns[k].seq_in_index;
          crow[_index_columns._coll]= strcmp2(index->index_columns[k].collation,"A")==0?_("Ascending"):"";
        }
      }
    }
  }
}

void MACatalogsPanel::refresh_event_info(const Glib::ustring &schema)
{
  get_label("event_label")->set_markup(ufmt(_("<b>%s</b>\n"
                                              "All events in schema '%s'"),
                                            schema.c_str(),
                                            schema.c_str()));
}


void MACatalogsPanel::refresh_view_info(const Glib::ustring &schema)
{
  get_label("view_label")->set_markup(ufmt(_("<b>%s</b>\n"
                                             "All views in schema '%s'"),
                                           schema.c_str(),
                                           schema.c_str()));
}


void MACatalogsPanel::refresh_procedure_info(const Glib::ustring &schema)
{
  get_label("procedure_label")->set_markup(ufmt(_("<b>%s</b>\n"
                                                  "All stored procedures in schema '%s'"),
                                                schema.c_str(),
                                                schema.c_str()));
}


void MACatalogsPanel::refresh_doc_info(const Glib::ustring &schema)
{
  Gtk::TreeIter iter, jter;
  Gtk::TreeModel::Row row;
  
  get_label("doc_label")->set_markup(ufmt(_("<b>%s</b>\n"
                                            "Developer documents stored in schema '%s'"),
                                          schema.c_str(),
                                          schema.c_str()));

  _doc_store->clear();

  get_tree("doc_tree")->expand_all();
}


void MACatalogsPanel::refresh_user_info(const Glib::ustring &schema)
{
  get_label("user_label")->set_markup(ufmt(_("<b>%s</b>\n"
                                             "Users with privileges to connect to '%s'"),
                                           schema.c_str(),
                                           schema.c_str()));
}



void MACatalogsPanel::selected_table()
{
  Gtk::TreeView *tree= get_tree("table_tree");
  Gtk::TreeIter iter;
  Gtk::TreeModel::Row row;
  MYX_TABLE_STATUS *status= NULL;
  MYX_SCHEMA_TABLE *table= NULL;
  Gtk::Label *label= get_label("status_label");

  if (tree->get_selection()->count_selected_rows()==1)
  {
    iter= _table_store->get_iter(*tree->get_selection()->get_selected_rows().begin());
  }
  // else no selection

  if (iter)
  {
    get_widget("table_notebook")->set_sensitive(true);
    row= *iter;
    status= row[_table_columns._status];
    table= row[_table_columns._table];
  }
  else
  {
    get_widget("table_notebook")->set_sensitive(false);
  }
  
  if (status)
    label->set_markup(ufmt("<b>%s\n%s\n%s\n%s\n%s</b>",
                           (char*)status->table_name?:"",
                           (char*)status->table_type?:"",
                           (char*)status->row_format?:"",
                           (char*)status->auto_increment?:"",
                           (char*)status->create_options?:""));
  else
    label->set_text("");
  
  label= get_label("data_label");
  if (status)
    label->set_markup(ufmt("<b>%s\n%s\n%s\n%s\n%s\n%s</b>",
                           (char*)status->rows?:"",
                         (char*)status->avg_row_length?:"",
                         (char*)status->data_length?:"",
                         (char*)status->max_data_length?:"",
                         (char*)status->index_length?:"",
                         (char*)status->data_free?:""));
  else
    label->set_text("");

  label= get_label("time_label");
  if (status)
    label->set_markup(ufmt("<b>%s\n%s\n%s</b>",
                           (char*)status->create_time?:"",
                           (char*)status->update_time?:"",
                           (char*)status->check_time?:""));
  else
    label->set_text("");
  
  
  // fill columns
  _column_store->clear();

  if (table)
  {
    for (unsigned int i= 0; i < table->columns_num; i++)
    {
      iter= _column_store->append();
      row= *iter;
      
      row[_column_columns._icon]= table->columns[i].is_primary_key ? _key_column_icon : _column_icon;
      row[_column_columns._name]= (char*)table->columns[i].column_name;
      row[_column_columns._type]= (char*)table->columns[i].column_type;
      row[_column_columns._key]= table->columns[i].is_primary_key ? _("Yes") : "";
      row[_column_columns._null]= table->columns[i].is_null_allowed ? _("Yes") : "";
      row[_column_columns._extra]= (char*)table->columns[i].extra ? : "";
      row[_column_columns._default]= (char*)table->columns[i].default_value ? : "";
    }
  }
  
  // fill index
  _tindex_store->clear();
  
  if (status)
  {
    for (unsigned int j= 0; j < status->indexes_num; j++)
    {
      Gtk::TreeIter iter;
      Gtk::TreeModel::Row row;

      MYX_TABLE_INDEX *index= status->indexes+j;
      
      iter= _tindex_store->append();
      row= *iter;

      row[_index_columns._icon]= _index_icon;
      row[_index_columns._name]= index->key_name;
      row[_index_columns._type]= index->index_type;
      row[_index_columns._unique]= index->is_non_unique?"":_("UNIQUE");
      row[_index_columns._null]= index->is_null_allowed?_("Yes"):"";
      
      for (unsigned int k= 0; k < index->index_columns_num; k++)
      {
        Gtk::TreeIter kter= _tindex_store->append(row.children());
        Gtk::TreeModel::Row crow= *kter;
        
        crow[_index_columns._icon]= _column_icon;
        crow[_index_columns._name]= index->index_columns[k].column_name;
        crow[_index_columns._seq]= index->index_columns[k].seq_in_index;
        crow[_index_columns._coll]= strcmp2(index->index_columns[k].collation,"A")==0?_("Ascending"):"";
      }
    }
  }
}



void MACatalogsPanel::schema_selected(MGBrowserList *sender,
                                      const Gtk::TreeIter &node)
{
  Glib::ustring catalog;
  Glib::ustring schema;
  Glib::ustring table;
  Glib::ustring column;
  Glib::ustring child;

  if (node)
    _schema_browser->get_row_object(node, catalog, schema, table, column, child);

  if (_table_status)
  {
    myx_free_schema_table_status(_table_status);
    _table_status= NULL;
  }

  if (!schema.empty())
  {
    // this is owned by MDataInterface (XXX)
    _tables= _data->get_schema_tables("def", schema.c_str());

    _table_status= _data->get_schema_table_status("def", schema.c_str());
  }

  refresh_table_info(schema);
  refresh_index_info(schema);
  refresh_view_info(schema);
  refresh_procedure_info(schema);
  refresh_event_info(schema);
  refresh_doc_info(schema);
  refresh_user_info(schema);
}


Glib::ustring MACatalogsPanel::get_selected_tables()
{
  Gtk::TreeView *tree= get_tree("table_tree");
  std::list<Gtk::TreePath> selected;
  Gtk::TreeIter iter;
  Gtk::TreeModel::Row row;
  Glib::ustring objects;
  Glib::ustring schema;

  {
    Glib::ustring catalog;
    Glib::ustring table;
    Glib::ustring column;
    Glib::ustring child;

    _schema_browser->get_row_object(_schema_browser->get_selected(), 
                                    catalog, schema, table, column, child);
  }

  selected= tree->get_selection()->get_selected_rows();

  for (std::list<Gtk::TreePath>::const_iterator i= selected.begin();
       i != selected.end(); ++i)
  {
    Glib::ustring table;

    iter= _table_store->get_iter(*i);
    row= *iter;

    table= row[_table_columns._name];

    if (objects.empty())
    {
      objects= schema+"."+table;
    }
    else
    {
      objects= objects + ", "+schema+"."+table;
    }
  }

  return objects;
}


void MACatalogsPanel::optimize_table()
{
  Glib::ustring objects= get_selected_tables();
  
  if (objects.empty())
  {
    Gtk::MessageDialog dlg(*_app->window(), 
                           _("Please select the tables you wish to optimize."),
                           Gtk::MESSAGE_ERROR,
                           Gtk::BUTTONS_OK,
                           true);
    dlg.set_title(_("Optimize Table"));

    dlg.run();
    return;
  }

  {
    Gtk::MessageDialog dlg(*_app->window(), 
                           _("Optimize Table to defragment the datafile and reclaim the unused space"
                               "resulting from large changes in tables with variable-length records"
                               "(tables with VARCHAR, BLOB or TEXT columns)\n"
                               "Warning: the selected table(s) will be locked during this operation."),
                           Gtk::MESSAGE_INFO,
                           Gtk::BUTTONS_OK_CANCEL,
                           true);
    Gtk::CheckButton *btn= Gtk::manage(new Gtk::CheckButton(_("Write entry to binary log")));
    dlg.set_title(_("Optimize Table"));
    btn->set_active(true);

    dlg.get_vbox()->pack_start(*btn);
    btn->show();

    if (dlg.run() != Gtk::RESPONSE_OK)
      return;
  }

  MYX_TABLE_COMMAND_STATUSES *status;

  status= (MYX_TABLE_COMMAND_STATUSES*)
    _data->get_instance()->perform_data_fetch2((MInstanceInfo::DataFetcher2)myx_optimize_table,
                                               (void*)objects.c_str(),
                                               _("Optimizing table(s)..."));
  if (!status)
  {
    _data->show_last_error(_("Error executing OPTIMIZE TABLE."));
    return;
  }
  
  Glib::ustring msg;
  
  for (unsigned int i= 0; i < status->status_num; i++)
  {
    msg += "\n";
    switch (status->status[i].message_type)
    {
    case MYX_MESSAGE_STATUS:
      msg += ufmt(_("Table: %s  Status: %s"),
                  status->status[i].table,
                  status->status[i].message);
      break;
    case MYX_MESSAGE_ERROR:
      msg += ufmt(_("Table: %s  ERROR: %s"),
                  status->status[i].table,
                  status->status[i].message);
      break;
    case MYX_MESSAGE_WARNING:
      msg += ufmt(_("Table: %s  WARNING: %s"),
                  status->status[i].table,
                  status->status[i].message);
      break;
    case MYX_MESSAGE_INFO:
      msg += ufmt(_("Table: %s  Info: %s"),
                  status->status[i].table,
                  status->status[i].message);
      break;
    }
  }
  myx_free_command_status(status);

  {
    Gtk::MessageDialog dlg(*_app->window(), 
                           _("Table Optimization finished. Return status is:\n")+msg,
                           Gtk::MESSAGE_INFO,
                           Gtk::BUTTONS_OK,
                           true, true);
    dlg.set_title(_("Optimize Table"));

    dlg.run();
  }
}


void MACatalogsPanel::check_table()
{
  struct {
    char *name;
    int mask;
  } options[]= {
    {"quick_check", MYX_CHECK_QUICK},
    {"fast_check", MYX_CHECK_FAST},
    {"medium_check", MYX_CHECK_MEDIUM},
    {"extended_check", MYX_CHECK_EXTENDED},
    {"changed_check", MYX_CHECK_CHANGED},
    {NULL, 0}
  };
  
  Glib::ustring objects= get_selected_tables();

  if (objects.empty())
  {
    Gtk::MessageDialog dlg(*_app->window(), 
                           _("Please select the tables you wish to check."),
                           Gtk::MESSAGE_ERROR,
                           Gtk::BUTTONS_OK,
                           true);
    dlg.set_title(_("Check Table"));

    dlg.run();
    return;
  }

  int type_mask= 0;
  Gtk::Dialog *dlg= (Gtk::Dialog*)_check_dlg_xml->get_widget("check_dialog");
  
  if (dlg->run() != Gtk::RESPONSE_OK)
  {
    dlg->hide();
    return;
  }
  dlg->hide();

  for (int i= 0; options[i].name; i++)
  {
    if (((Gtk::ToggleButton*)_check_dlg_xml->get_widget(options[i].name))->get_active())
      type_mask |= options[i].mask;
  }


  MYX_TABLE_COMMAND_STATUSES *status;

  status= (MYX_TABLE_COMMAND_STATUSES*)
    _data->get_instance()->perform_data_fetch3((MInstanceInfo::DataFetcher3)myx_check_table,
                                               (void*)objects.c_str(), (void*)type_mask,
                                               _("Checking table(s)..."));
  if (!status)
  {
    _data->show_last_error(_("Error executing CHECK TABLE."));
    return;
  }
  
  Glib::ustring msg;
  char *prev_table="";
  
  for (unsigned int i= 0; i < status->status_num; i++)
  {
    msg += "\n";
    
    if (strcmp(prev_table, status->status[i].table)!=0)
    {
      msg += ufmt(_("Table: %s\n"), status->status[i].table);
      prev_table= status->status[i].table;
    }
    switch (status->status[i].message_type)
    {
    case MYX_MESSAGE_STATUS:
      msg += ufmt(_("\tStatus: %s"), status->status[i].message);
      break;
    case MYX_MESSAGE_ERROR:
      msg += ufmt(_("\tERROR: %s"), status->status[i].message);
      break;
    case MYX_MESSAGE_WARNING:
      msg += ufmt(_("\tWARNING: %s"), status->status[i].message);
      break;
    case MYX_MESSAGE_INFO:
      msg += ufmt(_("\tInfo: %s"), status->status[i].message);
      break;
    }
  }
  myx_free_command_status(status);

  {
    Gtk::MessageDialog dlg(*_app->window(), 
                           _("Table check finished. Return status is:\n")+msg,
                           Gtk::MESSAGE_INFO,
                           Gtk::BUTTONS_OK,
                           true, true);
    dlg.set_title(_("Check Table"));

    dlg.run();
  }
}


void MACatalogsPanel::repair_table()
{
  struct {
    char *name;
    int mask;
  } options[]= {
    {"quick_check", MYX_CHECK_QUICK},
    {"extended_check", MYX_CHECK_EXTENDED},
    {"use_frm_check", MYX_REPAIR_USE_FRM},
    {NULL, 0}
  };
  
  Glib::ustring objects= get_selected_tables();
  
  if (objects.empty())
  {
    Gtk::MessageDialog dlg(*_app->window(), 
                           _("Please select the tables you wish to repair."),
                           Gtk::MESSAGE_ERROR,
                           Gtk::BUTTONS_OK,
                           true);
    dlg.set_title(_("Repair Table"));

    dlg.run();
    return;
  }

  int type_mask= 0;
  Gtk::Dialog *dlg= (Gtk::Dialog*)_repair_dlg_xml->get_widget("repair_dialog");

  ((Gtk::CheckButton*)_repair_dlg_xml->get_widget("binlog_check"))->set_active(true);
  
  if (dlg->run() != Gtk::RESPONSE_OK)
  {
    dlg->hide();
    return;
  }
  dlg->hide();

  for (int i= 0; options[i].name; i++)
  {
    if (((Gtk::ToggleButton*)_repair_dlg_xml->get_widget(options[i].name))->get_active())
      type_mask |= options[i].mask;
  }

  if (((Gtk::ToggleButton*)_repair_dlg_xml->get_widget("binlog_check"))->get_active())
    type_mask &= ~MYX_REPAIR_NO_WRITE_TO_BINLOG;
  else
    type_mask |= MYX_REPAIR_NO_WRITE_TO_BINLOG;

  MYX_TABLE_COMMAND_STATUSES *status;

  status= (MYX_TABLE_COMMAND_STATUSES*)
    _data->get_instance()->perform_data_fetch3((MInstanceInfo::DataFetcher3)myx_repair_table,
                                               (void*)objects.c_str(), (void*)type_mask,
                                               _("Repairing table(s)..."));
  if (!status)
  {
    _data->show_last_error(_("Error executing REPAIR TABLE."));
    return;
  }
  
  Glib::ustring msg;
  char *prev_table="";
  
  for (unsigned int i= 0; i < status->status_num; i++)
  {
    msg += "\n";
    
    if (strcmp(prev_table, status->status[i].table)!=0)
    {
      msg += ufmt(_("Table: %s\n"), status->status[i].table);
      prev_table= status->status[i].table;
    }
    switch (status->status[i].message_type)
    {
    case MYX_MESSAGE_STATUS:
      msg += ufmt(_("\tStatus: %s"), status->status[i].message);
      break;
    case MYX_MESSAGE_ERROR:
      msg += ufmt(_("\tERROR: %s"), status->status[i].message);
      break;
    case MYX_MESSAGE_WARNING:
      msg += ufmt(_("\tWARNING: %s"), status->status[i].message);
      break;
    case MYX_MESSAGE_INFO:
      msg += ufmt(_("\tInfo: %s"), status->status[i].message);
      break;
    }
  }
  myx_free_command_status(status);

  {
    Gtk::MessageDialog dlg(*_app->window(), 
                           _("Table reparation finished. Return status is:\n")+msg,
                           Gtk::MESSAGE_INFO,
                           Gtk::BUTTONS_OK,
                           true, true);
    dlg.set_title(_("Repair Table"));

    dlg.run();
  }
}


MAPanel *create_catalogs_panel(MAdministrator *app, MDataInterface *data)
{
  return new MACatalogsPanel(app, data);
}
