/* Copyright (C) 2003 MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */


#ifndef _MACATALOGSPANEL_H_
#define _MACATALOGSPANEL_H_

#include "MAPanel.h"
#include <MGSchemaBrowserList.h>
#include <myx_admin_public_interface.h>

class MDataInterface;

class MACatalogsPanel : public MAPanel {
    class TableColumns : public Gtk::TreeModel::ColumnRecord {
      public:
        TableColumns() {
          add(_icon); add(_name); add(_type);
          add(_format); add(_rows); add(_length); add(_time); add(_status);
          add(_table);
        };
        Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > _icon;
        Gtk::TreeModelColumn<Glib::ustring> _name;
        Gtk::TreeModelColumn<Glib::ustring> _type;
        Gtk::TreeModelColumn<Glib::ustring> _format;
        Gtk::TreeModelColumn<Glib::ustring> _rows;
        Gtk::TreeModelColumn<Glib::ustring> _length;
        Gtk::TreeModelColumn<Glib::ustring> _time;
        Gtk::TreeModelColumn<MYX_TABLE_STATUS*> _status;
        Gtk::TreeModelColumn<MYX_SCHEMA_TABLE*> _table;
    } _table_columns;

    class ColumnColumns : public Gtk::TreeModel::ColumnRecord {
      public://XXX
        ColumnColumns() {
          add(_icon); add(_name); add(_key); add(_type);
          add(_null); add(_default); add(_extra);
        };
        Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > _icon;
        Gtk::TreeModelColumn<Glib::ustring> _name;
        Gtk::TreeModelColumn<Glib::ustring> _key;
        Gtk::TreeModelColumn<Glib::ustring> _type;
        Gtk::TreeModelColumn<Glib::ustring> _default;
        Gtk::TreeModelColumn<Glib::ustring> _null;
        Gtk::TreeModelColumn<Glib::ustring> _extra;
    } _column_columns;

    class IndexColumns : public Gtk::TreeModel::ColumnRecord {
      public:
        IndexColumns() {
          add(_icon); add(_name); add(_key); add(_type); 
          add(_unique); add(_null); add(_seq); add(_coll);
        };
        Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > _icon;
        Gtk::TreeModelColumn<Glib::ustring> _name;
        Gtk::TreeModelColumn<Glib::ustring> _key;
        Gtk::TreeModelColumn<Glib::ustring> _type;
        Gtk::TreeModelColumn<Glib::ustring> _unique;
        Gtk::TreeModelColumn<Glib::ustring> _null;
        Gtk::TreeModelColumn<Glib::ustring> _seq;
        Gtk::TreeModelColumn<Glib::ustring> _coll;
    } _index_columns;

    class ViewColumns : public Gtk::TreeModel::ColumnRecord {
      public:
        ViewColumns() {
          add(_icon); add(_name); add(_type); add(_tables); add(_time);
        };
        Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > _icon;
        Gtk::TreeModelColumn<Glib::ustring> _name;
        Gtk::TreeModelColumn<Glib::ustring> _type;
        Gtk::TreeModelColumn<Glib::ustring> _tables;
        Gtk::TreeModelColumn<Glib::ustring> _time;
    } _view_columns;

    class UserColumns : public Gtk::TreeModel::ColumnRecord {
      public:
        UserColumns() {
          add(_icon); add(_name); add(_fullname); add(_privileges);
        };
        Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > _icon;
        Gtk::TreeModelColumn<Glib::ustring> _name;
        Gtk::TreeModelColumn<Glib::ustring> _fullname;
        Gtk::TreeModelColumn<Glib::ustring> _privileges;
    } _user_columns;

    class DocColumns : public Gtk::TreeModel::ColumnRecord {
      public:
        DocColumns() {
          add(_icon); add(_name); add(_type); add(_version); add(_user);
          add(_checked_out); add(_description);
        };
        Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > _icon;
        Gtk::TreeModelColumn<Glib::ustring> _name;
        Gtk::TreeModelColumn<Glib::ustring> _type;
        Gtk::TreeModelColumn<Glib::ustring> _version;
        Gtk::TreeModelColumn<Glib::ustring> _user;
        Gtk::TreeModelColumn<Glib::ustring> _checked_out;
        Gtk::TreeModelColumn<Glib::ustring> _description;
    } _doc_columns;

    class EventColumns : public Gtk::TreeModel::ColumnRecord {
      public:
        EventColumns() {
          add(_icon); add(_name); add(_type); add(_ctime); add(_utime);
        };
        Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > _icon;
        Gtk::TreeModelColumn<Glib::ustring> _name;
        Gtk::TreeModelColumn<Glib::ustring> _type;
        Gtk::TreeModelColumn<Glib::ustring> _ctime;
        Gtk::TreeModelColumn<Glib::ustring> _utime;
    } _event_columns;

    class ProcedureColumns : public Gtk::TreeModel::ColumnRecord {
      public:
        ProcedureColumns() {
          add(_icon); add(_name); add(_parameter); add(_ctime); add(_utime);
        };
        Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > _icon;
        Gtk::TreeModelColumn<Glib::ustring> _name;
        Gtk::TreeModelColumn<Glib::ustring> _parameter;
        Gtk::TreeModelColumn<Glib::ustring> _ctime;
        Gtk::TreeModelColumn<Glib::ustring> _utime;
    } _procedure_columns;

    MGGladeXML *_check_dlg_xml;
    MGGladeXML *_repair_dlg_xml;

    MGSchemaBrowserList *_schema_browser;

    MYX_SCHEMA_TABLES *_tables;
    MYX_SCHEMA_TABLE_STATUS *_table_status;
    
    Glib::RefPtr<Gdk::Pixbuf> _schema_icon;
    Glib::RefPtr<Gdk::Pixbuf> _sys_schema_icon;
    Glib::RefPtr<Gdk::Pixbuf> _table_icon;
    Glib::RefPtr<Gdk::Pixbuf> _column_icon;
    Glib::RefPtr<Gdk::Pixbuf> _key_column_icon;

    Glib::RefPtr<Gdk::Pixbuf> _index_icon;
    Glib::RefPtr<Gdk::Pixbuf> _doc_icon;


    Glib::RefPtr<Gtk::ListStore> _table_store;
    Glib::RefPtr<Gtk::TreeStore> _column_store;
    Glib::RefPtr<Gtk::TreeStore> _tindex_store;
    Glib::RefPtr<Gtk::ListStore> _view_store;
    Glib::RefPtr<Gtk::ListStore> _event_store;
    Glib::RefPtr<Gtk::TreeStore> _index_store;
    Glib::RefPtr<Gtk::ListStore> _user_store;
    Glib::RefPtr<Gtk::TreeStore> _doc_store;
    Glib::RefPtr<Gtk::ListStore> _procedure_store;
    
    void setup_trees();
    
    void refresh_table_info(const Glib::ustring &schema);
    void refresh_event_info(const Glib::ustring &schema);
    void refresh_view_info(const Glib::ustring &schema);
    void refresh_index_info(const Glib::ustring &schema);
    void refresh_procedure_info(const Glib::ustring &schema);
    void refresh_doc_info(const Glib::ustring &schema);
    void refresh_user_info(const Glib::ustring &schema);

    void selected_table();

    
    Glib::ustring get_selected_tables();

    void optimize_table();
    void check_table();
    void repair_table();
    
    void schema_selected(MGBrowserList *sender,
                         const Gtk::TreeIter &node);

  public:
    MACatalogsPanel(MAdministrator *app, MDataInterface *info);
    ~MACatalogsPanel();

    virtual void show();
    virtual bool before_show();
    virtual bool before_hide();

    virtual bool init();

    virtual bool is_local_only() { return false; };
    virtual bool needs_connection() { return true; };
};

extern MAPanel *create_catalogs_panel(MAdministrator *app, MDataInterface *info);

#endif /* _MACATALOGSPANEL_H_ */
