/* Copyright (C) 2003, 2004 MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

   
#include "myadmin.h"
#include "MAdministrator.h"
#include "MAStartupParametersPanel.h"
#include "MDataInterface.h"
#include "myg_utils.h"



MAStartupParametersPanel::MAStartupParametersPanel(MAdministrator *app, MDataInterface *data)
    : MAPanel(app, data), _file_dlg(0), _dint(0), _my_cnf_missing_error_displayed(false)
{
}


MAStartupParametersPanel::~MAStartupParametersPanel()
{
  delete _dint;
}


bool MAStartupParametersPanel::init()
{
  if (_xml)
    return true;

  if (!MAPanel::init_from_glade(GLADE_DATADIR"/"GLADE_STARTUPPARAMETERS_FILE, "panel_frame"))
    return false;

  get_button("select_button")->signal_clicked().connect(SigC::slot(*this,&MAStartupParametersPanel::select_file));
  get_button("discard_button")->signal_clicked().connect(SigC::slot(*this,&MAStartupParametersPanel::revert_changes));
  get_button("apply_button")->signal_clicked().connect(SigC::slot(*this,&MAStartupParametersPanel::apply_changes));
  
  return true;
}


void MAStartupParametersPanel::show_configuration(const std::string &file, const std::string &section)
{
  _dint= new MDynamicInterface((Gtk::Notebook*)get_widget("notebook"),
                               _app->load_pixbuf("24x24_ServerStartupParams.png"),
                               _data->get_instance());
  
  if (_dint->open(APP_DATADIR"/mysqlx_startup_variables_description.xml", 4.0, 
                  file, section) == MYX_ADMIN_NO_ERROR)
  {
    get_label("path_label")->set_text(ufmt(_("Editing file '%s'\nSection '%s'"),
                                           file.c_str(), section.c_str()));
  }
}


void MAStartupParametersPanel::select_file()
{
  Gtk::Dialog *dlg;
  
  if (!_file_dlg)
  {
    _file_dlg= new MGGladeXML(GLADE_DATADIR"/"GLADE_STARTUPPARAMETERS_FILE, "select_dialog");
  }

  dlg= (Gtk::Dialog*)_file_dlg->get_widget("select_dialog");
  
  dlg->run();
}


void MAStartupParametersPanel::revert_changes()
{
  _app->push_status(_("Reverting changes..."));
  _dint->revert_values();
  _app->pop_status();
}


void MAStartupParametersPanel::apply_changes()
{
  int res;

  _app->push_status(_("Saving configurations..."));
  res= _dint->save();
  _app->pop_status();

  if (res==0)
  {
    _app->set_status(_("Configuration saved."));
  }
  else if (res == -1)
  {
    show_error(*_app->window(), _("Could not save configurations."));
  }
  else
  {
    show_error(*_app->window(), _("Error updating configuration."));
  }
}


void MAStartupParametersPanel::show()
{
  MAPanel::show();

  if (!_dint)
  {
    if (!myx_get_my_cnf_path())
    {
      if (!_my_cnf_missing_error_displayed)
      {
        show_error(*_app->window(), 
                   _("The my.cnf configuration file could not be found."));
        _my_cnf_missing_error_displayed= true;
      }
    }
    else
    {
      _app->push_status(_("Processing configuration file..."));
      show_configuration(myx_get_my_cnf_path(), "mysqld");
      _app->pop_status();
    }
  }
}


bool MAStartupParametersPanel::before_show()
{
  return true;
}


bool MAStartupParametersPanel::before_hide()
{
  return true;
}



MAPanel *create_startup_parameters_panel(MAdministrator *app, MDataInterface *data)
{
  return new MAStartupParametersPanel(app, data);
}

