/* Copyright (C) 2004 MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */



#ifndef _MINSTANCEINFO_H_
#define _MINSTANCEINFO_H_

#include <glibmm.h>

#include <pthread.h>
#include <mysql.h>
#include "MDispatcher.h"

#include <list>

#include "MYXInterface.h"
//#include <wmyx_public_interface.h>
#include <myx_public_interface.h>


class MAdministrator;

class MInstanceInfo : public Glib::ObjectBase {
 public:
    typedef void *(*DataFetcher3)(MYSQL *,void*,void*);
    typedef void *(*DataFetcher2)(MYSQL *,void*);
    typedef void *(*DataFetcher)(MYSQL *);
    typedef bool (*AsyncCallback)(void *,void*);

    enum ServerState {
      SStopped,
      SRunning
    };
    
    
    struct FilesystemInfo {
      std::string fs;
      std::string path;
      std::string type;
      double size;
      double free;
      bool ro;
    };
    
    enum ServerPath {
      PInnoDBLog,
      PErrorLog,
      PLogs
    };
    
 private:
    MDispatcher _dispatcher;

    SigC::Signal0<void> _disconnect_signal;

    pthread_mutex_t _mysql_mx;
    MYSQL *_mysql;
    MYX::UserConnection _user_conn;
    std::map<std::string,MYSQL*> _extra_mysql;
    
    MAdministrator *_main_win;

    bool _socket_connection;
    
    bool process_service_output(Glib::IOCondition cond, FILE *file,
                                Glib::RefPtr<Gtk::TextBuffer> text);
 public:
    MInstanceInfo(MAdministrator *main_win,
                  MYSQL *mysql, const MYX::UserConnection &user_conn);
    ~MInstanceInfo();
    void disconnect();
    
    void set_connection(MYSQL *mysql, const MYX::UserConnection &user_conn);

    void perform_async_data_fetch3(const char *name, 
                                   DataFetcher3 fetcher, void *arg1, void *arg2,
                                   AsyncCallback callback, void *data,
                                   long delay);
    void perform_async_data_fetch2(const char *name,
                                   DataFetcher2 fetcher, void *arg,
                                   AsyncCallback callback, void *data,
                                   long delay);
    void perform_async_data_fetch(const char *name,
                                  DataFetcher fetcher,
                                  AsyncCallback callback, void *data,
                                  long delay);

    void cancel_async_data_fetch(const char *name);

    void *perform_data_fetch3(DataFetcher3 fetcher, void *arg1, void *arg2,
                              const Glib::ustring &msg=Glib::ustring());
    void *perform_data_fetch2(DataFetcher2 fetcher, void *arg,
                              const Glib::ustring &msg=Glib::ustring());
    void *perform_data_fetch(DataFetcher fetcher,
                             const Glib::ustring &msg=Glib::ustring());

    void cancel_data_fetch();

    //--------------------------------------------------

    MYX_MACHINE_INFO *get_client_info();
    MYX_MACHINE_INFO *get_server_info();

    bool is_local();
    bool is_connected();
    bool is_socket_connection() { return _socket_connection; };

    void start_stop_service(bool stop, const Glib::RefPtr<Gtk::TextBuffer> &text);

    void lock_mysql(bool flag);
    MYSQL *get_mysql() const { return _mysql; };
    MYSQL *clone_mysql();
    MYX::UserConnection get_connection_data() const { return _user_conn; };
    
    bool get_fs_info(std::list<FilesystemInfo> &fslist);
    
    std::string get_server_path(ServerPath type);
    
    ServerState get_server_state();
    
    
    SigC::Signal0<void> signal_disconnect() { return _disconnect_signal; };
};


#endif /* _MINSTANCEINFO_H_ */


