unit AdminServerHealth;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, ExtCtrls, StrUtils, Contnrs, Clipbrd,
  Series, InstanceSections,
  ApplicationDataModule, AuxFuncs, Translations,
  myx_admin_public_interface, myx_public_interface,
  AdditionalClasses,
  AdminServerHealthLineGraph,
  AdminServerHealthBarGraph, ImgList, Menus,
  PNGImage, InstanceDataModule,
  Options, MyxError,
  AdminServerHealthGraphSettings;

type
  TVarListId = class (TObject)
    mysql_id: string;

    constructor Create(mysql_id: string);
  end;

  TGraphFrameBundle = class (TObject)
    Graph: TMYX_HEALTH_GRAPH;
    GraphFrame: TObject;
    value_formular: string;
    max_formular: string;

    constructor Create(Graph: TMYX_HEALTH_GRAPH;
      GraphFrame: TObject);
  end;

  TAdminServerHealthForm = class(TInstanceSectionForm)
    ServerStatusPnl: TPanel;
    HealthPageControl: TPageControl;
    ServerStatusTabSheet: TTabSheet;
    ServerVariablesTabSheet: TTabSheet;
    ServerStatusMainPnl: TPanel;
    ServerStatusTreeView: TTreeView;
    StatusVarSplitter: TSplitter;
    ServerStatusListView: TListView;
    ServerVariablesMainPnl: TPanel;
    ServerVarSplitter: TSplitter;
    ServerVariablesTreeView: TTreeView;
    VariablesListView: TListView;
    GraphRefreshTmr: TTimer;
    VarImageList: TImageList;
    VariablesPopupMenu: TPopupMenu;
    CopyVariablestoClipboardMI: TMenuItem;
    CopyselectedVariableMI: TMenuItem;
    HeaderPnl: TPanel;
    MemoryHealthBevel: TBevel;
    HeaderDescriptionLbl: TLabel;
    HeaderImg: TImage;
    HeaderLbl: TLabel;
    BottomPnl: TPanel;
    RefreshBtn: TButton;
    Panel1: TPanel;
    Button1: TButton;
    CopyselectedVariableNameMI: TMenuItem;
    PageControlPopupMenu: TPopupMenu;
    AddaPageMI: TMenuItem;
    AddaGroupMI: TMenuItem;
    DeletePageMI: TMenuItem;
    N1: TMenuItem;
    GroupBoxPopupMenu: TPopupMenu;
    AddaGraphMI: TMenuItem;
    DeleteGroupMI: TMenuItem;
    N2: TMenuItem;
    GraphPopupMenu: TPopupMenu;
    EditGraphMI: TMenuItem;
    N3: TMenuItem;
    DeleteGraph2: TMenuItem;
    AddaGraph2MI: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ServerStatusPnlResize(Sender: TObject);
    procedure HealthPageControlChange(Sender: TObject);
    procedure RefreshBtnClick(Sender: TObject);
    procedure BuildVariablesTreeViews;
    procedure RefreshVars(Treeview: TTreeView; var CurrentVarGroup: Pointer;
      ListView: TListView; Vars: TMYX_VARIABLES;
      VarListing: TMYX_VARIABLES_LISTING; NotListedVars: TStringList);

    procedure FetchStatusVars(Sender: TObject);
    procedure FetchedStatusVars(Sender: TObject);
    procedure RefreshStatusVars;
    procedure FetchServerVars(Sender: TObject);
    procedure FetchedServerVars(Sender: TObject);
    procedure RefreshServerVars;
    procedure ServerStatusTreeViewChange(Sender: TObject; Node: TTreeNode);
    procedure ServerVariablesTreeViewChange(Sender: TObject;
      Node: TTreeNode);

    procedure FetchStatusVarsTimer(Sender: TObject);
    procedure FetchedStatusVarsTimer(Sender: TObject);
    procedure GraphRefreshTmrTimer(Sender: TObject);
    function GetVarIndex(vars: TMYX_VARIABLES;
      VarName: string): integer;
    procedure PrepareGraphFormulars;

    procedure LoadGraphs;
    procedure BuildGraphs;
    procedure AddPage(page: TMYX_HEALTH_PAGE;
      var PageTabSheet: TTabSheet; var ScrollBox: TScrollBox);
    procedure AddGraphToGroupBox(GroupBox: TGroupBox;
      graph: TMYX_HEALTH_GRAPH; var current_widget_ypos: integer);

    procedure CopyVariablestoClipboardMIClick(Sender: TObject);
    procedure VariablesListViewDblClick(Sender: TObject);
    procedure CopyselectedVariableMIClick(Sender: TObject);
    procedure CopyselectedVariableNameMIClick(Sender: TObject);

    procedure Disconnected(var Message: TMessage); message WM_Disconnected;
    procedure Reconnected(var Message: TMessage); message WM_Reconnected;

    procedure AddaPageMIClick(Sender: TObject);
    procedure DeletePageMIClick(Sender: TObject);
    procedure AddaGroupMIClick(Sender: TObject);
    procedure EditGraphMIClick(Sender: TObject);
    procedure ServerStatusListViewMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure AddaGraphMIClick(Sender: TObject);

    procedure OptionsChanged(var Message: TMessage); message WM_OptionsChanged;

  private
    { Private declarations }
    server_vars,
    status_vars,
    status_vars_timer,
    status_vars_timer_old: TMYX_VARIABLES;

    StatusVarListing,
    ServerVarListing: TMYX_VARIABLES_LISTING;

    NotListedStatusVars,
    NotListedServerVars: TStringList;

    CurrentStatusVarGroup,
    CurrentServerVarGroup: Pointer;

    StatusVar_PMySQL: Pointer;

    HealthPages: TMYX_HEALTH_PAGES;
    GraphFrameBundleList: TObjectList;

    ClosingApplication: Boolean;

    HeaderPNGImg: TPNGObject;

    AdminServerHealthGraphSettingsForm: TAdminServerHealthGraphSettingsForm;
  public
    RefreshSystemVarsInFormulas: Boolean;
  end;

const
  GroupBoxWidth = 535;
  GroupBoxVSpace = 10;

var
  AdminServerHealthForm: TAdminServerHealthForm;

implementation

{$R *.dfm}

constructor TVarListId.Create(mysql_id: string);
begin
  self.mysql_id:=mysql_id;
end;

constructor TGraphFrameBundle.Create(Graph: TMYX_HEALTH_GRAPH;
  GraphFrame: TObject);
begin
  self.Graph:=Graph;
  self.GraphFrame:=GraphFrame;

  self.value_formular:=Graph.value_formula;
  self.max_formular:=Graph.max_formula;
end;

procedure TAdminServerHealthForm.FormCreate(Sender: TObject);
begin
  DockedPanel:=ServerStatusPnl;

  server_vars:=nil;
  status_vars:=nil;
  status_vars_timer:=nil;
  status_vars_timer_old:=nil;

  StatusVarListing:=nil;
  ServerVarListing:=nil;

  CurrentStatusVarGroup:=nil;
  CurrentServerVarGroup:=nil;

  NotListedStatusVars:=nil;
  NotListedServerVars:=nil;

  StatusVar_PMySQL:=nil;

  AdminServerHealthGraphSettingsForm:=nil;

  ClosingApplication:=False;

  HealthPages:=nil;

  RefreshSystemVarsInFormulas:=False;

  HeaderPNGImg:=LoadPNGImageFromResource('health', HeaderImg);

  BuildVariablesTreeViews;

  ServerStatusTreeView.FullExpand;
  ServerVariablesTreeView.FullExpand;

  if(MYXCommonOptions.XPStyleEnabled)then
  begin
    StatusVarSplitter.Color:=clWhite;
    ServerVarSplitter.Color:=clWhite;
  end;

  GraphFrameBundleList:=TObjectList.Create;

  //Paint ProgressGraphs for the first time
  //ServerStatusPnlResize(self);

  if(InstanceDM.Connected)then
  begin
    BuildGraphs;

    GraphRefreshTmr.Enabled:=True;
  end
  else
  begin
    DisableEnableControls(ServerStatusTabSheet, False);
    DisableEnableControls(ServerVariablesTabSheet, False);
  end;

  HealthPageControl.ActivePageIndex:=0;
  HealthPageControlChange(self);
end;

procedure TAdminServerHealthForm.FormDestroy(Sender: TObject);
begin
  GraphFrameBundleList.Free;

  if(server_vars<>nil)then
    server_vars.Free;
  if(status_vars_timer<>nil)then
  begin
    status_vars_timer.Free;
    status_vars_timer:=nil;
  end;
  if(status_vars_timer_old<>nil)then
  begin
    status_vars_timer_old.Free;
    status_vars_timer_old:=nil;
  end;

  if(StatusVarListing<>nil)then
    StatusVarListing.Free;
  if(ServerVarListing<>nil)then
    ServerVarListing.Free;

  if(NotListedStatusVars<>nil)then
    NotListedStatusVars.Free;
  if(NotListedServerVars<>nil)then
    NotListedServerVars.Free;

  if(StatusVar_PMySQL<>nil)then
    FreeMem(StatusVar_PMySQL);

  if(HealthPages<>nil)then
    HealthPages.Free;
end;

procedure TAdminServerHealthForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  ClosingApplication:=True;
end;

procedure TAdminServerHealthForm.ServerStatusPnlResize(
  Sender: TObject);
var i: integer;
begin
  for i:=0 to GraphFrameBundleList.Count-1 do
    if(TGraphFrameBundle(GraphFrameBundleList[i]).GraphFrame is THealthLineGraphFrame)then
      THealthLineGraphFrame(TGraphFrameBundle(GraphFrameBundleList[i]).GraphFrame).FrameResize(self);
end;

procedure TAdminServerHealthForm.HealthPageControlChange(
  Sender: TObject);
begin
  HeaderPnl.Parent:=HealthPageControl.ActivePage;
  HeaderPnl.Align:=alTop;
  HeaderPnl.Visible:=True;

  if(HealthPageControl.ActivePage=ServerStatusTabSheet)then
  begin
    HeaderLbl.Caption:='Status Variables';
    HeaderDescriptionLbl.Caption:='Current status variables of the server.';
  end
  else if(HealthPageControl.ActivePage=ServerVariablesTabSheet)then
  begin
    HeaderLbl.Caption:='System Variables';
    HeaderDescriptionLbl.Caption:='Overview of the system variables.';
  end
  else
  begin
    if(HealthPageControl.ActivePageIndex<=HealthPages.pages.Count)then
    begin
      HeaderLbl.Caption:=HealthPages.pages[HealthPageControl.ActivePageIndex].caption;
      HeaderDescriptionLbl.Caption:=HealthPages.pages[HealthPageControl.ActivePageIndex].description;
    end;
  end;

  ServerStatusPnlResize(self);
end;

procedure TAdminServerHealthForm.RefreshBtnClick(Sender: TObject);
begin
  if(HealthPageControl.ActivePage=ServerStatusTabSheet)then
  begin
    if(status_vars<>nil)then
      status_vars.Free;
    status_vars:=nil;

    RefreshStatusVars;
  end
  else if(HealthPageControl.ActivePage=ServerVariablesTabSheet)then
  begin
    if(server_vars<>nil)then
      server_vars.Free;
    server_vars:=nil;

    RefreshServerVars;
  end;
end;

procedure TAdminServerHealthForm.BuildVariablesTreeViews;
var PVarListing: PMYX_VARIABLES_LISTING;
  error: MYX_ADMIN_LIB_ERROR;
  i, j: integer;
  GroupNode: TTreeNode;
begin
  PVarListing:=myx_get_variables_listing(
    PChar(AnsiToUtf8(MYXCommonOptions.XMLDir+'mysqlx_status_variables.xml')), @error);
  if(error<>MYX_ADMIN_NO_ERROR)then
    raise EMyxError.Create('The status variables definition could '+
      'not be loaded from the XML file.'#13#10+
      'Error Nr.'+IntToStr(Ord(error)));
  try
    StatusVarListing:=TMYX_VARIABLES_LISTING.create(PVarListing);
  finally
    myx_free_variables_listing(PVarListing);
  end;

  ServerStatusTreeView.Items.BeginUpdate;
  try
    ServerStatusTreeView.Items.Clear;

    for i:=0 to StatusVarListing.groups.Count-1 do
    begin
      GroupNode:=AddTreeViewChildNode(ServerStatusTreeView, nil,
        StatusVarListing.groups[i].name, 4, StatusVarListing.groups[i]);

      for j:=0 to StatusVarListing.groups[i].subgroups.Count-1 do
      begin
        AddTreeViewChildNode(ServerStatusTreeView, GroupNode,
          StatusVarListing.groups[i].subgroups[j].name, 4,
          StatusVarListing.groups[i].subgroups[j]);
      end;
    end;
  finally
    ServerStatusTreeView.Items.EndUpdate;
  end;


  PVarListing:=myx_get_variables_listing(
   PChar(AnsiToUtf8(MYXCommonOptions.XMLDir+'mysqlx_system_variables.xml')), @error);
  if(error<>MYX_ADMIN_NO_ERROR)then
    raise EMyxError.Create('The status variables definition could '+
      'not be loaded from the XML file.'#13#10+
      'Error Nr.'+IntToStr(Ord(error)));
  try
    ServerVarListing:=TMYX_VARIABLES_LISTING.create(PVarListing);
  finally
    myx_free_variables_listing(PVarListing);
  end;

  ServerVariablesTreeView.Items.BeginUpdate;
  try
    ServerVariablesTreeView.Items.Clear;

    for i:=0 to ServerVarListing.groups.Count-1 do
    begin
      GroupNode:=AddTreeViewChildNode(ServerVariablesTreeView, nil,
        ServerVarListing.groups[i].name, 4, ServerVarListing.groups[i]);

      for j:=0 to ServerVarListing.groups[i].subgroups.Count-1 do
      begin
        AddTreeViewChildNode(ServerVariablesTreeView, GroupNode,
          ServerVarListing.groups[i].subgroups[j].name, 4,
          ServerVarListing.groups[i].subgroups[j]);
      end;
    end;
  finally
    ServerVariablesTreeView.Items.EndUpdate;
  end;
end;

procedure TAdminServerHealthForm.RefreshVars(Treeview: TTreeView;
  var CurrentVarGroup: Pointer; ListView: TListView; Vars: TMYX_VARIABLES;
  VarListing: TMYX_VARIABLES_LISTING; NotListedVars: TStringList);
var i, j, k, l: integer;
  Item: TListItem;
  found: Boolean;
  VarGrp: TMYX_VARIABLES_GROUP;
begin
  if(VarListing=nil)then
    Exit;

  //get all vars that are not listed
  if(NotListedVars=nil)then
  begin
    NotListedVars:=TStringList.Create;

    for k:=0 to Vars.variables.Count-1 do
    begin
      found:=False;

      for i:=0 to VarListing.groups.Count-1 do
      begin
        for j:=0 to VarListing.groups[i].variables.Count-1 do
        begin
          if(CompareText(VarListing.groups[i].variables[j].mysql_id,
            Vars.variables[k].name)=0)then
          begin
            found:=True;
            break;
          end;
        end;

        if(Not(found))then
        begin
          for j:=0 to VarListing.groups[i].subgroups.Count-1 do
          begin
            for l:=0 to VarListing.groups[i].subgroups[j].variables.Count-1 do
              if(CompareText(VarListing.groups[i].subgroups[j].variables[l].mysql_id,
                Vars.variables[k].name)=0)then
              begin
                found:=True;
                break;
              end;

            if(found)then
              break;
          end;
        end;

        if(found)then
          break;
      end;

      if(Not(found))then
        NotListedVars.Add(Vars.variables[k].name);
    end;

    if(NotListedVars.Text<>'')then
    begin
      VarGrp:=TMYX_VARIABLES_GROUP.create(VarListing.groups.Count,
        'New Variables', '');
      VarListing.groups.Add(VarGrp);

      for i:=0 to NotListedVars.Count-1 do
        VarGrp.variables.Add(TMYX_VARIABLE_ELEMENT.create(
          NotListedVars[i], 'New Variable', 0));

      AddTreeViewChildNode(Treeview, nil,
        'New Variables', 4, VarGrp);
    end;
  end;


  ListView.Items.BeginUpdate;
  try
    if(Treeview.Selected<>nil)then
      if(Treeview.Selected.Data<>nil)then
      begin
        if(CurrentVarGroup<>Treeview.Selected.Data)then
        begin
          if(TObject(Treeview.Selected.Data) is TMYX_VARIABLES_GROUP)then
          begin
            //Current Group has changed, so clear and build ListView
            for i:=0 to ListView.Items.Count-1 do
              if(ListView.Items[i].Data<>nil)then
                if(TObject(ListView.Items[i].Data) is TVarListId)then
                  TVarListId(ListView.Items[i].Data).Free;
            ListView.Items.Clear;

            for i:=0 to Vars.variables.Count-1 do
              for j:=0 to TMYX_VARIABLES_GROUP(Treeview.Selected.Data).variables.Count-1 do
                if(Vars.variables[i].name=
                  TMYX_VARIABLES_GROUP(Treeview.Selected.Data).variables[j].mysql_id)then
                begin
                  Item:=AddListViewItem(ListView, nil,
                    Vars.variables[i].name,
                    TMYX_VARIABLES_GROUP(Treeview.Selected.Data).variables[j].editable,
                    TVarListId.Create(Vars.variables[i].name));

                  Item.SubItems.Add(Vars.variables[i].value);
                  Item.SubItems.Add(_T('MySQLVariables',
                    TMYX_VARIABLES_GROUP(Treeview.Selected.Data).variables[j].desc_id,
                    ''));
                end;

            CurrentVarGroup:=Treeview.Selected.Data;
          end
          else if(TObject(Treeview.Selected.Data) is TMYX_VARIABLES_SUBGROUP)then
          begin
            if(CurrentVarGroup<>Treeview.Selected.Data)then
            begin
              //Current Group has changed, so clear and build ListView
              for i:=0 to ListView.Items.Count-1 do
                if(ListView.Items[i].Data<>nil)then
                  if(TObject(ListView.Items[i].Data) is TVarListId)then
                    TVarListId(ListView.Items[i].Data).Free;
              ListView.Items.Clear;

              for i:=0 to Vars.variables.Count-1 do
                for j:=0 to TMYX_VARIABLES_SUBGROUP(Treeview.Selected.Data).variables.Count-1 do
                  if(Vars.variables[i].name=
                    TMYX_VARIABLES_SUBGROUP(Treeview.Selected.Data).variables[j].mysql_id)then
                  begin
                    Item:=AddListViewItem(ListView, nil,
                      Vars.variables[i].name, TMYX_VARIABLES_SUBGROUP(Treeview.Selected.Data).variables[j].editable,
                      TVarListId.Create(Vars.variables[i].name));

                    Item.SubItems.Add(Vars.variables[i].value);
                    Item.SubItems.Add(_T('MySQLVariables',
                      TMYX_VARIABLES_SUBGROUP(Treeview.Selected.Data).variables[j].desc_id,
                      ''));
                  end;

              CurrentVarGroup:=Treeview.Selected.Data;
            end;
          end;
        end
        else
        begin
          //Current Group has NOT changed, so simply refresh Values
          for i:=0 to Vars.variables.Count-1 do
            for j:=0 to ListView.Items.Count-1 do
              if(ListView.Items[j].Data<>nil)then
                if(TObject(ListView.Items[j].Data) is TVarListId)then
                  if(Vars.variables[i].name=
                    TVarListId(ListView.Items[j].Data).mysql_id)then
                  begin
                    if(ListView.Items[j].SubItems.Count>0)then
                      ListView.Items[j].SubItems[0]:=Vars.variables[i].value;
                  end;

        end;
      end;
  finally
    ListView.Items.EndUpdate;
  end;
end;

procedure TAdminServerHealthForm.FetchStatusVars(Sender: TObject);
var pvars: PMYX_VARIABLES;
begin
  pvars:=myx_get_status_variables(InstanceDM.PMySQL);
  if(pvars=nil)then
    raise EMyxError.Create('Error while fetching Status Variables.');
  try
    status_vars:=TMYX_VARIABLES.create(pvars);
  finally
    myx_free_variables(pvars);
  end;
end;

procedure TAdminServerHealthForm.FetchedStatusVars(Sender: TObject);
begin
  RefreshStatusVars;
end;

procedure TAdminServerHealthForm.RefreshStatusVars;
begin
  if(Not(InstanceDM.Connected))then
    Exit;

  if(status_vars=nil)then
    InstanceDM.FetchData(KindOfData_StatusVars,
      FetchStatusVars, FetchedStatusVars,
      nil, 'Fetching Status Variables ...')
  else
    RefreshVars(ServerStatusTreeView, CurrentStatusVarGroup,
      ServerStatusListView, status_vars,
      StatusVarListing, NotListedStatusVars);
end;

procedure TAdminServerHealthForm.FetchServerVars(Sender: TObject);
var pvars: PMYX_VARIABLES;
begin
  pvars:=myx_get_server_variables(InstanceDM.PMySQL);
  if(pvars=nil)then
    raise EMyxError.Create('Error while fetching Server Variables.');
  try
    server_vars:=TMYX_VARIABLES.create(pvars);
  finally
    myx_free_variables(pvars);
  end;
end;

procedure TAdminServerHealthForm.FetchedServerVars(Sender: TObject);
begin
  RefreshServerVars;
end;

procedure TAdminServerHealthForm.RefreshServerVars;
begin
  if(Not(InstanceDM.Connected))then
    Exit;

  if(server_vars=nil)then
    InstanceDM.FetchData(KindOfData_ServerVars,
      FetchServerVars, FetchedServerVars,
      nil, 'Fetching System Variables ...')
  else
  begin
    RefreshVars(ServerVariablesTreeView, CurrentServerVarGroup,
      VariablesListView, server_vars,
      ServerVarListing, NotListedServerVars);

    RefreshSystemVarsInFormulas:=True;
  end;
end;

procedure TAdminServerHealthForm.ServerStatusTreeViewChange(
  Sender: TObject; Node: TTreeNode);
begin
  if(status_vars=nil)then
    RefreshStatusVars
  else
    RefreshVars(ServerStatusTreeView,
      CurrentStatusVarGroup, ServerStatusListView, status_vars,
      StatusVarListing, NotListedStatusVars);
end;

procedure TAdminServerHealthForm.ServerVariablesTreeViewChange(
  Sender: TObject; Node: TTreeNode);
begin
  if(server_vars=nil)then
    RefreshServerVars
  else
    RefreshVars(ServerVariablesTreeView,
      CurrentServerVarGroup, VariablesListView, server_vars,
      ServerVarListing, NotListedServerVars);
end;

procedure TAdminServerHealthForm.CopyVariablestoClipboardMIClick(
  Sender: TObject);
var i: integer;
  txt: string;
  ListView: TListView;
begin
  if(HealthPageControl.ActivePage=ServerStatusTabSheet)then
    ListView:=ServerStatusListView
  else
    ListView:=VariablesListView;

  with ListView do
  begin
    txt:='';
    for i:=0 to Items.Count-1 do
    begin
      txt:=txt+StringAlignLeft(Items[i].Caption, 32);
      if(Items[i].SubItems.Count>0)then
        txt:=txt+Items[i].SubItems[0];
      txt:=txt+#13#10;
    end;

    Clipboard.AsText:=txt;
  end;
end;

procedure TAdminServerHealthForm.CopyselectedVariableMIClick(
  Sender: TObject);
var
  txt: string;
  ListView: TListView;
begin
  if(HealthPageControl.ActivePage=ServerStatusTabSheet)then
    ListView:=ServerStatusListView
  else
    ListView:=VariablesListView;

  if(ListView.Selected<>nil)then
  begin
    txt:=txt+StringAlignLeft(ListView.Selected.Caption, 32);
    if(ListView.Selected.SubItems.Count>0)then
      txt:=txt+ListView.Selected.SubItems[0];

    Clipboard.AsText:=txt;
  end;
end;

procedure TAdminServerHealthForm.CopyselectedVariableNameMIClick(
  Sender: TObject);
var
  txt: string;
  ListView: TListView;
begin
  if(HealthPageControl.ActivePage=ServerStatusTabSheet)then
    ListView:=ServerStatusListView
  else
    ListView:=VariablesListView;

  if(ListView.Selected<>nil)then
  begin
    txt:=txt+ListView.Selected.Caption;

    Clipboard.AsText:=txt;
  end;
end;

procedure TAdminServerHealthForm.VariablesListViewDblClick(
  Sender: TObject);
var  ListView: TListView;
  value: string;
begin
  if(HealthPageControl.ActivePage=ServerStatusTabSheet)then
    ListView:=ServerStatusListView
  else
    ListView:=VariablesListView;

  if(ListView.Selected<>nil)then
    if(ListView.Selected.ImageIndex=1)and
      (ListView.Selected.Data<>nil)then
    begin
      if(ListView.Selected.SubItems.Count>0)then
        value:=ListView.Selected.SubItems[0]
      else
        value:='';

      if(ShowModalEditDialog('Set Variable',
        'Please enter the new value of the variable.',
        myx_mtEdit, 'Set'#13#10'Abort',
        True, TVarListId(ListView.Selected.Data).mysql_id+':',
        value)=1)then
      begin
        if(myx_set_variable(InstanceDM.PMySQL,
          PChar(AnsiToUtf8(TVarListId(ListView.Selected.Data).mysql_id)),
          PChar(AnsiToUtf8(value)))<>0)then
        begin
          raise EMyxSQLError.Create(
            Format('Variable %s cannot be set.', [TVarListId(ListView.Selected.Data).mysql_id]),
            myx_mysql_errno(InstanceDM.PMySQL),
            myx_mysql_error(InstanceDM.PMySQL));
        end
        else
          RefreshBtnClick(self);
      end;
    end;
end;

procedure TAdminServerHealthForm.FetchStatusVarsTimer(Sender: TObject);
var pvars: PMYX_VARIABLES;
begin
  if(StatusVar_PMySQL=nil)then
  begin
    StatusVar_PMySQL:=myx_mysql_init();
    if(StatusVar_PMySQL=nil)then
      raise EMyxError.Create('Error while allocating memory for MySQL Struct.');

    if(myx_connect_to_instance(
      InstanceDM.User_Connection.get_record_pointer,
      StatusVar_PMySQL)<>0)then
    begin
      StatusVar_PMySQL:=nil;
      Exit;
    end;
  end;

  pvars:=myx_get_status_variables(StatusVar_PMySQL);
  if(pvars=nil)then
  begin
    TFetchDataThread(Sender).StatusBarText:='Error fetching Status Variables ('+
      IntToStr(myx_mysql_errno(StatusVar_PMySQL))+')';
    Exit;
  end;

  try
    status_vars_timer:=TMYX_VARIABLES.create(pvars);
  finally
    myx_free_variables(pvars);
  end;
end;

procedure TAdminServerHealthForm.FetchedStatusVarsTimer(Sender: TObject);
var i: integer;
  val, MaxVal: Double;
  Pcompiled_expr,
  Pcompiled_MaxExpr: PMYX_COMPILED_EXPRESSION;
  expr_error: MYX_EXPRESSION_ERROR;
  PVars, POldVars: PMYX_VARIABLES;
  graph: TMYX_HEALTH_GRAPH;
begin
  if(HealthPages=nil)or(ClosingApplication)or(Not(InstanceDM.Connected))then
    Exit;

  if(status_vars_timer_old<>nil)then
  begin
    PVars:=status_vars_timer.get_record_pointer;
    POldVars:=status_vars_timer_old.get_record_pointer;

    if(RefreshSystemVarsInFormulas)then
    begin
      PrepareGraphFormulars;
      RefreshSystemVarsInFormulas:=False;
    end;

    for i:=0 to GraphFrameBundleList.Count-1 do
    begin
      graph:=TGraphFrameBundle(GraphFrameBundleList[i]).Graph;

      Pcompiled_expr:=myx_compile_expression(
        PChar(AnsiToUtf8(TGraphFrameBundle(GraphFrameBundleList[i]).value_formular)),
        @expr_error);
      if(expr_error=MYX_EXPRESSION_SYNTAX_ERROR)then
      begin
        SetStatusText('Expression error in graph '+graph.graph_caption+': '+
          TGraphFrameBundle(GraphFrameBundleList[i]).value_formular);
        continue;
      end;

      if(TGraphFrameBundle(GraphFrameBundleList[i]).max_formular<>'')then
      begin
        Pcompiled_MaxExpr:=myx_compile_expression(
          PChar(AnsiToUtf8(TGraphFrameBundle(GraphFrameBundleList[i]).max_formular)),
          @expr_error);
        if(expr_error=MYX_EXPRESSION_SYNTAX_ERROR)then
        begin
          SetStatusText('Expression error in graph '+graph.graph_caption+': '+
            TGraphFrameBundle(GraphFrameBundleList[i]).max_formular);
          continue;
        end;
      end
      else
        Pcompiled_MaxExpr:=nil;

      try
        val:=Abs(myx_eval_expression(Pcompiled_expr, PVars, POldVars, @expr_error));
        if(expr_error=MYX_EXPRESSION_DIVISION_BY_ZERO)then
          val:=0
        else if(expr_error=MYX_EXPRESSION_SYNTAX_ERROR)then
        begin
          SetStatusText('Expression error in graph '+graph.graph_caption+': '+
            TGraphFrameBundle(GraphFrameBundleList[i]).value_formular);
          continue;
        end;
      finally
        myx_free_expression(Pcompiled_expr);
      end;

      if(Pcompiled_MaxExpr<>nil)then
      begin
        try
          MaxVal:=myx_eval_expression(Pcompiled_MaxExpr, PVars, POldVars, @expr_error);
          if(expr_error=MYX_EXPRESSION_DIVISION_BY_ZERO)then
            val:=0
          else if(expr_error=MYX_EXPRESSION_SYNTAX_ERROR)then
          begin
            SetStatusText('Expression error in graph '+graph.graph_caption+': '+
              TGraphFrameBundle(GraphFrameBundleList[i]).max_formular);
            continue;
          end;
        finally
          myx_free_expression(Pcompiled_MaxExpr);
        end;
      end
      else
        MaxVal:=graph.max;

      if(graph.graphtype=MYX_LINE_GRAPH)then
      begin
        THealthLineGraphFrame(
          TGraphFrameBundle(GraphFrameBundleList[i]).GraphFrame
          ).SetValue(val);

        THealthLineGraphFrame(
          TGraphFrameBundle(GraphFrameBundleList[i]).GraphFrame
          ).SetMaxValue(MaxVal);
      end
      else if(graph.graphtype=MYX_BAR_GRAPH)then
      begin
        THealthBarGraphFrame(
          TGraphFrameBundle(GraphFrameBundleList[i]).GraphFrame
          ).SetMaxValue(MaxVal);

        THealthBarGraphFrame(
          TGraphFrameBundle(GraphFrameBundleList[i]).GraphFrame
          ).SetValue(val);
      end;
    end;
  end
  else
  begin
    PrepareGraphFormulars;

    //Initialize graphs with 0 values
    for i:=0 to GraphFrameBundleList.Count-1 do
    begin
      if(TGraphFrameBundle(GraphFrameBundleList[i]).Graph.graphtype=
        MYX_LINE_GRAPH)then
      begin
        THealthLineGraphFrame(
          TGraphFrameBundle(GraphFrameBundleList[i]).GraphFrame
          ).SetValue(0);
      end
      else if(TGraphFrameBundle(GraphFrameBundleList[i]).Graph.graphtype=
        MYX_BAR_GRAPH)then
      begin
        THealthBarGraphFrame(
          TGraphFrameBundle(GraphFrameBundleList[i]).GraphFrame
          ).SetValue(0);
      end;
    end;
  end;

  if(status_vars_timer_old<>nil)then
    status_vars_timer_old.Free;

  status_vars_timer_old:=status_vars_timer;
  status_vars_timer:=nil;
end;

procedure TAdminServerHealthForm.PrepareGraphFormulars;
var i, j: integer;
begin
  if(status_vars_timer=nil)or(server_vars=nil)then
    Exit;

  for i:=0 to GraphFrameBundleList.Count-1 do
  begin
    TGraphFrameBundle(GraphFrameBundleList[i]).value_formular:=
      TGraphFrameBundle(GraphFrameBundleList[i]).graph.value_formula;
    TGraphFrameBundle(GraphFrameBundleList[i]).max_formular:=
      TGraphFrameBundle(GraphFrameBundleList[i]).graph.max_formula;
  end;

  //Replace all status variable names with index numbers
  for i:=0 to status_vars_timer.variables.Count-1 do
    for j:=0 to GraphFrameBundleList.Count-1 do
    begin
      TGraphFrameBundle(GraphFrameBundleList[j]).value_formular:=
        AnsiReplaceText(TGraphFrameBundle(GraphFrameBundleList[j]).value_formular,
        '['+status_vars_timer.variables[i].name+']',
        '['+IntToStr(i)+']');

      TGraphFrameBundle(GraphFrameBundleList[j]).max_formular:=
        AnsiReplaceText(TGraphFrameBundle(GraphFrameBundleList[j]).max_formular,
        '['+status_vars_timer.variables[i].name+']',
        '['+IntToStr(i)+']');
    end;

  //Replace all system variable names with values
  for i:=0 to server_vars.variables.Count-1 do
    for j:=0 to GraphFrameBundleList.Count-1 do
    begin
      TGraphFrameBundle(GraphFrameBundleList[j]).value_formular:=
        AnsiReplaceText(TGraphFrameBundle(GraphFrameBundleList[j]).value_formular,
        '['+server_vars.variables[i].name+']',
        server_vars.variables[i].value);

      TGraphFrameBundle(GraphFrameBundleList[j]).max_formular:=
        AnsiReplaceText(TGraphFrameBundle(GraphFrameBundleList[j]).max_formular,
        '['+server_vars.variables[i].name+']',
        server_vars.variables[i].value);
    end;
end;

procedure TAdminServerHealthForm.GraphRefreshTmrTimer(Sender: TObject);
begin
  if(ClosingApplication)then
    Exit;

  //Get new status vars every time
  if(status_vars_timer<>nil)then
  begin
    status_vars_timer.Free;
    status_vars_timer:=nil;
  end;

  InstanceDM.FetchData(KindOfData_StatusVars,
    FetchStatusVarsTimer, FetchedStatusVarsTimer,
    nil, '');
end;

function TAdminServerHealthForm.GetVarIndex(vars: TMYX_VARIABLES;
  VarName: string): integer;
var i, VarPos: integer;
begin
  VarPos:=0;

  for i:=0 to vars.variables.Count-1 do
    if(CompareText(vars.variables[i].Name, VarName)=0)then
    begin
      VarPos:=i;
      break;
    end;

  Result:=VarPos;
end;

procedure TAdminServerHealthForm.LoadGraphs;
var phealth_pages: PMYX_HEALTH_PAGES;
  error: MYX_ADMIN_LIB_ERROR;
begin
  phealth_pages:=myx_read_in_health_pages(
    PChar(AnsiToUtf8(MYXCommonOptions.XMLDir+'mysqlx_health.xml')),
    @error);
  if(phealth_pages=nil)or(error<>MYX_ADMIN_NO_ERROR)then
    raise EMyxAdminLibError.Create(Ord(error),
      MYXCommonOptions.XMLDir+'mysqlx_health.xml');

  try
    HealthPages:=TMYX_HEALTH_PAGES.Create(phealth_pages);
  finally
    myx_free_health_pages(phealth_pages);
  end;
end;

procedure TAdminServerHealthForm.BuildGraphs;
var i, j, k: integer;
  PageTabSheet: TTabSheet;
  ScrollBox: TScrollBox;
  GroupBox: TGroupBox;
  page: TMYX_HEALTH_PAGE;
  current_widget_ypos,
  current_group_height,
  current_group_ypos: integer;
begin
  LoadGraphs;

  if(HealthPages<>nil)then
  begin
    for i:=0 to HealthPages.pages.Count-1 do
    begin
      page:=HealthPages.pages[i];

      PageTabSheet:=nil;
      ScrollBox:=nil;
      AddPage(page, PageTabSheet, ScrollBox);

      current_group_ypos:=8;

      for j:=0 to page.groups.Count-1 do
      begin
        GroupBox:=TGroupBox.Create(ScrollBox);
        GroupBox.Parent:=ScrollBox;
        GroupBox.Name:='GBox'+IntToStr(i)+'_'+IntToStr(j);
        GroupBox.Caption:=page.groups[j].caption;
        GroupBox.Left:=12;
        GroupBox.Top:=current_group_ypos;
        GroupBox.Width:=GroupBoxWidth;
        GroupBox.Anchors:=[akLeft, akTop, akRight];
        GroupBox.PopupMenu:=GroupBoxPopupMenu;

        current_group_height:=25;

        current_widget_ypos:=22;
        for k:=0 to page.groups[j].graphs.Count-1 do
        begin
          AddGraphToGroupBox(GroupBox, page.groups[j].graphs[k],
            current_widget_ypos);

          current_group_height:=current_widget_ypos;
        end;

        GroupBox.Height:=current_group_height;

        inc(current_group_ypos, GroupBox.Height+GroupBoxVSpace);
      end;
    end;

    //Now display Pages
    for i:=0 to HealthPageControl.PageCount-1 do
      HealthPageControl.Pages[i].TabVisible:=True;
  end;

  ServerStatusTabSheet.PageIndex:=HealthPageControl.PageCount-1;
  ServerVariablesTabSheet.PageIndex:=HealthPageControl.PageCount-1;
end;

procedure TAdminServerHealthForm.AddPage(page: TMYX_HEALTH_PAGE;
  var PageTabSheet: TTabSheet; var ScrollBox: TScrollBox);
begin
  PageTabSheet:=TTabSheet.Create(self);
  PageTabSheet.Name:='HeathPage'+IntToStr(HealthPageControl.PageCount-2);
  PageTabSheet.Caption:=page.caption;
  PageTabSheet.TabVisible:=False;
  PageTabSheet.PageControl:=HealthPageControl;
  PageTabSheet.Width:=563;
  PageTabSheet.Height:=412;

  ScrollBox:=TScrollBox.Create(PageTabSheet);
  ScrollBox.Parent:=PageTabSheet;
  ScrollBox.Name:='SB'+IntToStr(HealthPageControl.PageCount-2);
  ScrollBox.Top:=46;
  ScrollBox.Left:=0;
  ScrollBox.Height:=PageTabSheet.Height-ScrollBox.Top;
  ScrollBox.Width:=PageTabSheet.Width;
  ScrollBox.Anchors:=[akLeft, akTop, akRight, akBottom];
  //ScrollBox.Align:=alClient;
  ScrollBox.BorderStyle:=bsNone;
  if(MYXCommonOptions.XPStyleEnabled)then
    ScrollBox.Color:=clWhite;
  ScrollBox.VertScrollBar.Smooth:=True;
  ScrollBox.VertScrollBar.Tracking:=True;
  ScrollBox.AutoScroll:=True;
  ScrollBox.Width:=PageTabSheet.Width;
end;

procedure TAdminServerHealthForm.AddGraphToGroupBox(GroupBox: TGroupBox;
  graph: TMYX_HEALTH_GRAPH; var current_widget_ypos: integer);
var LineGraph: THealthLineGraphFrame;
  BarGraph: THealthBarGraphFrame;
begin
  if(graph.graphtype=MYX_LINE_GRAPH)then
  begin
    LineGraph:=THealthLineGraphFrame.Create(
      GroupBox,
      graph,
      graph.value_caption,
      graph.min,
      graph.max,
      graph.value_unit);
    LineGraph.Parent:=GroupBox;
    LineGraph.Name:='HealthGraph_'+GroupBox.Name+'_'+IntToStr(GroupBox.ComponentCount);
    LineGraph.Left:=16;
    LineGraph.Top:=current_widget_ypos;
    LineGraph.Width:=GroupBox.Width-30; //504;
    LineGraph.Anchors:=[akLeft, akTop, akRight];
    LineGraph.PopupMenu:=GraphPopupMenu;

    GraphFrameBundleList.Add(TGraphFrameBundle.Create(
      graph, LineGraph));

    Inc(current_widget_ypos, LineGraph.Height+10);
  end
  else if(graph.graphtype=MYX_BAR_GRAPH)then
  begin
    BarGraph:=THealthbarGraphFrame.Create(
      GroupBox,
      graph.graph_caption,
      graph.display_graph_caption,
      graph.value_caption,
      graph.max_caption,
      graph.min,
      graph.max);
    BarGraph.Parent:=GroupBox;
    BarGraph.Name:='HealthGraph_'+GroupBox.Name+'_'+IntToStr(GroupBox.ComponentCount);
    BarGraph.Left:=16;
    BarGraph.Top:=current_widget_ypos;
    BarGraph.Anchors:=[akLeft, akTop, akRight];
    BarGraph.PopupMenu:=GraphPopupMenu;

    GraphFrameBundleList.Add(TGraphFrameBundle.Create(
      graph, BarGraph));

    Inc(current_widget_ypos, 19+10);
  end;
end;

procedure TAdminServerHealthForm.Disconnected(var Message: TMessage);
begin
  DisableEnablePages(HealthPageControl, False);

  GraphRefreshTmr.Enabled:=False;
end;

procedure TAdminServerHealthForm.Reconnected(var Message: TMessage);
begin
  DisableEnablePages(HealthPageControl, True);

  if(HealthPages=nil)then
    BuildGraphs;

  if(server_vars=nil)then
    FetchServerVars(self);

  if(status_vars=nil)then
    FetchStatusVars(self);

  GraphRefreshTmr.Enabled:=True;

  HealthPageControl.ActivePageIndex:=0;
  HealthPageControlChange(self);
end;

procedure TAdminServerHealthForm.AddaPageMIClick(Sender: TObject);
var Caption, Description: string;
  pos: integer;
  page: TMYX_HEALTH_PAGE;
  PageTabSheet: TTabSheet;
  ScrollBox: TScrollBox;
begin
  if(ShowModalEditDialog('Add a Page',
    'Enter the caption of the page to add.',
    myx_mtEdit, 'Continue'#13#10'Cancel',
    True, 'Page Caption:', Caption)=1)then
  begin
    if(ShowModalEditDialog('Page Description',
      'Enter the page description of the new '+Caption+' page ',
      myx_mtEdit, 'Create Page'#13#10'Cancel',
      True, 'Page Description:', Description)=1)then
    begin
      pos:=HealthPageControl.ActivePageIndex+1;
      if(pos>HealthPages.pages.Count-1)then
        pos:=HealthPages.pages.Count-1;
      if(pos<0)then
        pos:=0;

      page:=TMYX_HEALTH_PAGE.create(Caption, Caption,
        Description, Description,
        pos);
      HealthPages.pages.Insert(pos, page);


      PageTabSheet:=nil;
      ScrollBox:=nil;
      AddPage(page, PageTabSheet, ScrollBox);

      PageTabSheet.PageIndex:=pos;
      PageTabSheet.TabVisible:=True;

      HealthPageControl.ActivePage:=PageTabSheet;

      HealthPageControlChange(self);
    end;
  end;
end;

procedure TAdminServerHealthForm.DeletePageMIClick(Sender: TObject);
var pos: integer;
  TabSheet: TTabSheet;
  i, j, k: integer;
begin
  pos:=HealthPageControl.ActivePageIndex;
  if(pos<HealthPageControl.PageCount-2)and
    (pos<HealthPages.pages.Count)then
  begin
    if(ShowModalDialog('Delete Page?',
      'Are you sure you want to delete the selected page '+
      HealthPages.pages[pos].caption+'?',
      myx_mtConfirmation, 'Yes'#13#10'No')=1)then
    begin
      HealthPageControl.ActivePageIndex:=pos+1;

      //Hide Tab since it cannot be removed
      TabSheet:=HealthPageControl.Pages[pos];
      TabSheet.TabVisible:=False;
      TabSheet.PageIndex:=HealthPageControl.PageCount-1;

      while(TabSheet.ComponentCount>0)do
      begin
        DeleteComponentWithChildren(TabSheet.Components[0]);
      end;

      for i:=0 to HealthPages.pages[pos].groups.Count-1 do
        for j:=0 to HealthPages.pages[pos].groups[i].graphs.Count-1 do
          for k:=0 to GraphFrameBundleList.Count-1 do
            if(TGraphFrameBundle(GraphFrameBundleList[k]).Graph=
              HealthPages.pages[pos].groups[i].graphs[j])then
            begin
              GraphFrameBundleList.Delete(k);
              break;
            end;

      HealthPages.pages.Delete(pos);

      HealthPageControlChange(self);
    end;
  end;
end;

procedure TAdminServerHealthForm.AddaGroupMIClick(Sender: TObject);
var Caption: string;
  pos: integer;
  group: TMYX_HEALTH_GROUP;
  GroupBox: TGroupBox;
  ScrollBox: TScrollBox;
  current_group_ypos: integer;
  i: integer;
begin
  if(HealthPageControl.ActivePage.Controls[0] is TScrollBox)then
    ScrollBox:=TScrollBox(HealthPageControl.ActivePage.Controls[0])
  else
    Exit;

  if(ShowModalEditDialog('Add a Group',
    'Enter the caption of the group to add.',
    myx_mtEdit, 'Create Group'#13#10'Cancel',
    True, 'Group Caption:', Caption)=1)then
  begin
    pos:=HealthPages.pages[HealthPageControl.ActivePageIndex].groups.Count;

    group:=TMYX_HEALTH_GROUP.create(Caption, Caption, pos);

    HealthPages.pages[HealthPageControl.ActivePageIndex].groups.Add(group);

    current_group_ypos:=0;

    for i:=0 to ScrollBox.ComponentCount-1 do
      if(ScrollBox.Components[i] is TGroupBox)then
        if(TGroupBox(ScrollBox.Components[i]).Top+
          TGroupBox(ScrollBox.Components[i]).Height+GroupBoxVSpace>
            current_group_ypos)then
          current_group_ypos:=TGroupBox(ScrollBox.Components[i]).Top+
            TGroupBox(ScrollBox.Components[i]).Height+GroupBoxVSpace;

    GroupBox:=TGroupBox.Create(ScrollBox);
    GroupBox.Parent:=ScrollBox;
    GroupBox.Name:='GBox'+IntToStr(HealthPageControl.ActivePageIndex)+'_'+IntToStr(pos);
    GroupBox.Caption:=Caption;
    GroupBox.Left:=12;
    GroupBox.Top:=current_group_ypos;
    GroupBox.Width:=ScrollBox.Width-28;
    GroupBox.Anchors:=[akLeft, akTop, akRight];
    GroupBox.PopupMenu:=GroupBoxPopupMenu;

    GroupBox.Height:=25;
  end;
end;

procedure TAdminServerHealthForm.EditGraphMIClick(Sender: TObject);
var Frame: TFrame;
  i: integer;
begin
  if(TPopupMenu(TMenuItem(Sender).GetParentMenu).PopupComponent.InheritsFrom(TFrame))then
    Frame:=TFrame(TPopupMenu(TMenuItem(Sender).GetParentMenu).PopupComponent)
  else
    Frame:=nil;

  if(Frame<>nil)then
  begin
    if(AdminServerHealthGraphSettingsForm=nil)then
      AdminServerHealthGraphSettingsForm:=
        TAdminServerHealthGraphSettingsForm.Create(self);

    for i:=0 to GraphFrameBundleList.Count-1 do
      if(TGraphFrameBundle(GraphFrameBundleList[i]).GraphFrame=Frame)then
      begin
        AdminServerHealthGraphSettingsForm.SetGraph(
          TGraphFrameBundle(GraphFrameBundleList[i]).Graph);
        break;
      end;

    AdminServerHealthGraphSettingsForm.Show;
  end;
end;

procedure TAdminServerHealthForm.ServerStatusListViewMouseDown(
  Sender: TObject; Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
begin
  TListView(Sender).BeginDrag(False, 5);
end;

procedure TAdminServerHealthForm.AddaGraphMIClick(Sender: TObject);
var GroupBox: TGroupBox;
begin
  if(TPopupMenu(TMenuItem(Sender).GetParentMenu).PopupComponent is TGroupBox)then
    GroupBox:=TGroupBox(TPopupMenu(TMenuItem(Sender).GetParentMenu).PopupComponent)
  else if(TPopupMenu(TMenuItem(Sender).GetParentMenu).PopupComponent.InheritsFrom(TFrame))then
    GroupBox:=TGroupBox(TPopupMenu(TMenuItem(Sender).GetParentMenu).PopupComponent.Owner)
  else
    Exit;

  if(AdminServerHealthGraphSettingsForm=nil)then
    AdminServerHealthGraphSettingsForm:=
      TAdminServerHealthGraphSettingsForm.Create(self);

  AdminServerHealthGraphSettingsForm.NewGraph(GroupBox);

  AdminServerHealthGraphSettingsForm.Show;
end;

procedure TAdminServerHealthForm.OptionsChanged(var Message: TMessage);
begin
  //
end;


end.
