unit AdminServerHealthBarGraph;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, 
  Dialogs, AuxFuncs, StdCtrls, ExtCtrls, PNGImage;

type
  THealthBarGraphFrame = class(TFrame)
    ValueImg: TImage;
    MainPnl: TPanel;
    EmptyImg: TImage;
    ValueLbl: TLabel;
    MaxLbl: TLabel;
    CaptionPnl: TPanel;
    CaptionLbl: TLabel;

    constructor Create(AOwner: TComponent;
      GraphCaption: string = '';
      DisplayGraphCaption: integer = 0;
      ValueCaption: string = '';
      MaxCaption: string = '';
      MinValue: Double = 0; MaxValue: Double = 100); reintroduce;
    destructor Destroy; override;
    procedure SetValue(value: double);
    procedure SetMaxValue(MaxValue: double);
    procedure FrameResize(Sender: TObject);
  private
    { Private declarations }
    EmptyPNGImg,
    FilledPNGImg: TPNGObject;

    MinValue, MaxValue: Double;

    ValueCaption,
    MaxCaption: string;

    value: double;
  public
    { Public declarations }
  end;

implementation

{$R *.dfm}

constructor THealthBarGraphFrame.Create(AOwner: TComponent;
  GraphCaption: string = '';
  DisplayGraphCaption: integer = 0;
  ValueCaption: string = '';
  MaxCaption: string = '';
  MinValue: Double = 0; MaxValue: Double = 100);
begin
  inherited Create(AOwner);

  EmptyPNGImg:=LoadPNGImageFromResource('health_bar_empty', EmptyImg);
  FilledPNGImg:=LoadPNGImageFromResource('health_bar_filled', ValueImg);

  ValueImg.Width:=0;

  self.ValueCaption:=ValueCaption;
  self.MaxCaption:=MaxCaption;

  ValueLbl.Caption:='';
  MaxLbl.Caption:='';

  self.MinValue:=MinValue;
  self.MaxValue:=MaxValue;

  CaptionLbl.Caption:=GraphCaption+':';

  if(DisplayGraphCaption=0)then
    CaptionPnl.Width:=0;
end;

destructor THealthBarGraphFrame.Destroy;
begin
  EmptyPNGImg.Free;
  FilledPNGImg.Free;

  inherited Destroy;
end;

procedure THealthBarGraphFrame.SetValue(value: double);
begin
  if(value>MaxValue)then
    value:=MaxValue;

  ValueLbl.Caption:=ValueCaption+' '+FormatFloat('###,###,##0', Value);

  ValueImg.Width:=Round((value-MinValue)/(MaxValue-MinValue)*EmptyImg.Width);

  self.value:=value;
end;

procedure THealthBarGraphFrame.SetMaxValue(MaxValue: double);
begin
  MaxLbl.Caption:=MaxCaption+' '+FormatFloat('###,###,##0', MaxValue);

  self.MaxValue:=MaxValue;

  SetValue(self.value);
end;

procedure THealthBarGraphFrame.FrameResize(Sender: TObject);
begin
  SetValue(value);
end;

end.
