unit AdminServerHealthGraphSettings;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, myx_admin_public_interface, ComCtrls;

type
  TAdminServerHealthGraphSettingsForm = class(TForm)
    GroupBox1: TGroupBox;
    CancelBtn: TButton;
    ApplyBtn: TButton;
    Label3: TLabel;
    ValueUnitCBox: TComboBox;
    ValueFormulaMemo: TMemo;
    Label6: TLabel;
    GroupBox2: TGroupBox;
    Label1: TLabel;
    CaptionEd: TEdit;
    DisplayCaptionCBox: TCheckBox;
    Label2: TLabel;
    GraphTypeCBox: TComboBox;
    GroupBox3: TGroupBox;
    Label4: TLabel;
    MinValueEd: TEdit;
    Label5: TLabel;
    MaxValueEd: TEdit;
    AutoextendValueCBox: TCheckBox;
    Label7: TLabel;
    MaxFormularEd: TEdit;
    Label8: TLabel;
    ValueCaption: TEdit;
    Label9: TLabel;
    MaxCaptionEd: TEdit;
    Label10: TLabel;
    RefreshTimeCBox: TComboBox;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    Label17: TLabel;
    Label18: TLabel;
    Label19: TLabel;
    Label20: TLabel;
    Label21: TLabel;
    Label22: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);

    procedure SetGraph(graph: TMYX_HEALTH_GRAPH);
    procedure ApplyBtnClick(Sender: TObject);
    procedure CancelBtnClick(Sender: TObject);
    procedure ValueFormulaMemoDragOver(Sender, Source: TObject; X,
      Y: Integer; State: TDragState; var Accept: Boolean);
    procedure ValueFormulaMemoDragDrop(Sender, Source: TObject; X,
      Y: Integer);
    procedure SetGraphValues(graph: TMYX_HEALTH_GRAPH);
    function GetGraphWithValues(pos: integer): TMYX_HEALTH_GRAPH;

    procedure NewGraph(GroupBox: TGroupBox);
    procedure FormDeactivate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
  private
    { Private declarations }
    graph: TMYX_HEALTH_GRAPH;
    GroupBox: TGroupBox;
  public
    { Public declarations }
  end;

implementation

uses AdminServerHealth;

{$R *.dfm}

procedure TAdminServerHealthGraphSettingsForm.FormCreate(Sender: TObject);
begin
  Top:=12;
  Left:=12;
end;

procedure TAdminServerHealthGraphSettingsForm.FormDestroy(Sender: TObject);
begin
  //
end;

procedure TAdminServerHealthGraphSettingsForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  //
end;

procedure TAdminServerHealthGraphSettingsForm.SetGraph(graph: TMYX_HEALTH_GRAPH);
begin
  self.graph:=graph;

  if(graph<>nil)then
  begin
    CaptionEd.Text:=graph.graph_caption;
    DisplayCaptionCBox.Checked:=(graph.display_graph_caption=1);
    GraphTypeCBox.ItemIndex:=Ord(graph.graphtype)-1;
    GraphTypeCBox.Enabled:=False;

    ValueFormulaMemo.Text:=graph.value_formula;
    ValueUnitCBox.ItemIndex:=Ord(graph.value_unit);
    ValueCaption.Text:=graph.value_caption;

    MinValueEd.Text:=FormatFloat('#######0.##', graph.min);
    MaxValueEd.Text:=FormatFloat('#######0.##', graph.max);
    AutoextendValueCBox.Checked:=(graph.autoextend_max=1);
    MaxFormularEd.Text:=graph.max_formula;
    MaxCaptionEd.Text:=graph.max_caption;
  end
  else
  begin
    CaptionEd.Text:='';
    DisplayCaptionCBox.Checked:=False;
    GraphTypeCBox.ItemIndex:=0;
    GraphTypeCBox.Enabled:=True;

    ValueFormulaMemo.Text:='';
    ValueUnitCBox.ItemIndex:=0;
    ValueCaption.Text:='';

    MinValueEd.Text:='0';
    MaxValueEd.Text:='100';
    AutoextendValueCBox.Checked:=False;
    MaxFormularEd.Text:='';
    MaxCaptionEd.Text:='';
  end;
end;

procedure TAdminServerHealthGraphSettingsForm.NewGraph(
  GroupBox: TGroupBox);
begin
  self.GroupBox:=GroupBox;

  SetGraph(nil);
end;

procedure TAdminServerHealthGraphSettingsForm.SetGraphValues(
  graph: TMYX_HEALTH_GRAPH);
begin
  GroupBox:=nil;

  graph.graph_caption:=CaptionEd.Text;
  graph.display_graph_caption:=Ord(DisplayCaptionCBox.Checked);
  case GraphTypeCBox.ItemIndex+1 of
    Ord(MYX_LINE_GRAPH):
      graph.graphtype:=MYX_LINE_GRAPH;
    Ord(MYX_BAR_GRAPH):
      graph.graphtype:=MYX_BAR_GRAPH;
  end;


  graph.value_formula:=ValueFormulaMemo.Text;
  case ValueUnitCBox.ItemIndex of
    Ord(MYX_HGVU_PERCENTAGE):
      graph.value_unit:=MYX_HGVU_PERCENTAGE;
    Ord(MYX_HGVU_COUNT):
      graph.value_unit:=MYX_HGVU_COUNT;
    Ord(MYX_HGVU_BYTE):
      graph.value_unit:=MYX_HGVU_BYTE;
    Ord(MYX_HGVU_SECONDS):
      graph.value_unit:=MYX_HGVU_SECONDS;
  end;

  graph.value_caption:=ValueCaption.Text;

  graph.min:=StrToFloat(MinValueEd.Text);
  graph.max:=StrToFloat(MaxValueEd.Text);
  graph.autoextend_max:=Ord(AutoextendValueCBox.Checked);
  graph.max_formula:=MaxFormularEd.Text;
  graph.max_caption:=MaxCaptionEd.Text;

  case RefreshTimeCBox.ItemIndex of
    0:
      graph.refreshtime:=200;
    1:
      graph.refreshtime:=500;
    2:
      graph.refreshtime:=1000;
    3:
      graph.refreshtime:=2000;
    4:
      graph.refreshtime:=5000;
    5:
      graph.refreshtime:=10000;
    6:
      graph.refreshtime:=15000;
    7:
      graph.refreshtime:=30000;
    8:
      graph.refreshtime:=60000;
  end;
end;

function TAdminServerHealthGraphSettingsForm.GetGraphWithValues(pos: integer): TMYX_HEALTH_GRAPH;
var graph: TMYX_HEALTH_GRAPH;
  graphtype: MYX_HEALTH_GRAPH_TYPE;
  value_unit: MYX_HEALTH_GRAPH_VALUE_UNIT;
  refreshtime: integer;
begin
  case GraphTypeCBox.ItemIndex+1 of
    Ord(MYX_LINE_GRAPH):
      graphtype:=MYX_LINE_GRAPH;
    Ord(MYX_BAR_GRAPH):
      graphtype:=MYX_BAR_GRAPH;
  else
    graphtype:=MYX_LINE_GRAPH;
  end;

  case ValueUnitCBox.ItemIndex of
    Ord(MYX_HGVU_PERCENTAGE):
      value_unit:=MYX_HGVU_PERCENTAGE;
    Ord(MYX_HGVU_COUNT):
      value_unit:=MYX_HGVU_COUNT;
    Ord(MYX_HGVU_BYTE):
      value_unit:=MYX_HGVU_BYTE;
    Ord(MYX_HGVU_SECONDS):
      value_unit:=MYX_HGVU_SECONDS;
  else
    value_unit:=MYX_HGVU_PERCENTAGE;
  end;

  case RefreshTimeCBox.ItemIndex of
    0:
      refreshtime:=200;
    1:
      refreshtime:=500;
    2:
      refreshtime:=1000;
    3:
      refreshtime:=2000;
    4:
      refreshtime:=5000;
    5:
      refreshtime:=10000;
    6:
      refreshtime:=15000;
    7:
      refreshtime:=30000;
    8:
      refreshtime:=60000;
  else
    refreshtime:=1000;
  end;

  graph:=TMYX_HEALTH_GRAPH.create(CaptionEd.Text,
    Ord(DisplayCaptionCBox.Checked), graphtype, value_unit,
    StrToFloat(MinValueEd.Text), StrToFloat(MaxValueEd.Text),
    Ord(AutoextendValueCBox.Checked), ValueFormulaMemo.Text,
    MaxFormularEd.Text, ValueCaption.Text,
    ValueCaption.Text, MaxCaptionEd.Text, MaxCaptionEd.Text,
    refreshtime, pos);

  Result:=graph;
end;

procedure TAdminServerHealthGraphSettingsForm.ApplyBtnClick(
  Sender: TObject);
var current_widget_ypos: integer;
  i: integer;
begin
  if(graph<>nil)then
    SetGraphValues(graph)
  else
  begin
    graph:=GetGraphWithValues(GroupBox.ComponentCount);

    current_widget_ypos:=GroupBox.Height;

    TAdminServerHealthForm(Owner).AddGraphToGroupBox(GroupBox, graph, current_widget_ypos);

    for i:=0 to GroupBox.Owner.ComponentCount-1 do
      if(GroupBox.Owner.Components[i] is TGroupBox)then
        if(TGroupBox(GroupBox.Owner.Components[i]).Top>
          GroupBox.Top)then
          TGroupBox(GroupBox.Owner.Components[i]).Top:=
            TGroupBox(GroupBox.Owner.Components[i]).Top+
            current_widget_ypos-GroupBox.Height;
            
    GroupBox.Height:=current_widget_ypos;
  end;


  TAdminServerHealthForm(Owner).RefreshSystemVarsInFormulas:=True;

  graph:=nil;

  ModalResult:=mrOK;
  Hide;
end;

procedure TAdminServerHealthGraphSettingsForm.CancelBtnClick(
  Sender: TObject);
begin
  graph:=nil;

  ModalResult:=mrCancel;
  Hide;
end;

procedure TAdminServerHealthGraphSettingsForm.ValueFormulaMemoDragOver(
  Sender, Source: TObject; X, Y: Integer; State: TDragState;
  var Accept: Boolean);
begin
  Accept:=False;

  if(Source is TListView)then
    if(TListView(Source).Owner=Owner)then
      Accept:=True;
end;

procedure TAdminServerHealthGraphSettingsForm.ValueFormulaMemoDragDrop(
  Sender, Source: TObject; X, Y: Integer);
var i: integer;
  lastChar: string;
  ListView: TListView;
begin
  if(Source is TListView)and(Sender.InheritsFrom(TCustomEdit))then
  begin
    with Owner as TAdminServerHealthForm do
    begin
      if(HealthPageControl.ActivePage=ServerStatusTabSheet)then
        ListView:=ServerStatusListView
      else if(HealthPageControl.ActivePage=ServerVariablesTabSheet)then
        ListView:=VariablesListView
      else
        ListView:=nil;

      if(ListView<>nil)then
      begin
        for i:=0 to ListView.Items.Count-1 do
          if(ListView.Items[i].Selected)then
          begin
            lastChar:=Copy(TCustomEdit(Sender).Text, Length(TCustomEdit(Sender).Text), 1);
            if(lastChar<>'')and(lastChar<>'+')and(lastChar<>'-')and
              (lastChar<>'*')and(lastChar<>'/')then
              TCustomEdit(Sender).Text:=TCustomEdit(Sender).Text+'+';

            TCustomEdit(Sender).Text:=TCustomEdit(Sender).Text+
              '['+ListView.Items[i].Caption+']';
          end;
      end;
    end;

    TCustomEdit(Sender).SetFocus;
    TCustomEdit(Sender).SelStart:=Length(TCustomEdit(Sender).Text);
  end;
end;

procedure TAdminServerHealthGraphSettingsForm.FormDeactivate(
  Sender: TObject);
begin
  AlphaBlendValue:=220;
end;

procedure TAdminServerHealthGraphSettingsForm.FormActivate(
  Sender: TObject);
begin
  AlphaBlendValue:=255;
end;

end.
