unit AdminServerInfo;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, AdditionalClasses, Buttons, PNGImage,
  InstanceSections, AuxFuncs, myx_public_interface,
  ApplicationDataModule, InstanceDataModule, MyxError;

type
  TAdminServerInfoForm = class(TInstanceSectionForm)
    ServerInfoPnl: TPanel;
    LogoImg: TImage;
    RightBGImg: TImage;
    ConnectionGBox: TGroupBox;
    UserLbl: TLabel;
    HostLbl: TLabel;
    ClientInfoGBox: TGroupBox;
    Label12: TLabel;
    Label8: TLabel;
    Label3: TLabel;
    Label2: TLabel;
    Label1: TLabel;
    Label4: TLabel;
    Label15: TLabel;
    PortLbl: TLabel;
    InstanceCaptionLbl: TLabel;
    InstanceNameLbl: TLabel;
    ClientNetworkNameLbl: TLabel;
    ClientOSLbl: TLabel;
    ClientHardwareLbl: TLabel;
    ClientVersionLbl: TLabel;
    Label20: TLabel;
    ClientIPLbl: TLabel;
    ServiceStatusImg: TImage;
    Label5: TLabel;
    ServerStatusLbl: TLabel;
    ServerInfoGBox: TGroupBox;
    Label16: TLabel;
    ServerHardwareCaptionLbl: TLabel;
    ServerOSCaptionLbl: TLabel;
    Label24: TLabel;
    ServerNetworkNameLbl: TLabel;
    ServerOSLbl: TLabel;
    ServerHardwareLbl: TLabel;
    ServerVersionLbl: TLabel;
    Label29: TLabel;
    ServerIPLbl: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ServerStatusLblMouseEnter(Sender: TObject);
    procedure ServerStatusLblMouseLeave(Sender: TObject);
    procedure ServerStatusLblClick(Sender: TObject);
    procedure ServerInfoPnlResize(Sender: TObject);

    procedure SetServerInfos(Sender: TObject);
    procedure SetClientInfos(Sender: TObject);
    procedure FetchInfos(Sender: TObject);

    procedure GetInfos;
    procedure ClearInfos;

    procedure Disconnected(var Message: TMessage); message WM_Disconnected;
    procedure Reconnected(var Message: TMessage); message WM_Reconnected;
  private
    { Private declarations }
    RightBGPNGImg,
    MySQLLogoPNGImg: TPNGObject;

    ServiceStatusRunningPNGImg,
    //ServiceStatusStoppedPNGImg,
    ServiceStatusUnknownPNGImg: TPNGObject;

    machine_info: PMYX_MACHINE_INFO;
  public
    { Public declarations }
  end;

var
  AdminServerInfoForm: TAdminServerInfoForm;

implementation

{$R *.dfm}

uses Main;

procedure TAdminServerInfoForm.FormCreate(Sender: TObject);
begin
  RightBGImg.Width:=_d(RightBGImg.Width);

  DockedPanel:=ServerInfoPnl;

  RightBGPNGImg:=LoadPNGImageFromResource('dockpnl_bg', RightBGImg);
  MySQLLogoPNGImg:=LoadPNGImageFromResource('mysql_logo', LogoImg);

  ServiceStatusRunningPNGImg:=LoadPNGImageFromResource('service_status_running');
  //ServiceStatusStoppedPNGImg:=LoadPNGImageFromResource('service_status_stopped');
  ServiceStatusUnknownPNGImg:=LoadPNGImageFromResource('service_status_unknown');

  ServiceStatusImg.Picture.Graphic:=ServiceStatusRunningPNGImg;

  GetInfos;
end;

procedure TAdminServerInfoForm.FormDestroy(Sender: TObject);
begin
  RightBGPNGImg.Free;
  MySQLLogoPNGImg.Free;
end;

procedure TAdminServerInfoForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  //
end;

procedure TAdminServerInfoForm.ServerStatusLblMouseEnter(Sender: TObject);
begin
  ServerStatusLbl.Font.Style:=[fsBold, fsUnderline];
  ServerStatusLbl.Font.Color:=clBlue;
end;

procedure TAdminServerInfoForm.ServerStatusLblMouseLeave(Sender: TObject);
begin
  ServerStatusLbl.Font.Style:=[fsBold];
  ServerStatusLbl.Font.Color:=clBlack;
end;

procedure TAdminServerInfoForm.ServerStatusLblClick(Sender: TObject);
begin
  if(Owner is TMainForm)then
  begin
    TMainForm(Owner).SectionControls.AdminTreeView.Selected:=TMainForm(Owner).SectionControls.AdminTreeView.Items[1];
  end;
end;

procedure TAdminServerInfoForm.ServerInfoPnlResize(Sender: TObject);
begin
  LogoImg.Left:=ServerInfoPnl.Width-636+484;
end;

procedure TAdminServerInfoForm.SetServerInfos(Sender: TObject);
begin
  ServerVersionLbl.Caption:=machine_info.version;
  ServerNetworkNameLbl.Caption:=machine_info.network_name;
  ServerIPLbl.Caption:=machine_info.IP;
  ServerOSLbl.Caption:=machine_info.OS;
  ServerHardwareLbl.Caption:=machine_info.hardware;
end;

procedure TAdminServerInfoForm.SetClientInfos(Sender: TObject);
begin
  ClientVersionLbl.Caption:=machine_info.version;
  if(machine_info.network_name<>'')then
    ClientNetworkNameLbl.Caption:=machine_info.network_name
  else
  begin
    if(machine_info.IP='127.0.0.1')then
      ClientNetworkNameLbl.Caption:='localhost'
    else
      ClientNetworkNameLbl.Caption:='unknown';
  end;
  ClientIPLbl.Caption:=machine_info.IP;
  ClientOSLbl.Caption:=machine_info.OS;
  ClientHardwareLbl.Caption:=machine_info.hardware;
end;

procedure TAdminServerInfoForm.FetchInfos(Sender: TObject);
var PMySQL: Pointer;
begin
  PMySQL:=myx_mysql_init;
  try
    if(myx_connect_to_instance(InstanceDM.user_connection.get_record_pointer,
      PMySQL)<>0)then
      raise EMyxSQLError.Create('Cannot clone MySQL connection.',
        myx_mysql_errno(PMySQL), myx_mysql_error(PMySQL));

    //Display client infos
    machine_info:=myx_get_client_info(PMySQL);

    TFetchDataThread(Sender).ExecuteSynchronized(SetClientInfos);

    myx_free_pc_info(machine_info);

    //Display server infos
    machine_info:=myx_get_server_info(
      InstanceDM.user_connection.get_record_pointer,
      PMySQL);

    TFetchDataThread(Sender).ExecuteSynchronized(SetServerInfos);

    myx_free_pc_info(machine_info);
  finally
    myx_mysql_close(PMySQL);
  end;
end;

procedure TAdminServerInfoForm.GetInfos;
var NewHeight: integer;
begin
  //Start Thread fetching data
  InstanceDM.FetchData(KindOfData_StatusVars,
    FetchInfos, nil,
    nil, 'Fetching Server Status ...', True);

  //Display connection Infos
  UserLbl.Caption:=
    InstanceDM.user_connection.username;
  HostLbl.Caption:=
    InstanceDM.user_connection.hostname;
  PortLbl.Caption:=
    IntToStr(InstanceDM.user_connection.port);
  InstanceNameLbl.Caption:='';

  //Hide unknown fields
  NewHeight:=105;
  if(InstanceNameLbl.Caption='-')or
    (InstanceNameLbl.Caption='')then
  begin
    InstanceCaptionLbl.Hide;
    InstanceNameLbl.Hide;
    NewHeight:=NewHeight-18;
  end;
  ConnectionGBox.Height:=NewHeight;

  ServerInfoGBox.Top:=ConnectionGBox.Top+ConnectionGBox.Height+9;

  NewHeight:=127;

  if(ServerOSLbl.Caption='-')then
  begin
    ServerOSCaptionLbl.Hide;
    ServerOSLbl.Hide;
    NewHeight:=NewHeight-18;
  end;

  if(ServerHardwareLbl.Caption='-')then
  begin
    ServerHardwareCaptionLbl.Hide;
    ServerHardwareLbl.Hide;
    NewHeight:=NewHeight-18;
  end;

  ServerInfoGBox.Height:=NewHeight;

  ClientInfoGBox.Top:=ServerInfoGBox.Top+ServerInfoGBox.Height+9;
end;

procedure TAdminServerInfoForm.ClearInfos;
begin
  UserLbl.Caption:='-';
  HostLbl.Caption:='-';
  PortLbl.Caption:='-';
  InstanceNameLbl.Caption:='-';

  ServerVersionLbl.Caption:='-';
  ServerNetworkNameLbl.Caption:='-';
  ServerIPLbl.Caption:='-';
  ServerOSLbl.Caption:='-';
  ServerHardwareLbl.Caption:='-';

  ClientVersionLbl.Caption:='-';
  ClientNetworkNameLbl.Caption:='-';
  ClientIPLbl.Caption:='-';
  ClientOSLbl.Caption:='-';
  ClientHardwareLbl.Caption:='-';
end;

procedure TAdminServerInfoForm.Disconnected(var Message: TMessage);
begin
  ClearInfos;

  ServerStatusLbl.Caption:='Disconnected from Server.';
  ServiceStatusImg.Picture.Assign(ServiceStatusUnknownPNGImg);
end;

procedure TAdminServerInfoForm.Reconnected(var Message: TMessage);
begin
  GetInfos;

  ServerStatusLbl.Caption:='MySQL Server is running.';
  ServiceStatusImg.Picture.Assign(ServiceStatusRunningPNGImg);
end;

end.
