unit Main;

interface

{$include MySQLAdministrator.ini}

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, ImgList, Contnrs, Menus,
  ApplicationDataModule, InstanceDataModule, AdditionalClasses,
  AdminServerInfo, AdminServiceControl, AdminUsers,
  AdminStartupVariables, AdminServerHealth, AdminServerConnections,
  AdminCatalog, AdminBackup, AdminServerLogs, AdminReplication,
  AdminRestore,
  AuxFuncs, PNGImage, myx_public_interface, myx_admin_public_interface,
  Sections, Options, OptionsEditor, AdminOptionPages;

type
  TMainForm = class(TForm)
    StatusBar: TStatusBar;
    TopPnl: TPanel;
    Panel1: TPanel;
    Panel2: TPanel;
    MainMenu: TMainMenu;
    FileMI: TMenuItem;
    ConnecttoServerMI: TMenuItem;
    SaveConnectionMI: TMenuItem;
    N3: TMenuItem;
    ExportactivePagetoClipboard1: TMenuItem;
    N2: TMenuItem;
    CloseMI: TMenuItem;
    Edit1: TMenuItem;
    CopyMI: TMenuItem;
    CutMI: TMenuItem;
    PasteMI: TMenuItem;
    HelpMI: TMenuItem;
    OnlineHelpMI: TMenuItem;
    N6: TMenuItem;
    ReportBugMI: TMenuItem;
    VisitMySQLcom1: TMenuItem;
    N7: TMenuItem;
    AboutMI: TMenuItem;
    ManageConnectionsMI: TMenuItem;
    OptionsMI: TMenuItem;
    ToolsMI: TMenuItem;
    N8: TMenuItem;
    ViewMI: TMenuItem;
    CommandlineclientMI: TMenuItem;
    WindowsCommandLineMI: TMenuItem;
    N4: TMenuItem;

    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);

    procedure BuildSectionTree;
    function CreateSectionForm(AOwner: TComponent; SidebarSectionType: integer): TSectionForm;

    procedure ConnecttoServerMIClick(Sender: TObject);
    procedure CloseMIClick(Sender: TObject);

    procedure StatusBarDrawPanel(StatusBar: TStatusBar;
      Panel: TStatusPanel; const Rect: TRect);
    procedure StatusBarMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SaveConnectionMIClick(Sender: TObject);
    procedure PasteMIClick(Sender: TObject);
    procedure OptionsMIClick(Sender: TObject);
    procedure ManageConnectionsMIClick(Sender: TObject);
    procedure Disconnected(var Message: TMessage); message WM_Disconnected;
    procedure Reconnected(var Message: TMessage); message WM_Reconnected;

    function GetCommandLineClientPath: string;
    procedure CommandlineclientMIClick(Sender: TObject);
    procedure WindowsCommandLineMIClick(Sender: TObject);
    procedure ReportBugMIClick(Sender: TObject);
    procedure VisitMySQLcom1Click(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure OnlineHelpMIClick(Sender: TObject);
    procedure AboutMIClick(Sender: TObject);
  private
    { Private declarations }

    CatalogSection, BackupRestoreSection,
    UserAdministrationSection: TSidebarSection;

    PNGThreadStop: TPNGObject;
  public
    { Public declarations }
    InstanceDM: TInstanceDM;

    //DockPnl: TSectionPanel;

    SectionControls: TSectionControls;
  end;

const
    SSTAdmin_ServerInfo = 1;
    SSTAdmin_ServerConfig = 2;
    SSTAdmin_ServiceControl = 3;
    SSTAdmin_StartupVariables = 4;
    SSTAdmin_ServerConnections = 5;
    SSTAdmin_UserAdministration = 6;
    SSTAdmin_ServerHealth = 7;
    SSTAdmin_ServerLogs = 8;
    SSTAdmin_Backup = 9;
    SSTAdmin_Restore = 10;
    SSTAdmin_Replication = 11;
    SSTAdmin_Catalog = 12;

var
  MainForm: TMainForm;

implementation

uses About;

{$R *.dfm}

procedure TMainForm.FormCreate(Sender: TObject);
begin
  InitForm(self);

  InstanceDM:=TInstanceDM.Create(self, StatusBar);

  PNGThreadStop:=LoadPNGImageFromResource('thread_stop', nil);

  if(Screen.Width>=1024)then
  begin
    if(Screen.Height>=1024)then
    begin
      Width:=900;
      Height:=720;
    end
    else
    begin
      Width:=900;
      Height:=680;
    end;

    //Align center (till position is stored)
    Left:=(Screen.Width+Width) div 2 - Width;
    Top:=(Screen.Height+Height) div 2 - Height;
  end
  else
  begin
    Width:=800;
    Height:=572;

    Left:=(Screen.Width+Width) div 2 - Width;
    Top:=0;

    WindowState:=wsMaximized;
  end;


  SectionControls:=TSectionControls.Create(self,
    CreateSectionForm,
    ViewMI,
    ApplicationDM.Options.SectionSidebarHidden,
    ApplicationDM.Options.SectionSidebarWidth,
    ApplicationDM.SectionImageList,
    11);

  CommandlineclientMI.Enabled:=(GetCommandLineClientPath<>'');
  WindowsCommandLineMI.Enabled:=FileExists(GetSystemDir+'cmd.exe');
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  PNGThreadStop.Free;

  InstanceDM.Free;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
var i: integer;
begin
  if(HiWord(GetKeyState(VK_CONTROL))<>0)then
    ConnecttoServerMIClick(self);

  ApplicationDM.ApplicationIsTerminating:=True;

  //Close all Sub-windows
  for i:=0 to SectionControls.AdminTreeView.Items.Count-1 do
    if(SectionControls.AdminTreeView.Items[i].Data<>nil)then
      if(TSidebarSection(SectionControls.AdminTreeView.Items[i].Data).SectionForm<>nil)then
        TSidebarSection(SectionControls.AdminTreeView.Items[i].Data).SectionForm.Close;

  Action:=caFree;
end;

procedure TMainForm.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  //
end;

procedure TMainForm.ConnecttoServerMIClick(Sender: TObject);
begin
  CreateProz(Application.ExeName, '');
end;

procedure TMainForm.CloseMIClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.StatusBarDrawPanel(StatusBar: TStatusBar;
  Panel: TStatusPanel; const Rect: TRect);
begin
  if(Panel.Index=0)then
  begin
    if(InstanceDM.FetchingData)then
      PNGThreadStop.Draw(StatusBar.Canvas,
        Classes.Rect(Rect.Left+1, Rect.Top+1, Rect.Left+1+13, Rect.Top+1+13));
  end;
end;

procedure TMainForm.StatusBarMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if(X<20)and(InstanceDM.FetchingData)then
  begin
    InstanceDM.TerminateFetchingData;

    statusbar.Panels[1].Text:='';
    StatusBar.Invalidate;
  end;
end;

procedure TMainForm.PasteMIClick(Sender: TObject);
begin
  ShowMessage(IntToStr(GetFileSize('C:\Programme\MySQL\data\'+GetLocalHostName+'.err')));
end;

procedure TMainForm.OptionsMIClick(Sender: TObject);
var OptionsForm: TOptionsForm;
begin
  OptionsForm:=TOptionsForm.Create(self, TAdminOptionPagesForm.Create(self));
  try
    OptionsForm.ShowModal;
  finally
    OptionsForm.Free;
  end;
end;

procedure TMainForm.SaveConnectionMIClick(
  Sender: TObject);
var OptionsForm: TOptionsForm;
begin
  OptionsForm:=TOptionsForm.Create(self, TAdminOptionPagesForm.Create(self));
  try
    //Select Connections Page
    OptionsForm.ShowOptionPage(ConnectionsPage);
    
    OptionsForm.AddConnection(InstanceDM.user_connection);
    OptionsForm.ActiveControl:=OptionsForm.ConnectionEd;

    OptionsForm.ShowModal;
  finally
    OptionsForm.Free;
  end;
end;

procedure TMainForm.ManageConnectionsMIClick(Sender: TObject);
var OptionsForm: TOptionsForm;
begin
  OptionsForm:=TOptionsForm.Create(self, TAdminOptionPagesForm.Create(self));
  try
    //Select Connections Page
    OptionsForm.ShowOptionPage(ConnectionsPage);

    OptionsForm.ShowModal;
  finally
    OptionsForm.Free;
  end;
end;

procedure TMainForm.Disconnected(var Message: TMessage);
begin
  SectionControls.RefreshSidebarIcons(InstanceDM.Connected,
    InstanceDM.ConnectedToLocalhost);
end;

procedure TMainForm.Reconnected(var Message: TMessage);
begin
  SectionControls.RefreshSidebarIcons(InstanceDM.Connected,
    InstanceDM.ConnectedToLocalhost);
end;

function TMainForm.GetCommandLineClientPath: string;
begin
  if(FileExists(GetProgramFilesDir+'MySQL\bin\mysql.exe'))then
    Result:=GetProgramFilesDir+'MySQL\bin\mysql.exe'
  else if(FileExists('c:\mysql\bin\mysql.exe'))then
    Result:='c:\mysql\bin\mysql.exe'
  else if(FileExists('d:\mysql\bin\mysql.exe'))then
    Result:='d:\mysql\bin\mysql.exe'
  else
    Result:='';
end;

procedure TMainForm.CommandlineclientMIClick(Sender: TObject);
var cmd: string;
begin
  cmd:=GetCommandLineClientPath;
  if(cmd<>'')then
    CreateProz(cmd+' -h'+InstanceDM.user_connection.hostname+
      ' -u'+InstanceDM.user_connection.username+
      ' -p'+InstanceDM.user_connection.password+
      ' -P'+IntToStr(InstanceDM.user_connection.port), '');
end;

procedure TMainForm.WindowsCommandLineMIClick(Sender: TObject);
var cmd: string;
begin
  cmd:=GetSystemDir+'cmd.exe';
  if(FileExists(cmd))then
    CreateProz(cmd, GetHomeDir);
end;

procedure TMainForm.ReportBugMIClick(Sender: TObject);
begin
  CreateProz('explorer http://bugs.mysql.com', '');
end;

procedure TMainForm.VisitMySQLcom1Click(Sender: TObject);
begin
  CreateProz('explorer http://www.mysql.com', '');
end;

procedure TMainForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if(Key=VK_F1)then
    ShowHelp;
end;

procedure TMainForm.OnlineHelpMIClick(Sender: TObject);
begin
  ShowHelp;
end;

procedure TMainForm.AboutMIClick(Sender: TObject);
begin
  AboutForm:=TAboutForm.Create(self);
  try
    AboutForm.ShowModal;
  finally
    AboutForm.Free;
  end;
end;

procedure TMainForm.BuildSectionTree;
begin
  SectionControls.AdminTreeView.Items.Clear;

  //Create all AdminTree nodes and attach the corresponding action

  if(Not(ApplicationDM.Options.ShowOnlyServiceSections))then
  begin
    //Create Root Node
    AddTreeViewChildNode(SectionControls.AdminTreeView, nil, 'Server Information', 0,
      TSidebarSection.Create(SSTAdmin_ServerInfo,
        0, 1, True, False,
        SectionControls.AddViewMenuItem('ServerInformationMI', 'Server Information', 0)));
  end;

  //Create Service Control node
  AddTreeViewChildNode(SectionControls.AdminTreeView, nil, 'Service Control', 2,
    TSidebarSection.Create(SSTAdmin_ServiceControl,
      2, 3, False,
      Not(ApplicationDM.Options.ShowOnlyServiceSections),
      SectionControls.AddViewMenuItem('ServiceControlMI', 'Service Control',
        0+Ord(Not(ApplicationDM.Options.ShowOnlyServiceSections))),
      ApplicationDM.Options.ShowOnlyServiceSections
      ));

  //Create Server Variables node
  AddTreeViewChildNode(SectionControls.AdminTreeView, nil, 'Startup Variables', 4,
    TSidebarSection.Create(SSTAdmin_StartupVariables,
      4, 5, False,
      Not(ApplicationDM.Options.ShowOnlyServiceSections),
      SectionControls.AddViewMenuItem('StartupParametersMI', 'Startup Variables',
        1+Ord(Not(ApplicationDM.Options.ShowOnlyServiceSections))),
      ApplicationDM.Options.ShowOnlyServiceSections
      ));

  if(Not(ApplicationDM.Options.ShowOnlyServiceSections))then
  begin
    //Create User Administration nodes
    UserAdministrationSection:=
      TSidebarSection.Create(SSTAdmin_UserAdministration,
        8, 9, True, False,
        SectionControls.AddViewMenuItem('UserAdministrationMI', 'User Administration', 4),
        True);
    AddTreeViewChildNode(SectionControls.AdminTreeView,
      nil, 'User Administration', 8,
      UserAdministrationSection);

    //Create Server Connections node
    AddTreeViewChildNode(SectionControls.AdminTreeView, nil, 'Server Connections', 6,
      TSidebarSection.Create(SSTAdmin_ServerConnections,
        6, 7, True, False,
        SectionControls.AddViewMenuItem('ServerConnectionsMI', 'Server Connections', 3)));

    //Create Health node
    AddTreeViewChildNode(SectionControls.AdminTreeView, nil, 'Health', 10,
      TSidebarSection.Create(SSTAdmin_ServerHealth,
        10, 11, True, False,
        SectionControls.AddViewMenuItem('HealthMI', 'Health', 5)));
  end;

  //Create Server Logs node
  AddTreeViewChildNode(SectionControls.AdminTreeView, nil, 'Server Logs',
    12+Ord(Not(InstanceDM.ConnectedToLocalhost)),
    TSidebarSection.Create(SSTAdmin_ServerLogs,
      12, 13, False,
      Not(ApplicationDM.Options.ShowOnlyServiceSections),
      SectionControls.AddViewMenuItem('ServerLogsMI', 'Server Logs', 6)));

  if(Not(ApplicationDM.Options.ShowOnlyServiceSections))then
  begin
    //Create Replication Status node
    AddTreeViewChildNode(SectionControls.AdminTreeView, nil, 'Replication Status', 18,
      TSidebarSection.Create(SSTAdmin_Replication,
        18, 19, True, False,
        SectionControls.AddViewMenuItem('ReplicationStatusMI', 'Replication Status', 9)));

    //Create Backup node
    BackupRestoreSection:=TSidebarSection.Create(SSTAdmin_Backup,
      14, 15, True, False,
      SectionControls.AddViewMenuItem('BackupRestoreMI', 'Backup', 7),
      True);
    AddTreeViewChildNode(SectionControls.AdminTreeView,
      nil, 'Backup', 14,
      BackupRestoreSection);

    //Create Restore node
    AddTreeViewChildNode(SectionControls.AdminTreeView,
      nil, 'Restore', 16,
      TSidebarSection.Create(SSTAdmin_Restore,
      16, 17, True, False,
      SectionControls.AddViewMenuItem('RestoreMI', 'Restore', 8),
      True));

    //Create Catalogs node
    CatalogSection:=TSidebarSection.Create(SSTAdmin_Catalog,
      20, 21, True, False,
      SectionControls.AddViewMenuItem('CatalogsMI', 'Catalogs', 10),
      True);
    AddTreeViewChildNode(SectionControls.AdminTreeView,
      nil, 'Catalogs', 20,
      CatalogSection);
  end;

  SectionControls.RefreshSidebarIcons(InstanceDM.Connected,
    InstanceDM.ConnectedToLocalhost);

  if(ApplicationDM.Options.ShowOnlyServiceSections)then
    SaveConnectionMI.Enabled:=False;
end;


function TMainForm.CreateSectionForm(AOwner: TComponent;
  SidebarSectionType: integer): TSectionForm;
begin
  Result:=nil;

  case SidebarSectionType of
    SSTAdmin_ServerInfo:
      Result:=TAdminServerInfoForm.Create(AOwner,
            InstanceDM, StatusBar);
    SSTAdmin_ServiceControl:
      Result:=TAdminServiceControlForm.Create(AOwner,
          InstanceDM, StatusBar);
    SSTAdmin_StartupVariables:
      Result:=TAdminStartupVariablesForm.Create(AOwner,
          InstanceDM, StatusBar);
    SSTAdmin_ServerHealth:
      Result:=TAdminServerHealthForm.Create(AOwner,
          InstanceDM, StatusBar);
    SSTAdmin_ServerConnections:
      Result:=TAdminServerConnectionsForm.Create(AOwner,
          InstanceDM, StatusBar);
    SSTAdmin_UserAdministration:
      Result:=TAdminUsersForm.Create(AOwner,
          InstanceDM, StatusBar);
    SSTAdmin_Catalog:
      Result:=TAdminCatalogForm.Create(AOwner,
          InstanceDM, StatusBar);
    SSTAdmin_Backup:
      Result:=TAdminBackupForm.Create(AOwner,
          InstanceDM, StatusBar);
    SSTAdmin_Restore:
      Result:=TAdminRestoreForm.Create(AOwner,
          InstanceDM, StatusBar);
    SSTAdmin_ServerLogs:
      Result:=TAdminServerLogsForm.Create(AOwner,
          InstanceDM, StatusBar);
    SSTAdmin_replication:
      Result:=TAdminReplicationForm.Create(AOwner,
          InstanceDM, StatusBar);
  end;
end;

end.
