/* Copyright (C) 2003 MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#include <glib.h>

#include "myx_library.h"


// Windows includes
#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
#include <windows.h>
#else
// unix/linux includes

#include <sys/time.h>
#include <sys/select.h>
#include <errno.h>
#include <unistd.h>
#include <signal.h>
#include <sys/wait.h>
#include <sys/utsname.h> // uname()
#include <fcntl.h>

#define SIZE_T size_t

#endif

#ifdef __USE_POSIX
# include <sys/types.h>
# include <sys/stat.h>
# include <unistd.h>
#endif

#include "myx_aux_functions.h"


/**
 * Appends the @addon string to the end of
 * @base_str. @base_str is expected to have
 * been allocated with a glib-memory allocatio
 * function.
 *
 * @base_str becomes invalid after calling this
 * function!
 *
 * Return value: The joined string
 **/
char *myx_gstr_append(char *base_str, const char *addon)
{
  unsigned int addon_len= (unsigned int)strlen(addon);
  char *tmp;

  tmp= g_realloc(base_str, (unsigned int)strlen(base_str)+addon_len+1);

  base_str= strncat(tmp, addon,addon_len);
  
  return base_str;
}


//from libmysql/strmov.c
char *strmov(register char *dst, register const char *src)
{
  while ((*dst++ = *src++)) ;
  return dst-1;
}

int strcmp2(const char *str1, const char *str2)
{
  if(((!str1)&&(str2))||
     ((str1)&&(!str2)))
  {
    return -1;
  }
  else
  {
    return strcmp(str1, str2);
  }
}

unsigned int strlen2(const char *str)
{
  if(!str)
  {
    return 0;
  }
  else
  {
    return (unsigned int)strlen(str);
  }
}

char *strcpy2(char *dst, const char *src)
{
  if(!src)
  {
    *dst= 0;
  }
  else
  {
    strcpy(dst, src);
  }

  return dst;
}

char *str_trim(char *str)
{
  SIZE_T i, j;
  SIZE_T length;

  length= strlen(str);

  //Get length of leading spaces
  for (i=0; (i<length)&&isspace(str[i]); i++) 
  {}

  //Get length of trailing spaces
  for (j=length-1; (j>i)&&isspace(str[j]); j--)
  {}

  j++;
  
  //shift chars to the begining of the string
  memmove(str, str+i, j-i);

  //terminate string
  str[j-i]=0;

  return str;
}

char *str_left(char *dest, char *src, unsigned int len)
{
    unsigned int i;
    unsigned int s_len= (int)strlen(src);

    if(s_len<=len)
    {
        strcpy(dest, src);
    }
    else
    {
        for(i=0;i<len;i++)
        {
            dest[i]= src[i];
        }
        dest[len]= 0;
    }

    return dest;
}

char *str_right(char *dest, char *src, unsigned int len)
{
    unsigned int i;
    unsigned int s_len= (unsigned int)strlen(src);

    if(s_len<=len)
    {
        strcpy(dest, src);
    }
    else
    {
        for(i=0;i<=len;i++)
        {
            dest[i]= src[i+s_len-len];
        }
    }

    return dest;
}

int sub_str_count(char *search_str, const char *src)
{
    unsigned int i;
    char *pdest;
    char *psrc = (char *)src;
    unsigned int ss_len= (unsigned int)strlen(search_str);

    i= 0;
    while((pdest= strstr(psrc, search_str)))
    {
        i++;
        psrc= pdest+ss_len;
    }

    return i;
}


int str_endswith(const char *str, const char *substr)
{
  int slen= (int)strlen(str);
  int sslen= (int)strlen(substr);
  
  if (slen >= sslen && strcmp(str+slen-sslen, substr)==0)
      return 1;
  else
      return 0;
}


int str_beginswith(const char *str, const char *substr)
{
  return strncmp(str, substr, strlen(substr))==0;
}


int txtcmp(const char *str1, const char *str2)
{
  char *s1, *s2;
  int res;

  s1= str_toupper(strdup((char *) str1));
  s2= str_toupper(strdup((char *) str2));
  
  res= strcmp(s1,s2);

  free(s1);
  free(s2);

  return res;
}

char *str_toupper(char *str)
{
  char *s= str;

  while (*s) 
  {
    *s= toupper(*s);
    s++;
  }
  return str;
}


char *name_of_str(char *dst, const char *src)
{
  char *dst_start= dst;

  while((*(src))&&(*(src)!='='))
  {
    *dst++ = *src++;
  }

  *dst++ = 0;

  return dst_start;
}

char *value_of_str(char *dst, const char *src)
{
  char *dst_start= dst;
  char *psrc= (char *)src;

  psrc = strchr(psrc, '=');

  if(*psrc)
  {
    psrc++;
    while ((*dst++ = *psrc++)) ;
  }
  else
  {
    *dst= 0;
    
  }

  return dst_start;
}

int set_value(char **string_list, unsigned int string_list_num, char *name, char *new_value)
{
  unsigned int i;
  unsigned int name_len= (unsigned int)strlen(name);
  int rc= -1;

  for(i=0; i<string_list_num; i++)
  {
    if(strncmp(string_list[i], name, name_len)==0 && 
       (string_list[i][name_len]=='\0' || string_list[i][name_len]=='='))
    {
      string_list[i]= realloc(string_list[i], sizeof(char)*(name_len+strlen(new_value)+2));

      if (string_list[i][name_len]=='=') 
      {
        strcpy(string_list[i]+name_len+1, new_value);
      }
      else
      {
        sprintf(string_list[i], "%s=%s", string_list[i], new_value);
      }

      rc= 0;
      break;
    }
  }

  return rc;
}

#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)

int get_value_from_registry(HKEY root_key, const char *sub_key, const char *key, const char *def, char *value)
{
  HKEY hSubKey;
  DWORD dwType;
  DWORD dwSize;
  LONG retval;
  unsigned char Buffer[512];

  if((retval=RegOpenKeyEx(root_key, sub_key, 0, KEY_READ, &hSubKey))==ERROR_SUCCESS)
  {
    dwSize = 512;
    if((RegQueryValueEx(hSubKey, key, NULL, &dwType, Buffer, &dwSize))==ERROR_SUCCESS)
    {
      if(dwType==REG_DWORD)
      {
        sprintf(value, "%d", Buffer[0]+Buffer[1]*256+Buffer[2]*65536+Buffer[3]*4294967296);
      }
      else
      {
        strcpy(value, (const char *)Buffer);
      }
    }
    else
    {
      strcpy(value, def);
    }

    return 0;
  }
  else
  {
    strcpy(value, "");
    return retval;
  }
}

int set_value_to_registry(HKEY root_key, const char *sub_key, const char *key, const char *value)
{
  HKEY hSubKey;
  LONG retval;
  DWORD dwDispo;

  if((retval= RegCreateKeyEx(root_key, sub_key, 0, "",
    REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hSubKey, &dwDispo))==ERROR_SUCCESS)
  {
    retval = RegSetValueEx(hSubKey, key, 0, REG_SZ, value, (DWORD)strlen(value)+1);

    if(retval != ERROR_SUCCESS)
    {
       return GetLastError();
    } 
    else 
    {
      return 0;
    }
  }
  else
  {
    return -1;
  }
}

char *get_local_os_name()
{
  char *os_name = malloc(32);

  OSVERSIONINFO info;
  info.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
  GetVersionEx(&info);

  if (!os_name)
      return NULL;
  
  strcpy(os_name, "unknown");

  switch (info.dwPlatformId)
  {
    case VER_PLATFORM_WIN32_WINDOWS:
      if((info.dwMajorVersion>=4)&&(info.dwMinorVersion==0))
      {
        strcpy(os_name, "Windows 95");
      }
      else if((info.dwMajorVersion>=4)&&(info.dwMinorVersion>0)&&(info.dwMinorVersion>90))
      {
        strcpy(os_name, "Windows 98");
      }
      else if((info.dwMajorVersion>=4)&&(info.dwMinorVersion>0)&&(info.dwMinorVersion>=90))
      {
        strcpy(os_name, "Windows ME");
      }
      break;
    case VER_PLATFORM_WIN32_NT:
      if((info.dwMajorVersion<=4)&&(info.dwMinorVersion==0))
      {
        strcpy(os_name, "Windows NT");
      }
      else if((info.dwMajorVersion==5)&&(info.dwMinorVersion==0))
      {
        strcpy(os_name, "Windows 2k");
      }
      else if((info.dwMajorVersion==5)&&(info.dwMinorVersion>0))
      {
        strcpy(os_name, "Windows XP");
      }
      break;
  }

  return os_name;
}


char *get_local_hardware_info()
{
  char *hardware_string;
  SYSTEM_INFO sysinfo;
  MEMORYSTATUS memstat;
  int retval;
  char processor_name[100];
  char processor_mhz[10];
  char total_phys_ram[16];
  SIZE_T total_phys_ram_val;


  GetSystemInfo(&sysinfo);
  
  GlobalMemoryStatus(&memstat);

  strcpy(processor_mhz, "");
  strcpy(processor_name, "");

  retval= get_value_from_registry(HKEY_LOCAL_MACHINE, 
    "HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0", "~MHz", "", 
    processor_mhz);
  
  retval= get_value_from_registry(HKEY_LOCAL_MACHINE, 
    "HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0", "ProcessorNameString", "", 
    processor_name);

  if(!processor_name[0])
  {
    retval= get_value_from_registry(HKEY_LOCAL_MACHINE, 
      "HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0", "Identifier", "", 
      processor_name);

    sprintf(processor_name, "%s %s", processor_name, processor_mhz);
  }

  str_trim(processor_name);


  total_phys_ram_val= memstat.dwTotalPhys;
  if(total_phys_ram_val>=1072410624/1.9)
  {
    sprintf(total_phys_ram, "%1.1f GB RAM", (double)total_phys_ram_val/1072410624.0);
  }
  else if(total_phys_ram_val>=1047276/1.9)
  {
    sprintf(total_phys_ram, "%1.0f MB RAM", (double)total_phys_ram_val/1047276.0);
  }
  else
  {
    sprintf(total_phys_ram, "%d B RAM", total_phys_ram_val);
  }
   
  hardware_string = malloc(16 + strlen(processor_name)
                           + strlen(total_phys_ram));

  if(sysinfo.dwNumberOfProcessors>1) 
  {
    sprintf(hardware_string, "%dx %s, %s", sysinfo.dwNumberOfProcessors, processor_name, total_phys_ram);
  }
  else
  {
    sprintf(hardware_string, "%s, %s", processor_name, total_phys_ram);
  }

  return hardware_string;
}
#else /* !__WIN__ */

// Linux/Unix version

char *get_local_os_name()
{
  char *buf;
  struct utsname info;

  if (uname(&info) < 0)
  {
    return NULL;
  }
  
  buf = malloc(strlen(info.sysname)+strlen(info.release)+8);
  if (!buf)
  {
      return NULL;
  }

  sprintf(buf, "%s %s", info.sysname, info.release);

  return buf;
}


char *get_local_hardware_info()
{
  char *hardware_string;
  int processor_count = 0;
  char processor_name[256];
  char processor_mhz[128];
  char total_phys_ram[64];
  long total_phys_ram_val = 0;
  FILE *proc;
  char line[256];

  
 
  // fetch processor info from /proc/cpuinfo

  proc= fopen("/proc/cpuinfo", "r");
  if (!proc) 
  {
    return NULL;
  }
    
  while (!feof(proc)) 
  {
    if (!fgets(line, sizeof(line), proc))
        break;

    if (str_beginswith(line,"model name")) 
    {
      processor_count++;
      str_trim(strcpy(processor_name, strchr(line, ':')+1));
    } 
    else if (str_beginswith(line,"cpu MHz"))
    {
      str_trim(strcpy(processor_mhz, strchr(line, ':')+1));
    }
  }
  fclose(proc);
  

  // fetch physical memory info from /proc/meminfo
  proc= fopen("/proc/meminfo", "r");
  if (proc) 
  {
    char *ptr, *end;
    
    fgets(line, sizeof(line), proc); // header
    fgets(line, sizeof(line), proc); // meminfo
    fclose(proc);
    
    ptr= strchr(line, ':')+1;
    while (*ptr && *ptr==' ') ptr++;
    end= strchr(ptr, ' ');
    if (end)
        *end= 0;
    total_phys_ram_val = atoi(str_trim(ptr));
  }

  if(total_phys_ram_val>=1072410624/1.9)
  {
    sprintf(total_phys_ram, "%1.1f GB RAM", (double)total_phys_ram_val/1072410624.0);
  }
  else if(total_phys_ram_val>=1047276/1.9)
  {
    sprintf(total_phys_ram, "%1.0f MB RAM", (double)total_phys_ram_val/1047276.0);
  }
  else
  {
    sprintf(total_phys_ram, "%ld B RAM", total_phys_ram_val);
  }

  hardware_string= malloc(16+strlen(processor_name)+strlen(processor_mhz)+strlen(total_phys_ram));

  if(processor_count>1)
  {
    sprintf(hardware_string, "%dx %s %s MHz, %s", processor_count, processor_name, processor_mhz, total_phys_ram);
  }
  else
  {
    sprintf(hardware_string, "%s %s MHz, %s", processor_name, processor_mhz, total_phys_ram);
  }

  return hardware_string;
}



FILE *myx_popen(char *const args[], pid_t *pid_ret)
{
  FILE *f;
  int fd[2];
  
  if (pipe(fd) >= 0)
  {
    *pid_ret = fork();
    if (*pid_ret == 0) 
    {
      close(1);
      close(2);
      dup2(fd[1], 1);
      dup2(fd[1], 2);

      close(fd[0]);

      execvp(args[0], args);
      exit(-1);
    }
    else if (*pid_ret > 0)
    {
      close(fd[1]);

      f = fdopen(fd[0], "r");
      if (!f)
      {
        int ret;
        kill(*pid_ret, 9);
        waitpid(*pid_ret, &ret, 0);
        close(fd[0]);
      }
      return f;
    }
  }

  return NULL;
}


int myx_read_timeout(FILE *f, int timeout, char *result, size_t result_len)
{
  fd_set fds;
  struct timeval tv;
  int ret;
  
  FD_ZERO(&fds);
  FD_SET(fileno(f), &fds);

  tv.tv_sec = timeout / 1000;
  tv.tv_usec = (timeout % 1000)*1000;
  
  do {
    ret= select(fileno(f)+1, &fds, NULL, NULL, &tv);
  } while (ret < 0 && errno == EINTR);
  
  if (ret > 0)
  {
    if (fgets(result, result_len, f))
        return strlen(result);
  }
  return -1;
}


int myx_pclose(FILE *f, pid_t pid)
{
  int status = -1;

  if (kill(pid, SIGKILL) == 0)
      waitpid(pid, &status, 0);
  fclose(f);

  return status;
}

#endif /* !__WIN__ */

#ifdef __USE_POSIX
int copy_file(const char *orig_file, const char *new_file)
{
  char buffer[4*1024];
  int from_fd, to_fd;
  int count;
  
  from_fd= open(orig_file, O_RDONLY, 0666);
  if (from_fd < 0)
    return -1;
  to_fd= open(new_file, O_WRONLY|O_CREAT, 0666);
  if (to_fd < 0)
  {
    close(from_fd);
    return -1;
  }
  
  do {
    count= read(from_fd, buffer, sizeof(buffer));
    if (count < 0)
    {
      close(from_fd);
      close(to_fd);
      return -1;
    }
    if (count > 0 && write(to_fd, buffer, count) < 0)
    {
      close(from_fd);
      close(to_fd);
      return -1;
    }
  } while (count > 0);

  close(from_fd);
  close(to_fd);

  return 0;
}
#endif


bigint get_file_size(const char *filename)
{
#ifdef __USE_POSIX
  struct stat buf;
  if (stat(filename, &buf) < 0)
    return -1;	  
  return buf.st_size;
#endif

#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
  DWORD dwSizeLow;
  DWORD dwSizeHigh = 0;
  HANDLE hfile;
  
  hfile = CreateFile(filename, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL,
                     OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

  if (hfile != INVALID_HANDLE_VALUE) 
  {
    dwSizeLow= GetFileSize(hfile, &dwSizeHigh);

    CloseHandle(hfile);

    if((dwSizeLow==INVALID_FILE_SIZE)&&(GetLastError()) != NO_ERROR )
    { 
      return -1;
    }
    else
    {
      return ((dwSizeHigh << 32) + dwSizeLow);
    }
  }
  else
  {
    return -1;
  }
#endif

}


