/* Copyright (C) 2003 MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <glib.h>

#include "myx_library.h"

/*
 *public functions definitions
 */

MYX_CATALOGS * myx_get_catalogs(MYSQL *mysql)
{
  MYX_CATALOGS *catalogs= g_malloc(sizeof(MYX_CATALOGS));
  MYX_CATALOG *catalog;
  //MYX_CATALOG *sys_catalog;
  int schema_index;
  MYSQL_RES *res;
  MYSQL_ROW row;

  if(myx_get_mysql_major_version(mysql)<6)
  {
    catalogs->catalogs_num= 1;
    catalogs->catalogs= g_malloc(sizeof(MYX_CATALOG)*catalogs->catalogs_num);

    catalog= catalogs->catalogs;
    catalog->catalog_name= g_strdup("def");

    if(!(res= mysql_list_dbs(mysql, (char*) NULL)))
    {
      return NULL;
    }

    catalog->schemata_num= (unsigned int)mysql_num_rows(res);
    catalog->schemata= g_malloc(sizeof(MYX_SCHEMA)*catalogs->catalogs->schemata_num);
    schema_index= 0;

    while((row= mysql_fetch_row(res)))
    {
      /*if(!strcmp(row[0], "mysql"))
      {
        catalog->schemata_num--;
        catalog->schemata= g_realloc(catalog->schemata, sizeof(MYX_SCHEMA)*catalogs->catalogs->schemata_num);

        catalogs->catalogs_num++;
        catalogs->catalogs= g_realloc(catalogs->catalogs, sizeof(MYX_CATALOG)*catalogs->catalogs_num);
        catalog= catalogs->catalogs;
        
        sys_catalog= (MYX_CATALOG *)(catalogs->catalogs+1);

        sys_catalog->catalog_name= g_strdup("sys");

        sys_catalog->schemata_num= 1;
        sys_catalog->schemata= g_malloc(sizeof(MYX_SCHEMA));

        sys_catalog->schemata->schema_tables= NULL;

        sys_catalog->schemata->schema_name= g_strdup(row[0]);

        sys_catalog->schemata->catalog_name= g_strdup(sys_catalog->catalog_name);
      }
      else
      {*/
        MYX_SCHEMA *schema= catalog->schemata + schema_index;
        
        schema->schema_tables= NULL;

        schema->schema_name= g_strdup(row[0]);

        schema->catalog_name= g_strdup(catalog->catalog_name);

        schema_index++;
      //}
    }

    mysql_free_result(res);

    return catalogs;
  }
  else
  {
    catalogs->catalogs_num= 0;

    return catalogs;
  }
}

MYX_SCHEMA_TABLES * myx_get_schema_tables(MYSQL *mysql, const char *catalog_name, const char *schema_name)
{
  MYX_SCHEMA_TABLES *schema_tables= g_malloc(sizeof(MYX_SCHEMA_TABLES));
  MYX_SCHEMA_TABLE *schema_table;
  MYX_SCHEMA_TABLE_COLUMN *schema_table_column;
  MYSQL_RES *res, *res2;
  MYSQL_ROW row;
  char *sqlcmd;
  unsigned int i= 0, j;

  schema_tables->schema_tables_num= 0;
  schema_tables->schema_tables= NULL;

  if(mysql_select_db(mysql, schema_name))
  {
    myx_free_schema_tables(schema_tables);
    return NULL;
  }

  if((res= mysql_list_tables(mysql, (char*) NULL)))
  {
    schema_tables->schema_tables_num= (unsigned int)mysql_num_rows(res);
    schema_tables->schema_tables= g_malloc0(sizeof(MYX_SCHEMA)*schema_tables->schema_tables_num);

    while((row= mysql_fetch_row(res)))
    {
      schema_table= schema_tables->schema_tables+i;

      schema_table->table_name= g_strdup(row[0]);

      //schema_table->table_status= NULL;

      sqlcmd= g_strdup_printf("SHOW COLUMNS FROM `%s`", schema_table->table_name);

      if (!(myx_mysql_query(mysql, sqlcmd)))
      {
        if ((res2= mysql_store_result(mysql)))
        {
          schema_table->columns_num= (unsigned int)mysql_num_rows(res2);
          schema_table->columns= g_malloc0(sizeof(MYX_SCHEMA_TABLE_COLUMN)*schema_table->columns_num);

          j= 0;
          while((row= mysql_fetch_row(res2)))
          {
            schema_table_column= schema_table->columns+j;

            schema_table_column->column_name= g_strdup(row[0]);

            schema_table_column->column_type= g_strdup(row[1]);
            schema_table_column->default_value= g_strdup(row[4]);
            schema_table_column->extra= g_strdup(row[5]);
            
            if (!(strcmp2(row[2], "YES")))
            {
              schema_table_column->is_null_allowed= 1;
            }
            else
            {
              schema_table_column->is_null_allowed= 0;
            }
            if (!(strcmp2(row[3], "PRI")))
            {
              schema_table_column->is_primary_key= 1;
            }
            else
            {
              schema_table_column->is_primary_key= 0;
            }
            
            j++;
          }
          mysql_free_result(res2);
        }
        else
        {
          g_free(sqlcmd);
          myx_free_schema_tables(schema_tables);
          return NULL;
        }
      }
      else
      {
        g_free(sqlcmd);
        myx_free_schema_tables(schema_tables);
        return NULL;
      }

      g_free(sqlcmd);

      i++;
    }

    mysql_free_result(res);
  }
  else
  {
    myx_free_schema_tables(schema_tables);
    return NULL;
  }

  return schema_tables;
}

MYX_TABLE_INDEX_COLUMN * copy_index_columns(MYX_TABLE_INDEX_COLUMN *src_columns, unsigned int count)
{
  unsigned int i;
  MYX_TABLE_INDEX_COLUMN *index_columns= g_malloc(sizeof(MYX_TABLE_INDEX_COLUMN)*count);
  MYX_TABLE_INDEX_COLUMN *current_col, *src_col;

#define XSTRDUP(s) g_strdup(s?s:"")
  for(i=0;i<count;i++)
  {
    current_col= index_columns+i;
    src_col= src_columns+i;
    current_col->column_name= XSTRDUP(src_col->column_name);
    current_col->collation= XSTRDUP(src_col->collation);
    current_col->seq_in_index= XSTRDUP(src_col->seq_in_index);

    g_free((MYX_TABLE_INDEX_COLUMN *)(src_columns+i)->column_name);
    g_free((MYX_TABLE_INDEX_COLUMN *)(src_columns+i)->collation);
    g_free((MYX_TABLE_INDEX_COLUMN *)(src_columns+i)->seq_in_index);
  }
#undef XSTRDUP

  return index_columns;
}

MYX_SCHEMA_TABLE_STATUS * myx_get_schema_table_status(MYSQL *mysql, const char *catalog_name, const char *schema_name)
{
  MYX_SCHEMA_TABLE_STATUS *schema_tables= g_malloc0(sizeof(MYX_SCHEMA_TABLE_STATUS));
  MYX_TABLE_STATUS *schema_table;
  MYX_TABLE_INDEX *table_index;
  MYX_SCHEMA_TABLE_COLUMN *table_column;

  MYSQL_RES *res, *res2;
  MYSQL_ROW row, row2;
  MYSQL_FIELD *fields, *fields2;
  unsigned int num_fields, num_fields2;
  char *sqlcmd, *sqlcmd2;
  char *current_index_name;
  unsigned int i= 0, j, k;
  unsigned int index_name_pos= 2;

  MYX_TABLE_INDEX_COLUMN *tmp_index_columns= g_malloc0(sizeof(MYX_TABLE_INDEX_COLUMN)*50);

  if(mysql_select_db(mysql, schema_name))
  {
    g_free(schema_tables);
    return NULL;
  }


  sqlcmd= "SHOW TABLE STATUS";

  if (!(myx_mysql_query(mysql, sqlcmd)))
  {
    if ((res= mysql_store_result(mysql)))
    {
      schema_tables->schema_tables_num= (unsigned int)mysql_num_rows(res);
      schema_tables->schema_tables= g_malloc0(sizeof(MYX_TABLE_STATUS)*schema_tables->schema_tables_num);

      //Depending on the version of the server there might be different columns
      num_fields = mysql_num_fields(res);
      fields = mysql_fetch_fields(res);

      while((row= mysql_fetch_row(res)))
      {
        schema_table= schema_tables->schema_tables+i;
        schema_table->table_name= NULL;
        schema_table->table_type= NULL;
        schema_table->row_format= NULL;
        schema_table->rows= NULL;
        schema_table->avg_row_length= NULL;
        schema_table->data_length= NULL;
        schema_table->max_data_length= NULL;
        schema_table->index_length= NULL;
        schema_table->data_free= NULL;
        schema_table->auto_increment= NULL;
        schema_table->create_time= NULL;
        schema_table->update_time= NULL;
        schema_table->check_time= NULL;
        schema_table->create_options= NULL;
        schema_table->comment= NULL;

#define XSTRDUP(s) g_strdup(s?s:"")
        for(k=0;k<num_fields;k++)
        {
          if (strcmp(fields[k].name, "Name")==0)
          {
            schema_table->table_name= myx_convert_dbstr_utf8(mysql, row[k]);
          }
          else if (strcmp(fields[k].name, "Type")==0)
          {
            schema_table->table_type= XSTRDUP(row[k]);
          }
          else if (strcmp(fields[k].name, "Row_format")==0)
          {
            schema_table->row_format= XSTRDUP(row[k]);
          }
          else if (strcmp(fields[k].name, "Rows")==0)
          {
            schema_table->rows= XSTRDUP(row[k]);
          }
          else if (strcmp(fields[k].name, "Avg_row_length")==0)
          {
            schema_table->avg_row_length= XSTRDUP(row[k]);
          }
          else if (strcmp(fields[k].name, "Data_length")==0)
          {
            schema_table->data_length= XSTRDUP(row[k]);
          }
          else if (strcmp(fields[k].name, "Max_data_length")==0)
          {
            schema_table->max_data_length= XSTRDUP(row[k]);
          }
          else if (strcmp(fields[k].name, "Index_length")==0)
          {
            schema_table->index_length= XSTRDUP(row[k]);
          }
          else if (strcmp(fields[k].name, "Data_free")==0)
          {
            schema_table->data_free= XSTRDUP(row[k]);
          }
          else if (strcmp(fields[k].name, "Auto_increment")==0)
          {
            schema_table->auto_increment= XSTRDUP(row[k]);
          }
          else if (strcmp(fields[k].name, "Create_time")==0)
          {
            schema_table->create_time= XSTRDUP(row[k]);
          }
          else if (strcmp(fields[k].name, "Update_time")==0)
          {
            schema_table->update_time= XSTRDUP(row[k]);
          }
          else if (strcmp(fields[k].name, "Check_time")==0)
          {
            schema_table->check_time= XSTRDUP(row[k]);
          }
          else if (strcmp(fields[k].name, "Create_options")==0)
          {
            schema_table->create_options= XSTRDUP(row[k]);
          }
          else if (strcmp(fields[k].name, "Comment")==0)
          {
            schema_table->comment= XSTRDUP(row[k]);
          }
        }

        sqlcmd2= g_strdup_printf("SHOW INDEX FROM `%s`", schema_table->table_name);
        if (!(myx_mysql_query(mysql, sqlcmd2))
            && (res2= mysql_store_result(mysql)))
        {
          schema_table->indexes_num= (unsigned int)mysql_num_rows(res2);
          schema_table->indexes= g_malloc0(sizeof(MYX_TABLE_INDEX)*schema_table->indexes_num);
          j= 0;

          //Depending on the version of the server there might be different columns
          num_fields2 = mysql_num_fields(res2);
          fields2 = mysql_fetch_fields(res2);

          for(k=0;k<num_fields2;k++)
          {
            if((strcmp(fields2[k].name, "Key_name")==0))
            {
              index_name_pos= k;
              break;
            }
          }

          current_index_name= NULL;

          while ((row2= mysql_fetch_row(res2)))
          {
            if(strcmp2(current_index_name, row2[index_name_pos])!=0)
            {
              if(current_index_name)
              {
                table_index->index_columns= copy_index_columns(tmp_index_columns, table_index->index_columns_num);
              }

              table_index= schema_table->indexes + j;
              table_index->index_columns_num= 1;
              table_index->index_columns= NULL;

              table_index->is_non_unique= 0;
              table_index->key_name= NULL;
              table_index->is_null_allowed= 0;
              table_index->index_type= NULL;

              tmp_index_columns[0].column_name= NULL;
              tmp_index_columns[0].seq_in_index= NULL;
              tmp_index_columns[0].collation= NULL;

              for(k=0;k<num_fields2;k++)
              {
                if((strcmp(fields2[k].name, "Non_unique")==0))
                {
                  table_index->is_non_unique= atoi(row2[k]);
                }
                else if((strcmp(fields2[k].name, "Key_name")==0))
                {
                  table_index->key_name= myx_convert_dbstr_utf8(mysql, row2[k]);
                }
                else if((strcmp(fields2[k].name, "Null")==0))
                {
                  table_index->is_null_allowed= strcmp2(row[k],"YES")==0;
                }
                else if((strcmp(fields2[k].name, "Index_type")==0))
                {
                  table_index->index_type= XSTRDUP(row2[k]);
                }
                else if((strcmp(fields2[k].name, "Column_name")==0))
                {
                  tmp_index_columns[0].column_name= myx_convert_dbstr_utf8(mysql, row2[k]);
                }
                else if((strcmp(fields2[k].name, "Seq_in_index")==0))
                {
                  tmp_index_columns[0].seq_in_index= XSTRDUP(row2[k]);
                }
                else if((strcmp(fields2[k].name, "Collation")==0))
                {
                  tmp_index_columns[0].collation= XSTRDUP(row2[k]);
                }
              }

              j++;
              current_index_name= table_index->key_name;
            }
            else
            {
              for(k=0;k<num_fields2;k++)
              {
                if((strcmp(fields2[k].name, "Column_name")==0))
                {
                  tmp_index_columns[table_index->index_columns_num].column_name= XSTRDUP(row2[k]);
                }
                else if((strcmp(fields2[k].name, "Seq_in_index")==0))
                {
                  tmp_index_columns[table_index->index_columns_num].seq_in_index= XSTRDUP(row2[k]);
                }
                else if((strcmp(fields2[k].name, "Collation")==0))
                {
                  tmp_index_columns[table_index->index_columns_num].collation= XSTRDUP(row2[k]);
                }
              }

              table_index->index_columns_num++;
            }
          }
          mysql_free_result(res2);

          if(j>0)
          {
            table_index->index_columns= copy_index_columns(tmp_index_columns, table_index->index_columns_num);
          }

          schema_table->indexes_num= j;
        }
        else
        {
          g_free(sqlcmd2);
          g_free(tmp_index_columns);
          myx_free_schema_table_status(schema_tables);
          return NULL;
        }
        g_free(sqlcmd2);

        schema_table->columns_num= 0;
        schema_table->columns= NULL;

        sqlcmd2= g_strdup_printf("DESCRIBE `%s`", schema_table->table_name);
        if (!(myx_mysql_query(mysql, sqlcmd2))
            && (res2= mysql_store_result(mysql)))
        {
          //Depending on the version of the server there might be different columns
          num_fields2 = mysql_num_fields(res2);
          fields2 = mysql_fetch_fields(res2);

          schema_table->columns_num= (unsigned int)mysql_num_rows(res2);
          schema_table->columns= g_malloc0(sizeof(MYX_SCHEMA_TABLE_COLUMN)*schema_table->columns_num);
          j= 0;

          while ((row2= mysql_fetch_row(res2)))
          {
            table_column= schema_table->columns + j;

            for(k=0;k<num_fields2;k++)
            {
              if (strcmp(fields2[k].name, "Field")==0)
              {
                table_column->column_name= myx_convert_dbstr_utf8(mysql, row2[k]);
              }
              else if (strcmp(fields2[k].name, "Type")==0)
              {
                table_column->column_type= XSTRDUP(row2[k]);
              }
              else if (strcmp(fields2[k].name, "Null")==0)
              {
                table_column->is_null_allowed= strcmp2(row2[k],"YES")==0;
              }
              else if (strcmp(fields2[k].name, "Key")==0)
              {
                table_column->is_primary_key= strcmp2(row2[k],"PRI")==0;
              }
              else if (strcmp(fields2[k].name, "Default")==0)
              {
                table_column->default_value= myx_convert_dbstr_utf8(mysql, row2[k]);
              }
              else if (strcmp(fields2[k].name, "Extra")==0)
              {
                table_column->extra= XSTRDUP(row2[k]);
              }
            }
            j++;
          }
          mysql_free_result(res2);
        }
        else
        {
          g_free(sqlcmd2);
          g_free(tmp_index_columns);
          myx_free_schema_table_status(schema_tables);
          return NULL;
        }
        g_free(sqlcmd2);
#undef XSTRDUP

        i++;
      }
      mysql_free_result(res);
    }
    else
    {
      g_free(tmp_index_columns);
      myx_free_schema_table_status(schema_tables);
      return NULL;
    }
  }
  else
  {
    g_free(tmp_index_columns);
    myx_free_schema_table_status(schema_tables);
    return NULL;
  }

  return schema_tables;
}


int myx_free_catalogs(MYX_CATALOGS *catalogs)
{
  unsigned int i, j;

  for(i=0;i<catalogs->catalogs_num;i++)
  {
    g_free((MYX_CATALOG *)(catalogs->catalogs+i)->catalog_name);

    for(j=0;j<((MYX_CATALOG *)(catalogs->catalogs+i))->schemata_num;j++)
    {
      myx_free_schema_tables((MYX_SCHEMA_TABLES *)(((MYX_SCHEMA *)(((MYX_CATALOG *)(catalogs->catalogs+i))->schemata+j))->schema_tables));

      g_free((MYX_SCHEMA *)(((MYX_CATALOG *)(catalogs->catalogs+i))->schemata+j)->schema_name);
      g_free((MYX_SCHEMA *)(((MYX_CATALOG *)(catalogs->catalogs+i))->schemata+j)->catalog_name);
    }
    g_free((MYX_CATALOG *)(catalogs->catalogs+i)->schemata);
  }
  g_free(catalogs->catalogs);
  g_free(catalogs);

  return 0;
}

int myx_free_schema_column(MYX_SCHEMA_TABLE_COLUMN *schema_column)
{
  g_free(schema_column->column_name);
  g_free(schema_column->column_type);
  g_free(schema_column->default_value);
  g_free(schema_column->extra);

  return 0;
}

int free_index_column(MYX_TABLE_INDEX_COLUMN *schema_column)
{
  g_free(schema_column->column_name);
  g_free(schema_column->seq_in_index);
  g_free(schema_column->collation);

  return 0;
}

int myx_free_schema_index(MYX_TABLE_INDEX *schema_index)
{
  unsigned int i;

  g_free(schema_index->key_name);
  g_free(schema_index->index_type);

  for(i=0;i<schema_index->index_columns_num;i++)
  {
    free_index_column(schema_index->index_columns+i);
  }
  g_free(schema_index->index_columns);

  return 0;
}

int myx_free_schema_tables(MYX_SCHEMA_TABLES *schema_tables)
{
  MYX_SCHEMA_TABLE *schema_table;
  MYX_SCHEMA_TABLE_COLUMN *schema_table_column;
  unsigned int i, j;

  if (schema_tables)
  {
    for(i=0;i<schema_tables->schema_tables_num;i++)
    {
      schema_table= schema_tables->schema_tables+i;
      for(j=0;j<schema_table->columns_num;j++)
      {
        schema_table_column= schema_table->columns+j;

        myx_free_schema_column(schema_table_column);
      }
      g_free(schema_table->columns);
    }
    g_free(schema_tables->schema_tables);
    g_free(schema_tables);
  }

  return 0;
}

int myx_free_schema_table_status_struct(MYX_TABLE_STATUS *schema_table)
{
  unsigned int i;

  g_free(schema_table->auto_increment);
  g_free(schema_table->avg_row_length);
  g_free(schema_table->check_time);
  g_free(schema_table->comment);
  g_free(schema_table->create_options);
  g_free(schema_table->create_time);
  g_free(schema_table->data_free);
  g_free(schema_table->data_length);
  g_free(schema_table->index_length);
  g_free(schema_table->max_data_length);
  g_free(schema_table->row_format);
  g_free(schema_table->rows);
  g_free(schema_table->table_name);
  g_free(schema_table->table_type);
  g_free(schema_table->update_time);

  for(i=0;i<schema_table->indexes_num;i++)
  {
    myx_free_schema_index(schema_table->indexes+i);
  }
  g_free(schema_table->indexes);

  for(i=0;i<schema_table->columns_num;i++)
  {
    myx_free_schema_column(schema_table->columns+i);
  }
  g_free(schema_table->columns);

  return 0;
}

int myx_free_schema_table_status(MYX_SCHEMA_TABLE_STATUS *schema_tables)
{
  unsigned int i;

  if (schema_tables)
  {
    for(i=0;i<schema_tables->schema_tables_num;i++)
    {
      myx_free_schema_table_status_struct((MYX_TABLE_STATUS *)(schema_tables->schema_tables+i));
    }
    g_free(schema_tables->schema_tables);

    g_free(schema_tables);
  }

  return 0;
}
