/* Copyright (C) 2003 MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */


/*
 *
 * (XML Load/Save) Helper Functions
 *
 *
 */

#include "myx_xml_aux_functions.h"
#include <string.h>
#include <glib.h>

char* int_to_str(int number)
{
    int i;
    char *result;
    int orig_number = number;

    i=1;
    number = number/10;
    while (number > 0)
    {
        i++;
	number = number/10;
    }
    result = malloc(sizeof(char) * i +10);
    sprintf(result, "%d", (int)orig_number);
    return result;
}

char* bool_to_str(myx_bool b)
{
    char *result = malloc(2);

    if (b)
        result[0]= '1';
    else
        result[0]= '0';

    result[1] = '\0';
    return result;
}

int atoi_and_free(xmlChar *string)
{
    int a = 0;

    if (string != NULL) {
        a = atoi(string);
        xmlFree(string);
    }
    return a;
}

double atof_and_free(xmlChar *string)
{
    double a = 0.0;

    if (string != NULL) {
        a = atof(string);
        xmlFree(string);
    }
    return a;
}

/*we just skip all characters above 127 and return safe ascii-chars only*/
unsigned char* get_failsafe_utf8(const unsigned char* latin_str)
{
    int i,j;
    unsigned char *result;
    long len;
    
    len = (long)strlen(latin_str);
    result = malloc(sizeof(char) * len +1); 


    j=0;
    for (i=0; i < len; i++)
    {
        if (latin_str[i] <= 127) 
        {
            result[j++] = latin_str[i];
        }

    }
    result[j] = '\0';

    if (j < i)
    {
        realloc(result, sizeof(char) *j +1);
    }

    return result;
}

/*checks if filename really exists*/
int file_exists(const unsigned char *filename)
{
    FILE *user_conns_file;
    /*check if we can open the file */    
    if ((user_conns_file=fopen(filename,"r")) == NULL)
    {
        return 0;
    } 
    else
    {
        fclose(user_conns_file);
        return 1;
    }
}


xmlNodePtr NewTextChild_int_content(xmlNodePtr parent,xmlNsPtr ns, const xmlChar *name, int content_int)
{
    xmlNodePtr node;

    char* content = int_to_str(content_int);
    node = xmlNewTextChild(parent, ns, name, content);

    free(content);
    return node;
}


xmlNodePtr NewTextChild_double_content(xmlNodePtr parent,xmlNsPtr ns, const xmlChar *name, double content_double)
{
    xmlNodePtr node;

    char* content =  g_strdup_printf("%f", content_double);
    node = xmlNewTextChild(parent, ns, name, content);

    g_free(content);
    return node;
}

/* returns the number of children of the specified type the parent-node has*/
int get_child_count(const xmlNodePtr parent,const xmlChar *element_type_name)
{
    xmlNodePtr cur;
    int node_count = 0;

    cur = parent->children;
    while (cur != NULL) 
    {        
        if ( !xmlStrcmp(cur->name, element_type_name) && cur->type == XML_ELEMENT_NODE)
            node_count++;

        cur = cur->next;
    }

    return node_count;
}

