/* Copyright (C) 2004 MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */


#ifndef _MGPREFERENCESEDITOR_H_
#define _MGPREFERENCESEDITOR_H_

#include <gtkmm.h>

#include <list>

#include "MGPreferences.h"
#include "MGGladeXML.h"

#include "MYXInterface.h"
#include "myg_utils.h"

using MYX::UserConnection;
using MYX::UserConnectionList;

class MGPreferencesEditor : public Gtk::Window {    
    class ConnectionColumns : public Gtk::TreeModel::ColumnRecord {
      public:
        ConnectionColumns() { add(_icon); add(_text); add(_is_sep); add(_data); };

        Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > _icon;
        Gtk::TreeModelColumn<Glib::ustring> _text;
        Gtk::TreeModelColumn<bool> _is_sep;
        Gtk::TreeModelColumn<UserConnection> _data;
    };
    ConnectionColumns _connection_columns;

    MGPreferences *_prefs;
    UserConnectionList *_connections;

    MGGladeXML *_xml;

    Glib::ustring _heading;
    bool _dirty;

    Glib::RefPtr<Gdk::Pixbuf> _conn_group_icon;
    Glib::RefPtr<Gdk::Pixbuf> _conn_icon;

    Glib::RefPtr<Gtk::TreeStore> _connection_store;
    bool _connection_ignore_changes;

    Gtk::Widget *get_widget(const std::string &name) { return _xml->get_widget(name); };

    void init();
    void update();

    void update_sensitivity();
    void set_dirty();
    void unset_dirty();

    bool close_window(GdkEventAny *ev);
    void discard_clicked();
    void apply_clicked();
    void close_clicked();

    void menu_changed(int page);

    void init_general();
    void init_connections();
    void init_administrator();

    void update_general();
    void update_connections();
    void update_administrator();

    void commit_general();
    void commit_connections();
    void commit_administrator();

    bool expose_heading(GdkEventExpose *expose);

    void general_pw_store_clicked();

    void connection_add();
    void connection_remove();
    void connection_show();
    void connection_data_changed(const char *field);

    ~MGPreferencesEditor();

  public:
    MGPreferencesEditor(GtkWindow *widget);
    static MGPreferencesEditor *create(MGPreferences *prefs);

    virtual void show();
};


#endif /* _MGPREFERENCESEDITOR_H_ */
