unit AuxFuncs;

interface

uses Classes, ExtCtrls, Windows, Winsock, Registry, ComCtrls, Forms,
  StdCtrls, Controls, Dialogs, ShlObj, Contnrs,
  SysUtils, ShellAPI, WinSvc, ActiveX,
  PNGImage;

type
  MYX_MESSAGE_DLG_TYPE = (
    myx_mtError=1, myx_mtInformation, myx_mtConfirmation, myx_mtEdit
  );

  MYX_NODE_DATA_TYPE = (
    myx_ndt_pointer=1, myx_ndt_tobject
  );

  PMYX_DRIVE_INFO = ^MYX_DRIVE_INFO;
  MYX_DRIVE_INFO = Record
    VolumeLabel: string;
    VolumeNumber: string;
    FileSystem: string;
    DriveType: string;
    TotalSize: int64;
    FreeSpace: int64;
  end;

  TMyxModalDialog = Class(TForm)
    MessageLbl: TLabel;
    ValueMemo: TMemo;
    DlgBtnList: TList;

    constructor Create(Titel: string; Messagetext: string;
      DlgType: MYX_MESSAGE_DLG_TYPE;
      buttons: string;
      doEdit: Boolean; editCaption: string; var editValue: string;
      editLinesNum: integer = 1; editReadOnly: Boolean = False); reintroduce;
    destructor Destroy; override;

    procedure doButtonClick(Sender: TObject);
    procedure HideEdit;
    procedure ShowEdit;
  protected
    EditLbl: TLabel;
    Image: TImage;
  end;

  TMySQLService = class(TObject)
    ExistingService: Boolean;
    StartType: integer;
    ServiceName: string;
    DisplayName: string;
    Description: string;
    ConfigFile: string;
    ConfigFileSection: string;
    ServerType: string;
    PathToBinary: string;
    Status: integer;
    ErrorLog: string;
    logfile_size: Longword;
  end;

function LoadPNGImageFromResource(Resourcename: string; Image: TImage = nil): TPNGObject;
function getXPStyleEnabled: Boolean;

function AddTreeViewChildNode(theTreeView: TTreeView;
  ParentNode: TTreeNode; Caption: string; Icon: integer;
  Data: Pointer = nil): TTreeNode;
function AddListViewItem(theListView: TListView;
  ParentItem: TListItem; Caption: string; Icon: integer;
  Data: Pointer = nil): TListItem;

procedure ClearTreeView(theTreeView: TTreeView;
  NodeDataType: MYX_NODE_DATA_TYPE);
procedure ClearListView(theListView: TListView;
  NodeDataType: MYX_NODE_DATA_TYPE);


function ShowModalDialog(Titel: string; Messagetext: string;
  DlgType: MYX_MESSAGE_DLG_TYPE = myx_mtInformation;
  buttons: string = 'OK'): Integer;
function ShowModalEditDialog(Titel: string; Messagetext: string;
  DlgType: MYX_MESSAGE_DLG_TYPE;
  buttons: string;
  doEdit: Boolean; editCaption: string; var editValue: string;
  editLinesNum: integer = 1; editReadOnly: Boolean = False): Integer;

function GetLocalHostName: string;
function GetFileSize(const FileName: string): Int64;
function GetFileVersion(fname: string): string;
function GetDrives: string;
function GetDriveInfo(drive: string): PMYX_DRIVE_INFO;
function FormatFileSize(size: double): string;

//Create prozess
procedure CreateProz(command: string; workingdir: string = ''; show: Boolean = True; wait4proz: Boolean = False);

procedure MessageToAllForms(Msg: UINT; wParam: WPARAM; lParam: LPARAM);

function ServiceStop(sMachine, sService: string): boolean;
function ServiceStart(sMachine, sService : string): boolean;
function ServiceStatus(sMachine, sService : string): integer;

procedure DisableEnableControls(WinControl: TWinControl; Enable: Boolean);
procedure DisableEnablePages(PageControl: TPageControl; Enable: Boolean);
procedure InitHandles(WinControl: TWinControl);

function LoadTextFromFile(filename: string): string;

function ExtractNumber(s: string): integer;
function ExtractString(s: string): string;
function ExtractText(s: string): string;

function ReadFromReg(theRoot: HKEY; Key, Name: string; Typ: integer; DefaultVal: string): string;

function GetSystemDir: String;
function GetWindowsDir: String;

const
  CSIDL_PROGRAM_FILES = $0026;
  CSIDL_PROGRAM_FILES_COMMON = $002B;
  CSIDL_COMMON_APPDATA = $0023;
  CSIDL_PERSONAL = $0005;

function GetSpecialFolder(Folder: Integer): String;
function GetProgramFilesDir: String;
function GetCommonProgFilesDir: String;
function GetApplicationDataDir: String;
function GetHomeDir: String;

procedure ScanForServices(TreeView: TTreeView; Icon: integer);

function _s(val: integer): integer;
function _d(val: integer): integer;

function StringAlignLeft(s: string; len: integer): string;
function StringAlignRight(s: string; len: integer): string;

procedure CopyDiskFile(sourcefile, destinationfile: string; PromtBeforeOverwrite: Boolean);

procedure ShowHelp;

procedure InitForm(Form: TForm);

procedure DeleteComponentWithChildren(Component: TComponent; RemoveFromList: TList = nil);

function GetComputerNetName: string;


implementation

constructor TMyxModalDialog.Create(Titel: string; Messagetext: string;
  DlgType: MYX_MESSAGE_DLG_TYPE;
  buttons: string;
  doEdit: Boolean; editCaption: string; var editValue: string;
  editLinesNum: integer = 1; editReadOnly: Boolean = False);
var
  DlgBtn: TButton;
  DlgButtonCaptions: TStringList;
  MessageLines: TStringList;
  i, w, lines, editlines: integer;
begin
  CreateNew(nil);

  DlgButtonCaptions:=TStringList.Create;
  MessageLines:=TStringList.Create;
  DlgBtnList:=TList.Create;
  try
    MessageLines.Text:=Messagetext;
    DlgButtonCaptions.Text:=buttons;

    BorderStyle:=bsDialog;
    Caption:=Titel;
    Tag:=0;

    //Get number of text-lines
    lines:=0;
    for i:=0 to MessageLines.Count-1 do
    begin
      inc(lines);
      if(Canvas.TextWidth(MessageLines[i])>400)then
        inc(lines, (Canvas.TextWidth(MessageLines[i]) div 400));
    end;

    //Reserve space for Edits
    editlines:=0;
    if(doEdit)then
      editlines:=editLinesNum+1;

    if(lines+editlines>=3)then
      Height:=130+(lines+editlines)*13
    else
      Height:=143;


    //Display Message Text
    MessageLbl:=TLabel.Create(self);
    MessageLbl.Parent:=self;
    MessageLbl.Left:=80;
    MessageLbl.Top:=30;
    MessageLbl.Caption:=Messagetext;
    MessageLbl.Height:=lines*13;
    w:=Canvas.TextWidth(Messagetext);
    if(w>400)then
    begin
      MessageLbl.AutoSize:=False;
      MessageLbl.Width:=400;
      MessageLbl.WordWrap:=True;
    end
    else
    begin
      MessageLbl.AutoSize:=True;
    end;

    //Size the form according to the size of the label/buttons
    if(MessageLbl.Width+80+32>DlgButtonCaptions.Count*(80+10)+42)then
      Width:=MessageLbl.Width+80+32
    else
      Width:=DlgButtonCaptions.Count*(80+10)+42;

    //Create Edit
    if(doEdit)then
    begin
      if(editCaption<>'')then
      begin
        EditLbl:=TLabel.Create(self);
        EditLbl.Parent:=self;
        EditLbl.Left:=80;
        EditLbl.Top:=MessageLbl.Top+MessageLbl.Height+10;
        EditLbl.Width:=Canvas.TextWidth(editCaption)+4;
        EditLbl.Caption:=editCaption;
      end;

      ValueMemo:=TMemo.Create(self);
      ValueMemo.Parent:=self;
      if(editCaption<>'')then
        ValueMemo.Left:=EditLbl.Left+EditLbl.Width+14
      else
        ValueMemo.Left:=80;
      ValueMemo.Top:=MessageLbl.Top+MessageLbl.Height+10-4;

      ValueMemo.Width:=Width-ValueMemo.Left-30;
      ValueMemo.Height:=editLinesNum*13+6;
      ValueMemo.Text:=editValue;

      ActiveControl:=ValueMemo;

      //Disable mulitline functionality if editLinesNum is 1
      if(editLinesNum=1)then
      begin
        ValueMemo.WordWrap:=False;
        ValueMemo.WantReturns:=False;
      end;

      if(editReadOnly)then
      begin
        ValueMemo.ReadOnly:=True;
        ValueMemo.Color:=Color;
      end;

      if(ValueMemo.Text<>'')then
        ValueMemo.SelStart:=Length(ValueMemo.Text);
    end
    else
      ValueMemo:=nil;

    //Create buttons
    for i:=0 to DlgButtonCaptions.Count-1 do
    begin
      DlgBtn:=TButton.Create(self);
      DlgBtn.Parent:=self;
      DlgBtn.Caption:=DlgButtonCaptions[i];
      DlgBtn.Width:=80;
      DlgBtn.Height:=25;
      DlgBtn.Top:=Height-70;
      DlgBtn.Left:=(Width-DlgButtonCaptions.Count*80+19) div 2+
        (80+10)*i;
      DlgBtn.ModalResult:=i+1;

      DlgBtn.Default:=(i=0);

      DlgBtnList.Add(DlgBtn);
    end;

    //Create image
    Image:=TImage.Create(self);
    Image.Parent:=self;
    Image.Left:=20;
    Image.Top:=15;
    case DlgType of
      myx_mtError:
        LoadPNGImageFromResource('messagedlg_error', Image);
    else
      LoadPNGImageFromResource('messagedlg_warning', Image);
    end;

    if(DlgBtnList.Count>0)and(Not(doEdit))then
      ActiveControl:=DlgBtnList[0];


    Left:=(Screen.Width-Width) div 2;
    Top:=(Screen.Height-Height) div 2;
  finally
    MessageLines.Free;
    DlgButtonCaptions.Free;
  end;
end;

destructor TMyxModalDialog.Destroy;
begin
  DlgBtnList.Free;

  inherited Destroy;
end;

procedure TMyxModalDialog.doButtonClick(Sender: TObject);
begin
  TForm(Parent).Tag:=ModalResult;
  TForm(Parent).Close;
end;

procedure TMyxModalDialog.HideEdit;
var i: integer;
begin
  ValueMemo.Visible:=False;

  Height:=MessageLbl.Top+MessageLbl.Height+100;

  for i:=0 to DlgBtnList.Count-1 do
    TButton(DlgBtnList[i]).Top:=Height-70;

  if(DlgBtnList.Count>0)then
    ActiveControl:=DlgBtnList[0];
end;

procedure TMyxModalDialog.ShowEdit;
var i: integer;
begin
  Height:=ValueMemo.Top+ValueMemo.Height+80;

  for i:=0 to DlgBtnList.Count-1 do
    TButton(DlgBtnList[i]).Top:=Height-70;

  ValueMemo.Visible:=True;
end;

function getXPStyleEnabled: Boolean;
var VersionInfo: TOSVersionInfo;
  Registry: TRegistry;
begin
  getXPStyleEnabled:=False;

  VersionInfo.dwOSVersionInfoSize:=Sizeof(TOSVersionInfo);
  GetVersionEx(VersionInfo);

  if(VersionInfo.dwMajorVersion<5)then
    Exit;

  if(VersionInfo.dwMajorVersion=5)and
    (VersionInfo.dwMinorVersion=0)then
    Exit;

  Registry:=TRegistry.Create;
  Registry.RootKey:=HKEY_CURRENT_USER; //HKEY_LOCAL_MACHINE;
  Registry.OpenKey('Control Panel\Appearance', False);
  if(Registry.ReadString('Current')<>'Windows Standard')then
    Result:=True;
end;

function LoadPNGImageFromResource(Resourcename: string; Image: TImage): TPNGObject;
var tmpStream: TResourceStream;
  PNGImage: TPNGObject;
begin
  LoadPNGImageFromResource:=nil;

  tmpStream:=TResourceStream.Create(HInstance, Resourcename, 'PNG');
  try
    PNGImage:=TPNGObject.Create;
    try
      PNGImage.LoadFromStream(tmpStream);

      if(Image<>nil)then
        Image.Picture.Graphic:=PNGImage;

      LoadPNGImageFromResource:=PNGImage;
    except
      PNGImage.Free;
    end;
  finally
    tmpStream.Free;
  end;
end;

function AddTreeViewChildNode(theTreeView: TTreeView;
  ParentNode: TTreeNode; Caption: string; Icon: integer;
  Data: Pointer): TTreeNode;
var theTreeNode: TTreeNode;
begin
  if(ParentNode=nil)then
  begin
    theTreeNode:=theTreeView.Items.Add(nil, Caption);
    theTreeNode.ImageIndex:=Icon;
    theTreeNode.SelectedIndex:=Icon;
  end
  else
  begin
    theTreeNode:=theTreeView.Items.AddChild(ParentNode, Caption);
    theTreeNode.ImageIndex:=Icon;
    theTreeNode.SelectedIndex:=Icon;
  end;

  theTreeNode.Data:=Data;

  AddTreeViewChildNode:=theTreeNode;
end;

function AddListViewItem(theListView: TListView;
  ParentItem: TListItem; Caption: string; Icon: integer;
  Data: Pointer = nil): TListItem;
var theListItem: TListItem;
begin
  if(ParentItem=nil)then
  begin
    theListItem:=theListView.Items.Add;
    theListItem.Caption:=Caption;
    theListItem.ImageIndex:=Icon;
  end
  else
  begin
    theListItem:=theListView.Items.Insert(theListView.Items.IndexOf(ParentItem));
    theListItem.Caption:=Caption;
    theListItem.ImageIndex:=Icon;
  end;

  theListItem.Data:=Data;

  Result:=theListItem;
end;

procedure ClearTreeView(theTreeView: TTreeView;
  NodeDataType: MYX_NODE_DATA_TYPE);
var i: integer;
begin
  for i:=0 to theTreeView.Items.Count-1 do
    if(theTreeView.Items[i].Data<>nil)then
    begin
      case NodeDataType of
        myx_ndt_pointer:
          FreeMem(theTreeView.Items[i].Data);
        myx_ndt_tobject:
          TObject(theTreeView.Items[i].Data).Free;
      end;
    end;

  theTreeView.Items.Clear;
end;

procedure ClearListView(theListView: TListView;
  NodeDataType: MYX_NODE_DATA_TYPE);
var i: integer;
begin
  for i:=0 to theListView.Items.Count-1 do
    if(theListView.Items[i].Data<>nil)then
    begin
      case NodeDataType of
        myx_ndt_pointer:
          FreeMem(theListView.Items[i].Data);
        myx_ndt_tobject:
          TObject(theListView.Items[i].Data).Free;
      end;
    end;

  theListView.Clear;
end;

function ShowModalDialog(Titel: string; Messagetext: string;
  DlgType: MYX_MESSAGE_DLG_TYPE; buttons: string): Integer;
var NotUsed: string;
begin
  Result:=ShowModalEditDialog(Titel, Messagetext, DlgType, buttons,
    False, '', NotUsed);
end;

function ShowModalEditDialog(Titel: string; Messagetext: string;
  DlgType: MYX_MESSAGE_DLG_TYPE; buttons: string;
  doEdit: Boolean; editCaption: string; var editValue: string;
  editLinesNum: integer = 1; editReadOnly: Boolean = False): Integer;
var ModalDialog: TMyxModalDialog;
begin
  ModalDialog:=TMyxModalDialog.Create(Titel, Messagetext, DlgType, buttons,
    doEdit, editCaption, editValue, editLinesNum, editReadOnly);
  try
    Result:=ModalDialog.ShowModal;

    if(doEdit)and(ModalDialog.ValueMemo.Text<>'')then
      editValue:=ModalDialog.ValueMemo.Text;
  finally
    ModalDialog.Free;
  end;
end;

function GetLocalHostName: string;
var s: Array[0..128] of char;
begin
  gethostname(@s, 128);

  result:=s;
end;

function GetFileSize(const FileName: string): Int64;
var
  SizeLow, SizeHigh: DWord;
  hFile: THandle;
begin
  Result := 0;
  hFile := FileOpen(FileName, fmOpenRead or fmShareDenyNone);
  try
    if hFile <> 0 then
    begin
      SizeLow := Windows.GetFileSize(hFile, @SizeHigh);
      Result := (SizeHigh shl 32) + SizeLow;
    end;
  finally
    FileClose(hFile);
  end;
end;


function GetFileVersion(fname: string): string;
var
  size: longint;
  vlen: DWord;
  FInfo: pointer;
  FLang: PInteger;
  vptr: pchar;
begin
  Result := '';

  size := GetFileVersionInfoSize(PChar(fname), vlen);
  if(size>0)then
  begin
    GetMem(FInfo, size);
    if(GetFileVersionInfo(PChar(fname), vlen, size, FInfo))then
    begin
      // get languages
      VerQueryValue(FInfo, '\VarFileInfo\Translation', pointer(FLang), vlen);
      if VerQueryValue(FInfo, pchar(Format('\StringFileInfo\%4.4x%4.4x\FileVersion', [LoWord(FLang^), HiWord(FLang^)])), pointer(vptr), vlen) then
      begin
        Result := vptr;
      end;
    end;    
    FreeMem(FInfo, size);
  end;
end;

function GetDrives: string;
var
  r: LongWord;
  Drives: array[0..128] of char;
  pDrive: pchar;
begin
  r := GetLogicalDriveStrings(sizeof(Drives), Drives);
  if r = 0 then exit;
  if r > sizeof(Drives) then
    raise Exception.Create(SysErrorMessage(ERROR_OUTOFMEMORY));

  pDrive := Drives;  // Point to the first drive
  while pDrive^ <> #0 do begin
    Result:=Result+Copy(pDrive, 1, 2)+#13#10;

    inc(pDrive, 4);  // Point to the next drive
  end;

end;

function GetDriveInfo(drive: string): PMYX_DRIVE_INFO;
var DriveInfo: PMYX_DRIVE_INFO;
  x: integer;
  VLabel, FSys: array[0..255] of char;
  SNr, FL, SysFlag: DWord;
begin
  new(DriveInfo);

  if(GetVolumeInformation(PChar(Drive+'\'), VLabel, SizeOf(VLabel),
    @SNr, FL, SysFlag, FSys, SizeOf(FSys)))then
  begin
    DriveInfo.VolumeLabel:=VLabel;
    DriveInfo.VolumeNumber:=IntToStr(SNr);
    DriveInfo.FileSystem:=string(FSys);

    x:=GetDriveType(PChar(Drive+'\'));
    case x of
      2:
        DriveInfo.DriveType:='Floppy';
      3:
        DriveInfo.DriveType:='Fixed';
      4:
        DriveInfo.DriveType:='Network';
      5:
        DriveInfo.DriveType:='CDRom';
      6:
        DriveInfo.DriveType:='Ram';
      else
        DriveInfo.DriveType:='';
    end;

    DriveInfo.TotalSize:=DiskSize(Ord(UpCase(Drive[1]))-64);
    DriveInfo.FreeSpace:=DiskFree(Ord(UpCase(Drive[1]))-64);
  end
  else
  begin
    DriveInfo.VolumeLabel:='';
    DriveInfo.VolumeNumber:='';
    DriveInfo.FileSystem:='';
    DriveInfo.TotalSize:=0;
    DriveInfo.FreeSpace:=0;
  end;

  Result:=DriveInfo;
end;

function FormatFileSize(size: double): string;
var s: string;
begin
  if(size<1024*(1024-200))then
  begin
    size:=size/1024;
    s:='kB';
  end
  else if(size<1024*1024*(1024-200))then
  begin
    size:=size/(1024*1024);
    s:='MB';
  end
  else
  begin
    size:=size/(1024*1024*1024);
    s:='GB';
  end;

  Result:=FormatFloat('###,###,##0.#', size)+' '+s;
end;

procedure CreateProz(command: string; workingdir: string = ''; show: Boolean = True; wait4proz: Boolean = False);
var
{$IFDEF MSWINDOWS}
  StartupInfo : TStartupInfo;
  wdir: pchar;
  ProcessInfo : TProcessInformation;
{$ENDIF}

{$IFDEF LINUX}
  FCmdThread: TCmdExecThread;
{$ENDIF}
begin
{$IFDEF MSWINDOWS}
  if(workingdir<>'')then
    wdir:=PChar(workingdir)
  else
    wdir:=nil;

  FillChar(StartupInfo, SizeOf(TStartupInfo), 0);
  StartupInfo.cb := Sizeof(TStartupInfo);
  StartupInfo.dwFlags := STARTF_USESHOWWINDOW;
  if(show)then
    StartupInfo.wShowWindow := SW_Show
  else
    StartupInfo.wShowWindow := SW_Hide;

  if(Not(CreateProcess(nil, pchar(command), nil, nil,
    true, Normal_PRIORITY_CLASS and CREATE_DEFAULT_ERROR_MODE,
    nil, wdir, StartupInfo, ProcessInfo)))then
    raise Exception.Create(command+': '#13#10+
      'The Program could not be launched.'#13#10+
      'Error Nr.: '+IntToStr(GetLastError)+#13#10+
      SysErrorMessage(GetLastError));


  //If requested, wait for the programm to end
  if(wait4proz)then
  begin
    while(WaitForSingleObject(ProcessInfo.hProcess, 0)=WAIT_TIMEOUT)do
      Application.ProcessMessages;

    ProcessInfo.hProcess:=0;
  end;
{$ENDIF}

{$IFDEF LINUX}
  {if FCmdThread <> nil then
  begin
    if not FCmdThread.Done
      then raise Exception.Create('A command is already running')
    else FCmdThread.Free;
  end;}
  FCmdThread := TCmdExecThread.Create;
  //FCmdThread.OnComplete := InternalComplete;
  FCmdThread.Command := command;
  FCmdThread.Resume;

  //Wenn erwnscht, warten bis Programm beendet wird.
  if(wait4proz=1)then
  begin
    while(Not(FCmdThread.Done))do
    begin
      Sleep(500);
      //Do drawing and stuff
      Application.ProcessMessages;
    end;
  end;
{$ENDIF}
end;

procedure MessageToAllForms(Msg: UINT; wParam: WPARAM; lParam: LPARAM);
var i: integer;
begin
  for i:=0 to Screen.FormCount-1 do
    Screen.Forms[i].Perform(Msg, wParam, lParam);
end;

// sMachine .. \\SERVER or empty for local machine
// sService ... service name, ie: mysqld
// return TRUE if successful
function ServiceStart(sMachine, sService : string) : boolean;
var
  schm, // service control manager handle
  schs   : SC_Handle; // service handle
  ss     : TServiceStatus; // service status
  psTemp : PChar; // temp char pointer
  dwChkP : DWord; // check point
begin
  ss.dwCurrentState := 0;
  
  // connect to the service control manager
  schm:=OpenSCManager(PChar(sMachine), nil, SC_MANAGER_CONNECT);

  // if successful...
  if(schm>0)then
  begin
    // open a handle to the specified service
    schs:=OpenService(schm, PChar(sService), SERVICE_START or
      SERVICE_QUERY_STATUS);

    // if successful...
    if(schs>0)then
    begin
      psTemp := nil;
      if(StartService(schs, 0, psTemp))then
      begin
        // check status
        if(QueryServiceStatus(schs, ss))then
        begin
          while(SERVICE_RUNNING<>ss.dwCurrentState)do
          begin
            // dwCheckPoint contains a
            // value that the service
            // increments periodically
            // to report its progress
            // during a lengthy
            // operation.
            //
            // save current value

            dwChkP := ss.dwCheckPoint;

            // wait a bit before
            // checking status again
            //
            // dwWaitHint is the
            // estimated amount of time
            // the calling program
            // should wait before calling
            // QueryServiceStatus() again
            //
            // idle events should be
            // handled here...
            //Sleep(ss.dwWaitHint);
            Sleep(1000);

            if(not QueryServiceStatus(schs, ss))then
            begin
              // couldn't check status
              break;
            end;

            if(ss.dwCheckPoint<dwChkP)then
            begin
              // QueryServiceStatus
              // didn't increment
              // dwCheckPoint as it
              // should have.
              // avoid an infinite
              // loop by breaking
              break;
            end;
          end;
        end;
      end;

      // close service handle
      CloseServiceHandle(schs);
    end;

    // close service control
    // manager handle
    CloseServiceHandle(schm);
  end;

  // return TRUE if
  // the service status is running
  Result:=(SERVICE_RUNNING=ss.dwCurrentState);
end;



// sMachine .. \\SERVER or empty for local machine
// sService ... service name, ie: mysqld
// return TRUE if successful
function ServiceStop(sMachine, sService: string): boolean;
var
  schm, // service control manager handle
  schs   : SC_Handle; // service handle
  ss     : TServiceStatus; // service status
  dwChkP : DWord; // check point
begin
  // connect to the service
  // control manager
  schm := OpenSCManager(PChar(sMachine), Nil, SC_MANAGER_CONNECT);

  // if successful...
  if(schm>0)then
  begin
    // open a handle to the specified service
    schs:=OpenService(schm, PChar(sService), SERVICE_STOP or
      SERVICE_QUERY_STATUS);

    // if successful...
    if(schs > 0)then
    begin
      if(ControlService(schs, SERVICE_CONTROL_STOP, ss))then
      begin
        // check status
        if(QueryServiceStatus(
             schs,
             ss))then
        begin
          while(SERVICE_STOPPED
            <> ss.dwCurrentState)do
          begin
            // dwCheckPoint contains a
            // value that the service
            // increments periodically
            // to report its progress
            // during a lengthy
            // operation.
            //
            // save current value
            dwChkP := ss.dwCheckPoint;


            // wait a bit before
            // checking status again
            //
            // dwWaitHint is the
            // estimated amount of time
            // the calling program
            // should wait before calling
            // QueryServiceStatus() again
            //
            // idle events should be
            // handled here...
            //Sleep(ss.dwWaitHint);
            Sleep(1000);

            if(not QueryServiceStatus(schs, ss))then
            begin
              // couldn't check status
              break;
            end;

            if(ss.dwCheckPoint<dwChkP)then
            begin
              // QueryServiceStatus
              // didn't increment
              // dwCheckPoint as it
              // should have.
              // avoid an infinite
              // loop by breaking
              break;
            end;
          end;
        end;
      end;

      // close service handle
      CloseServiceHandle(schs);
    end;

    // close service control
    // manager handle
    CloseServiceHandle(schm);
  end;

  // return TRUE if
  // the service status is stopped
  Result:=(SERVICE_STOPPED=ss.dwCurrentState);
end;


// sMachine .. \\SERVER or empty for local machine
// sService ... service name, ie: mysqld
// return codes:
//   -1 if not successful
//   SERVICE_STOPPED
//   SERVICE_RUNNING
//   SERVICE_PAUSED
//   SERVICE_START_PENDING
//   SERVICE_STOP_PENDING
//   SERVICE_CONTINUE_PENDING
//   SERVICE_PAUSE_PENDING
function ServiceStatus(sMachine, sService : string): integer;
var
  schm,
  schs   : SC_Handle;
  ss     : TServiceStatus;
  dwStat : integer;
begin
  dwStat := -1;

  // connect to the service
  // control manager 
  schm := OpenSCManager(
    PChar(sMachine),
    Nil,
    SC_MANAGER_CONNECT);

  // if successful...
  if(schm>0)then
  begin
    // open a handle to
    // the specified service
    schs := OpenService(
      schm,
      PChar(sService),
      // we want to
      // query service status
      SERVICE_QUERY_STATUS);

    // if successful...
    if(schs > 0)then
    begin
      // retrieve the current status
      // of the specified service    
      if(QueryServiceStatus(schs, ss))then
      begin
        dwStat:=ss.dwCurrentState;
      end;
      
      // close service handle
      CloseServiceHandle(schs);
    end;

    // close service control
    // manager handle
    CloseServiceHandle(schm);
  end;

  Result := dwStat;
end;

procedure DisableEnableControls(WinControl: TWinControl; Enable: Boolean);
var i: integer;
begin
  for i:=0 to WinControl.ControlCount-1 do
  begin
    WinControl.Controls[i].Enabled:=Enable;

    if(WinControl.Controls[i].InheritsFrom(TWinControl))then
      DisableEnableControls(TWinControl(WinControl.Controls[i]), Enable);
  end;

  WinControl.Enabled:=Enable;
end;

procedure DisableEnablePages(PageControl: TPageControl; Enable: Boolean);
var i: integer;
begin
  for i:=0 to PageControl.PageCount-1 do
    DisableEnableControls(PageControl.Pages[i], Enable);
end;

procedure InitHandles(WinControl: TWinControl);
var i: integer;
begin
  WinControl.HandleNeeded;

  for i:=0 to WinControl.ControlCount-1 do
  begin
    if(WinControl.Controls[i].InheritsFrom(TWinControl))then
      InitHandles(TWinControl(WinControl.Controls[i]));
  end;
end;


function LoadTextFromFile(filename: string): string;
var Stream: TStream;
  Size: Integer;
  S: string;
begin
  Stream := TFileStream.Create(FileName, fmOpenRead or fmShareDenyWrite);
  try
    Size := Stream.Size - Stream.Position;
    SetString(S, nil, Size);
    Stream.Read(Pointer(S)^, Size);
  finally
    Stream.Free;
  end;

  LoadTextFromFile:=S;
end;

function ExtractNumber(s: string): integer;
var i: integer;
  num, fac: integer;
begin
  num:=0;
  fac:=1;

  for i:=Length(s) downto 1 do
    if(Ord(s[i])>=Ord('0'))and(Ord(s[i])<=Ord('9'))then
    begin
      num:=num+(Ord(s[i])-Ord('0'))*fac;
      fac:=fac*10;
    end;

  Result:=num;
end;

function ExtractString(s: string): string;
var i: integer;
  s1: string;
begin
  s1:='';
  for i:=1 to Length(s) do
    if(Ord(s[i])<Ord('0'))or(Ord(s[i])>Ord('9'))then
      s1:=s1+s[i];

  Result:=s1;
end;

function ExtractText(s: string): string;
var i: integer;
  s1: string;
begin
  s1:='';

  for i:=1 to Length(s) do
    if((Ord(s[i])>=Ord('0'))and(Ord(s[i])<=Ord('9')))or //numbers
      ((Ord(s[i])>=Ord('a'))and(Ord(s[i])<=Ord('z')))or //a-z
      ((Ord(s[i])>=Ord('A'))and(Ord(s[i])<=Ord('Z')))or //A-Z
      (s[i]=' ')or(s[i]='_')or(s[i]='.')
      then
      s1:=s1+s[i];

  Result:=s1;
end;


function ReadFromReg(theRoot: HKEY; Key, Name: string; Typ: integer; DefaultVal: string): string;
var MyReg: TRegistry;
begin
  try
    MyReg := TRegistry.Create;
    MyReg.RootKey:=theRoot;
    try
      MyReg.OpenKey(Key, True);
      case Typ of
        0: ReadFromReg:=MyReg.ReadString(Name);
        1: ReadFromReg:=IntToStr(MyReg.ReadInteger(Name));
      end;
      MyReg.CloseKey;
    finally
      MyReg.Free;
    end;
  except
    ReadFromReg:=DefaultVal;
  end;
end;

function GetSystemDir: String;
var SysDir: Array[0..MAX_PATH] of Char;
begin
  GetSystemDirectory(SysDir, MAX_PATH);

  GetSystemDir:=IncludeTrailingPathDelimiter(SysDir);
end;

function GetWindowsDir: String;
var WinDir: Array[0..MAX_PATH] of Char;
begin
  GetWindowsDirectory(WinDir, MAX_PATH);

  GetWindowsDir:=IncludeTrailingPathDelimiter(WinDir);
end;


//CSIDL_COOKIES              Cookies
//CSIDL_DESKTOPDIRECTORY     Desktop
//CSIDL_FAVORITES            Favoriten
//CSIDL_HISTORY              Internet-Verlauf
//CSIDL_INTERNET_CACHE       "Temporary Internet Files"
//CSIDL_PERSONAL             Eigene Dateien               $0005
//CSIDL_PROGRAMS             "Programme" im Startmen
//CSIDL_RECENT               "Dokumente" im Startmen
//CSIDL_SENDTO               "Senden an" im Kontextmen
//CSIDL_STARTMENU            Startmen
//CSIDL_STARTUP              Autostart

function GetSpecialFolder(Folder: Integer): String;
var
  pMalloc: IMalloc;
  pidl: PItemIDList;
  Path: PChar;
begin
  // get IMalloc interface pointer
  if (SHGetMalloc(pMalloc) <> S_OK) then begin
    raise EInOutError.Create('Couldn''t get pointer to IMalloc interface.');

    Exit;
  end;

  // retrieve path
  SHGetSpecialFolderLocation(0, Folder, pidl);
  GetMem(Path, MAX_PATH);
  SHGetPathFromIDList(pidl, Path);
  Result := Path;
  FreeMem(Path);

  // free memory allocated by SHGetSpecialFolderLocation
  pMalloc.Free(pidl);
end;

function GetProgramFilesDir: String;
var folder: string;
begin
  folder:=GetSpecialFolder(CSIDL_PROGRAM_FILES);

  if(folder='')then
    folder:=ReadFromReg(HKEY_LOCAL_MACHINE,
      '\Software\Microsoft\Windows\CurrentVersion',
      'ProgramFilesDir', 0, '');

  Result:=IncludeTrailingPathDelimiter(folder);
end;

function GetCommonProgFilesDir: String;
var folder: string;
begin
  folder:=GetSpecialFolder(CSIDL_PROGRAM_FILES_COMMON);

  if(folder='')then
    folder:=ReadFromReg(HKEY_LOCAL_MACHINE,
    '\Software\Microsoft\Windows\CurrentVersion',
    'CommonFilesDir', 0, '');

  Result:=IncludeTrailingPathDelimiter(folder);
end;

function GetApplicationDataDir: String;
var folder: string;
begin
  folder:=GetSpecialFolder(CSIDL_APPDATA);

  if(folder='')then
    folder:=ReadFromReg(HKEY_CURRENT_USER,
    'Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders',
    'AppData', 0, '');

  Result:=IncludeTrailingPathDelimiter(folder);
end;

function GetHomeDir: String;
var folder: string;
begin
  folder:=GetSpecialFolder(CSIDL_PERSONAL);

  if(folder='')then
    folder:=ReadFromReg(HKEY_CURRENT_USER,
    'Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders',
    'Personal', 0, '');

  Result:=IncludeTrailingPathDelimiter(folder);
end;

procedure ScanForServices(TreeView: TTreeView; Icon: integer);
var ServiceReg: TRegistry;
  KeyInfo: TRegKeyInfo;
  SubKeyNames: TStringList;
  i: integer;
  ImagePath, s, s1: string;
  MySQLService: TMySQLService;
begin
  ServiceReg:=TRegistry.Create;
  SubKeyNames:=TStringList.Create;
  try
    ServiceReg.RootKey:=HKEY_LOCAL_MACHINE;

    ServiceReg.OpenKey('SYSTEM\'+
      'CurrentControlSet\Services', False);

    ServiceReg.GetKeyInfo(KeyInfo);
    ServiceReg.GetKeyNames(SubKeyNames);

    ServiceReg.CloseKey;

    for i:=0 to SubKeyNames.Count-1 do
    begin
      ServiceReg.OpenKey('SYSTEM\'+
        'CurrentControlSet\Services\'+SubKeyNames[i], False);

      try
        ImagePath:=ServiceReg.ReadString('ImagePath');
      except
        ImagePath:='';
      end;

      //Check if mysqld is in ImagePath
      if(Pos('mysqld', LowerCase(ImagePath))>0)then
      begin
        MySQLService:=TMySQLService.Create;
        MySQLService.ExistingService:=True;
        //ServiceList.Add(MySQLService);

        MySQLService.ServiceName:=SubKeyNames[i];
        MySQLService.DisplayName:=ServiceReg.ReadString('DisplayName');
        MySQLService.Description:=ServiceReg.ReadString('Description');

        try
          MySQLService.StartType:=ServiceReg.ReadInteger('Start');
        except
          MySQLService.StartType:=3;
        end;

        //Typical ImagePath:
        //c:\programme\mysql\bin\mysqld-max-nt --defaults-file="C:\Programme\MySQL\my.cnf" MySQL

        //Get command from ImagePath
        //"xxx" yyy or xxx yyy
        if(Copy(ImagePath, 1, 1)='"')then
          s:=Copy(ImagePath, 1, Pos('"', Copy(ImagePath, 2, Length(ImagePath))))
        else
          if(Pos(' ', Copy(ImagePath, 1, Length(ImagePath)))>0)then
            s:=Copy(ImagePath, 1, Pos(' ', Copy(ImagePath, 2, Length(ImagePath))))
          else
            s:=ImagePath;

        MySQLService.PathToBinary:=ExtractFilePath(s);
        if(Copy(MySQLService.PathToBinary, 1, 1)='"')then
          MySQLService.PathToBinary:=IncludeTrailingPathDelimiter(
            Copy(MySQLService.PathToBinary, 2,
            Length(MySQLService.PathToBinary)-2));

        MySQLService.ServerType:=ExtractFileName(s);

        MySQLService.ConfigFile:='';
        MySQLService.ConfigFileSection:='';

        //Get defaults-file
        s:=Trim(Copy(ImagePath, Length(s)+1, Length(ImagePath)));
        if(s<>'')then
        begin

          if(Pos('--defaults-file=', s)>0)then
          begin
            s1:=Copy(s, Pos('--defaults-file=', s)+16, Length(s));

            //--defaults-file="xxx" or --defaults-file=xxx
            if(Copy(s1, 1, 1)='"')then
              s1:=Copy(s1, 1, Pos('"', Copy(s1, 2, Length(s1)))+1)
            else
              if(Pos(' ', s1)>0)then
                s1:=Copy(s1, 1, Pos(' ', s1)-1);

            MySQLService.ConfigFile:=s1;
            //Remove " if present
            if(Copy(MySQLService.ConfigFile, 1, 1)='"')then
              MySQLService.ConfigFile:=Copy(MySQLService.ConfigFile, 2,
                Length(MySQLService.ConfigFile)-2);

            //The rest is the serice name
            //Trim(Copy(s, Length(s1)+16+1, Length(s)));

            MySQLService.ConfigFileSection:='mysqld';
          end
          else
            //If there is no ConfigFile, the service name is the section name
            MySQLService.ConfigFileSection:=Trim(s);
        end;

        //if the ConfigFile isn't specified in the Registry
        //it can be c:\my.cnf or windows-directory\my.ini
        if(MySQLService.ConfigFile='')then
        begin
          s:=GetWindowsDir;

          if(FileExists(s+'my.ini'))then
            MySQLService.ConfigFile:=s+'my.ini'
          else if(FileExists('c:\my.cnf'))then
            MySQLService.ConfigFile:='c:\my.cnf'
          else
          begin
            //If there is no default cnf file yet,
            //use PathToBinaries\my.cnf
            s1:=IncludeTrailingPathDelimiter(
              Copy(MySQLService.PathToBinary, 1, Pos('\bin', MySQLService.PathToBinary)-1));
            if(s<>'')then
              MySQLService.ConfigFile:=s1+'my.cnf'
            else
              MySQLService.ConfigFile:=s+'my.ini';
          end;
        end;

        //if the ConfigFileSection isn't specified in the Registry
        //it is mysqld
        if(MySQLService.ConfigFileSection='')then
          MySQLService.ConfigFileSection:='mysqld';


        //-----------------------------------------------------------------
        //Try to find errorfile
        s:=GetLocalHostName;
        MySQLService.ErrorLog:='';

        if(FileExists(GetProgramFilesDir+'MySQL\data\'+s+'.err'))then
          MySQLService.ErrorLog:=GetProgramFilesDir+'MySQL\data\'+s+'.err'
        else if(FileExists('C:\MySQL\data\'+s+'.err'))then
          MySQLService.ErrorLog:='C:\MySQL\data\'+s+'.err'
        else if(FileExists('D:\MySQL\data\'+s+'.err'))then
          MySQLService.ErrorLog:='D:\MySQL\data\'+s+'.err';

        MySQLService.logfile_size:=0;

        //-----------------------------------------------------------------
        //Get Service Status
        MySQLService.Status:=ServiceStatus('', MySQLService.ServiceName);

        if(TreeView<>nil)then
          AddTreeViewChildNode(TreeView, nil, MySQLService.ServiceName,
            Icon+Ord(MySQLService.Status<>SERVICE_RUNNING), MySQLService);
      end;

      ServiceReg.CloseKey;
    end;

  finally
    SubKeyNames.Free;
    ServiceReg.Free;
  end;
end;

function _s(val: integer): integer;
begin
  Result:=Round(val*(Screen.PixelsPerInch/96));
end;

function _d(val: integer): integer;
begin
  Result:=Round(val/(Screen.PixelsPerInch/96));
end;

function StringAlignLeft(s: string; len: integer): string;
begin
  if(Length(s)>len)then
    Result:=Copy(s, 1, len)
  else
    Result:=s+StringOfChar(' ', len-Length(s));
end;

function StringAlignRight(s: string; len: integer): string;
begin
  if(Length(s)>len)then
    Result:=Copy(s, 1, len)
  else
    Result:=StringOfChar(' ', len-Length(s))+s;
end;

procedure CopyDiskFile(sourcefile, destinationfile: string; PromtBeforeOverwrite: Boolean);
var NewFile: TFileStream;
  OldFile: TFileStream;
begin
  if(FileExists(sourcefile))then
  begin
    if(FileExists(destinationfile))and(PromtBeforeOverwrite)then
    begin
      if(MessageDlg(Format('The destination file %s does already exist. '+
        'Do you want to overwrite this file?', [destinationfile]), mtCustom, [mbYes, mbNo], 0) = 3)then
        DeleteFile(destinationfile)
      else
        Exit;
    end;

    OldFile := TFileStream.Create(sourcefile, fmOpenRead or fmShareDenyWrite);
    try
      NewFile := TFileStream.Create(destinationfile, fmCreate{or fmShareDenyRead});

      try
        NewFile.CopyFrom(OldFile, OldFile.Size);
      finally
        FreeAndNil(NewFile);
      end;
    finally
      FreeAndNil(OldFile);
    end;
  end
  else
    MessageDlg(Format('The source file %s does not exist.', [sourcefile]), mtError, [mbOK], 0);
end;

procedure ShowHelp;
begin
  CreateProz('hh '+
    ExtractFilePath(Application.ExeName)+'mysqladministrator.chm', '');
end;

procedure InitForm(Form: TForm);
var ClearTypeOn: Boolean;
begin
  Form.Scaled:=True;

  //Check if the current user has turned ClearType on
  ClearTypeOn:=(ReadFromReg(HKEY_CURRENT_USER, '\Control Panel\Desktop', 'FontSmoothingType', 1, '1')=
    '2');

  if(Screen.PixelsPerInch<>96)or(ClearTypeOn)then
  begin
    Form.Font.Name:='Tahoma';
    Form.Font.Size:=_d(8);
    Form.Font.Style:=[];
  end;
end;

procedure DeleteComponentWithChildren(Component: TComponent; RemoveFromList: TList);
begin
  while(Component.ComponentCount>0)do
  begin
    DeleteComponentWithChildren(Component.Components[0]);
  end;

  if(RemoveFromList<>nil)then
    if(RemoveFromList.IndexOf(Component)<>-1)then
      RemoveFromList.Delete(RemoveFromList.IndexOf(Component));

  Component.Free;
end;

function GetComputerNetName: string;
var
  buffer: array[0..255] of char;
  size: dword;
begin
  size := 256;
  if GetComputerName(buffer, size) then
    Result := buffer
  else
    Result := ''
end;

end.
