unit SchemataTreeView;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, AdvancedEdit, StdCtrls, ExtCtrls, ComCtrls, Menus,
  InstanceDataModule, AuxFuncs, myx_public_interface, MyxError;

type
  TSchemataFrame = class(TFrame)
    SchemaTreeView: TTreeView;
    TopPnl: TPanel;
    SchemataLbl: TLabel;
    SpacerPnl: TPanel;
    AdvancedEdit: TAdvancedEditFrame;
    SchemaSearchPopupMenu: TPopupMenu;
    AllcatalogsMI: TMenuItem;
    SchemaTreeViewPopupMenu: TPopupMenu;
    RefreshCatalogsSchemataListMI: TMenuItem;

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure RefreshSchemaTree(searchStr: String = '');
    procedure SchemaTreeViewCollapsing(Sender: TObject; Node: TTreeNode;
      var AllowCollapse: Boolean);
    procedure AllcatalogsMIClick(Sender: TObject);
    procedure SchemaTreeViewChanging(Sender: TObject; Node: TTreeNode;
      var AllowChange: Boolean);
    procedure RefreshCatalogsSchemataListMIClick(Sender: TObject);
    procedure AdvancedEditSearchEdChange(Sender: TObject);
    procedure SchemaTreeViewDblClick(Sender: TObject);
    procedure SetCurrentSchema(Node: TTreeNode);
    procedure SchemaTreeViewAdvancedCustomDrawItem(Sender: TCustomTreeView;
      Node: TTreeNode; State: TCustomDrawState; Stage: TCustomDrawStage;
      var PaintImages, DefaultDraw: Boolean);
    procedure SchemaTreeViewExpanding(Sender: TObject; Node: TTreeNode;
      var AllowExpansion: Boolean);
    procedure RebuildSchemaTree(MySQLMajorVersion: integer; CatalogList: TMYX_CATALOGS;
      TreeView: TTreeView; Catalog: TMYX_CATALOG; searchStr: String);

    procedure GetCatalogList(Sender: TObject);
    procedure RefreshCatalogList(Sender: TObject);
    procedure SchemaTreeViewMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure FetchSchemaTables(Sender: TObject);
    procedure SchemaTablesFetched(Sender: TObject);
    procedure RefreshTableNames(schema: TMYX_SCHEMA);
  private
    { Private declarations }
  public
    { Public declarations }
    CatalogList: TMYX_CATALOGS;
    SchemasToAdd: TStringList;

    CurrentCatalog: TMYX_CATALOG;
    CurrentSchema: TMYX_SCHEMA;
    CurrentSchemaNodes: TList;

    //User must not expand SchemaTree manually
    LockSchemaTreeExpansion: Boolean;
    LockSchemaTreeSelection: Boolean;

    InstanceDM: TInstanceDM;

    ShowSchemaAssets,
    ShowTablesOnSchemaExpansion: Boolean;
  end;

  TSchemaSubNodeType = (TSNTables, TSNIndices, TSNViews,
    TSNStoredProcedures, TSNEvents, TSNDevDocs, TSNUsers);

  TSchemaSubNode = class(TObject)
    constructor Create(Schema: TMYX_SCHEMA; SchemaSubNodeType: TSchemaSubNodeType);

    function GetSchema: TMYX_SCHEMA;
    function GetSchemaSubNodeType: TSchemaSubNodeType;
  private
    Schema: TMYX_SCHEMA;
    SchemaSubNodeType: TSchemaSubNodeType;
  end;


implementation

uses ApplicationDataModule, AdditionalClasses;

{$R *.dfm}

constructor TSchemataFrame.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  InstanceDM:=nil;
  CatalogList:=nil;
  SchemasToAdd:=TStringList.Create;

  //Create Lists
  CurrentSchemaNodes:=TList.Create;

  //Init variables
  CurrentCatalog:=nil; //Catalog;
  CurrentSchema:=nil;
  LockSchemaTreeExpansion:=True;
  LockSchemaTreeSelection:=True;

  ShowTablesOnSchemaExpansion:=False;
  ShowSchemaAssets:=True;
end;

destructor TSchemataFrame.Destroy;
begin
  CurrentSchemaNodes.Free;

  if(CatalogList<>nil)then
    CatalogList.Free;

  SchemasToAdd.Free;

  inherited Destroy;
end;


procedure TSchemataFrame.RefreshSchemaTree(searchStr: String);
begin
  if(Not(InstanceDM.Connected))then
    Exit;

  if(CatalogList=nil)then
    InstanceDM.FetchData(KindOfData_CatalogSchema,
      GetCatalogList, RefreshCatalogList,
      nil, 'Fetching Catalogs/Schemata data ...')
  else
  begin
    //Refresh SchemaTree
    LockSchemaTreeExpansion:=False;
    try
      RebuildSchemaTree(InstanceDM.MySQLMajorVersion,
        CatalogList, SchemaTreeView, CurrentCatalog,
        searchStr);
    finally
      LockSchemaTreeExpansion:=True;
    end;
  end;
end;

procedure TSchemataFrame.GetCatalogList(Sender: TObject);
var catalogs: PMYX_CATALOGS;
begin
  catalogs:=myx_get_catalogs(TFetchDataThread(Sender).InstanceDM.PMySQL);
  if(catalogs=nil)then
    raise EMyxSQLError.Create('Could not fetch Catalogs/Schemata data.',
      myx_mysql_errno(TFetchDataThread(Sender).InstanceDM.PMySQL),
      myx_mysql_error(TFetchDataThread(Sender).InstanceDM.PMySQL));

  try
    CatalogList:=TMYX_CATALOGS.Create(catalogs);
  finally
    myx_free_catalogs(catalogs);
  end;
end;


procedure TSchemataFrame.RefreshCatalogList(Sender: TObject);
var i: integer;
  MenuItem: TMenuItem;
begin
  //Refresh Popup Menu
  while(SchemaSearchPopupMenu.Items.Count>1)do
  begin
    MenuItem:=SchemaSearchPopupMenu.Items[SchemaSearchPopupMenu.Items.Count-1];
    SchemaSearchPopupMenu.Items.Delete(SchemaSearchPopupMenu.Items.Count-1);
    MenuItem.Free;
  end;

  for i:=0 to CatalogList.Catalogs.Count-1 do
  begin
    MenuItem:=TMenuItem.Create(self);
    MenuItem.Name:='Catalog'+IntToStr(i)+'MI';
    MenuItem.Caption:=CatalogList.Catalogs[i].catalog_name;
    MenuItem.OnClick:=AllcatalogsMIClick;
    SchemaSearchPopupMenu.Items.Add(MenuItem);
  end;

  RefreshSchemaTree(AdvancedEdit.SearchEd.Text);
end;

procedure TSchemataFrame.SchemaTreeViewCollapsing(Sender: TObject;
  Node: TTreeNode; var AllowCollapse: Boolean);
begin
  if(Node.Data<>nil)then
    if(TObject(Node.Data) is TMYX_CATALOG)then
      AllowCollapse:=False;
end;

procedure TSchemataFrame.AllcatalogsMIClick(Sender: TObject);
var i: integer;
begin
  for i:=0 to TMenuItem(Sender).Parent.Count-1 do
    TMenuItem(Sender).Parent.Items[i].Checked:=False;

  TMenuItem(Sender).Checked:=True;

  if(TMenuItem(Sender).Parent.IndexOf(TMenuItem(Sender))=0)then
    CurrentCatalog:=nil
  else
    CurrentCatalog:=CatalogList.catalogs[TMenuItem(Sender).Parent.IndexOf(TMenuItem(Sender))-1];

  RefreshSchemaTree(AdvancedEdit.SearchEd.Text);

  AdvancedEdit.SearchEd.SetFocus;
end;

procedure TSchemataFrame.SchemaTreeViewChanging(Sender: TObject;
  Node: TTreeNode; var AllowChange: Boolean);
begin
  AllowChange:=Not(LockSchemaTreeSelection);

  if(CurrentSchema=nil)then
    AllowChange:=True;

  if(Node.Data<>nil)then
    if(TObject(Node.Data) is TSchemaSubNode)then
      AllowChange:=True;
end;

procedure TSchemataFrame.RefreshCatalogsSchemataListMIClick(
  Sender: TObject);
var Catalogname, SchemaName: string;
begin
  if(SchemaTreeView.Selected<>nil)then
    if(SchemaTreeView.Selected.Data<>nil)then
      if(TObject(SchemaTreeView.Selected.Data) is TMYX_SCHEMA)then
      begin
        Catalogname:=TMYX_SCHEMA(SchemaTreeView.Selected.Data).catalog_name;
        SchemaName:=TMYX_SCHEMA(SchemaTreeView.Selected.Data).schema_name;
      end;

  InstanceDM.FetchData(KindOfData_CatalogSchema,
    GetCatalogList, RefreshCatalogList,
    nil, 'Fetching Catalogs/Schemata data ...');

  {for i:=0 to SchemaTreeView.Items.Count-1 do
  begin
    if(SchemaTreeView.Items[i].Data<>nil)then
      if(TObject(SchemaTreeView.Items[i].Data) is TMYX_SCHEMA)then
        if(TMYX_SCHEMA(SchemaTreeView.Items[i].Data).catalog_name=Catalogname)and
          (TMYX_SCHEMA(SchemaTreeView.Items[i].Data).schema_name=SchemaName)then
        begin
          SchemaTreeView.Selected:=SchemaTreeView.Items[i];
          break;
        end;
  end;}
end;

procedure TSchemataFrame.AdvancedEditSearchEdChange(Sender: TObject);
begin
  AdvancedEdit.SearchEdChange(Sender);

  RefreshSchemaTree(AdvancedEdit.SearchEd.Text);

  //if there is no Node selected
  if(SchemaTreeView.SelectionCount=0)then
    CurrentSchema:=nil;
end;

procedure TSchemataFrame.SchemaTreeViewDblClick(Sender: TObject);
var Node: TTreeNode;
  P: TPoint;
begin
  P:=SchemaTreeView.ScreenToClient(Mouse.CursorPos);
  Node:=SchemaTreeView.GetNodeAt(P.X, P.Y);

  if(Node<>nil)then
    if(Node.Data<>nil)then
      if(TObject(Node.Data) is TMYX_SCHEMA)then
      begin
        if(CurrentSchema<>Node.Data)then
          SetCurrentSchema(Node)
        else
          if(CurrentSchemaNodes.Count>0)then
          begin
            TTreeNode(CurrentSchemaNodes[0]).Expanded:=False;
            //CatalogPageControl.ActivePageIndex:=0;
            CurrentSchema:=nil;
            SchemaTreeView.Invalidate;
          end;
      end
      else if(TObject(Node.Data) is TMYX_CATALOG)then
      begin
        if(CurrentSchemaNodes.Count>0)then
        begin
          TTreeNode(CurrentSchemaNodes[0]).Expanded:=False;
          //CatalogPageControl.ActivePageIndex:=0;
          CurrentSchema:=nil;
          SchemaTreeView.Invalidate;
        end;
      end;
end;

procedure TSchemataFrame.SetCurrentSchema(Node: TTreeNode);
var i: integer;
begin
  CurrentSchema:=TMYX_SCHEMA(Node.Data);

  if(CurrentSchemaNodes.Count>0)then
    TTreeNode(CurrentSchemaNodes[0]).Collapse(False);

  CurrentSchemaNodes.Clear;
  CurrentSchemaNodes.Add(Node);
  for i:=0 to Node.Count-1 do
    CurrentSchemaNodes.Add(Node.Item[i]);

  //SchemaTreeView.FullCollapse;

  LockSchemaTreeExpansion:=False;
  try
    Node.Expand(False);
  finally
    LockSchemaTreeExpansion:=True;
  end;

  {BuildSchemaPages(CurrentSchema);

  //For MySQL Versions < 5 hide Views, StoredProcedures and Events
  if(TInstanceForm(Owner).MySQLMajorVersion<5)and(SchemaPageControl.ActivePageIndex>1)then
    SchemaTreeView.Selected:=Node.Item[SchemaPageControl.ActivePageIndex-3]
  else
    SchemaTreeView.Selected:=Node.Item[SchemaPageControl.ActivePageIndex];

  CatalogPageControl.ActivePage:=SchemaSheet;

  CatalogPnlResize(self);}
end;


procedure TSchemataFrame.SchemaTreeViewAdvancedCustomDrawItem(
  Sender: TCustomTreeView; Node: TTreeNode; State: TCustomDrawState;
  Stage: TCustomDrawStage; var PaintImages, DefaultDraw: Boolean);
{var NodeRect: TRect;
  doClear, doHighlight: Boolean;}
begin
  Exit;

  {//Workaround for Delphi bug causing Node's caption to be written in Bold
  MyCanvas:=TControlCanvas.Create;
  try
    MyCanvas.Control:=TreeView1;
    MyCanvas.Brush.Color:=clRed;
    MyCanvas.FillRect(Rect);
  finally
    MyCanvas.Free;
  end;}

  {if(Stage=cdPostPaint)then
  begin
    SchemaTreeView.Canvas.Font.Size:=6;
    if(ApplicationDM.AdminTree16ImageList<>nil)then
      if(Node.Data<>nil)then
      begin
        NodeRect:=Node.DisplayRect(False);

        doHighlight:=False;
        if(Node.Level=2)then
          if(Node.Parent.Data=CurrentSchema)and
            (SchemaPageControl.ActivePageIndex=Node.Parent.IndexOf(Node))then
            doHighlight:=True;

        //Draw
        doClear:=True;
        if(TObject(Node.Data) is TMYX_SCHEMA)then
          if(TMYX_SCHEMA(Node.Data)=CurrentSchema)then
          begin
            SchemaTreeView.Canvas.Brush.Color:=$00EFD3C6;
            SchemaTreeView.Canvas.Pen.Color:=$00C66931;
            SchemaTreeView.Canvas.Rectangle(
              Rect(NodeRect.Left+Node.Level*19+20, NodeRect.Top,
              NodeRect.Left+Node.Level*19+28+
                SchemaTreeView.Canvas.TextWidth(Node.Text),
              NodeRect.Bottom));
            doClear:=False;
          end;

        if(doClear)then
        begin
          SchemaTreeView.Canvas.Brush.Color:=$00FFFFFF;
          SchemaTreeView.Canvas.FillRect(
            Rect(NodeRect.Left+Node.Level*19+18-ord(doHighlight)*20, NodeRect.Top,
            NodeRect.Right, NodeRect.Bottom));
        end;

        if(doHighlight)then
        begin
          SchemaTreeView.Canvas.Polygon(
            [Point(NodeRect.Left+Node.Level*19+22, NodeRect.Top+3),
            Point(NodeRect.Left+Node.Level*19+26, NodeRect.Top+7),
            Point(NodeRect.Left+Node.Level*19+22, NodeRect.Top+11)]);

          SchemaTreeView.Canvas.Font.Style:=[fsBold];
        end;

        AdminDM.AdminTree16ImageList.Draw(SchemaTreeView.Canvas,
          NodeRect.Left+Node.Level*19+2,
          NodeRect.Top,
          Node.ImageIndex);

        //Text Out
        SchemaTreeView.Canvas.Brush.Style:=bsClear;
        //SetBkMode(SchemaTreeView.Canvas.Handle, TRANSPARENT);

        SchemaTreeView.Canvas.Font.Color:=clBlack;

        SchemaTreeView.Canvas.TextRect(
          Rect(NodeRect.Left+Node.Level*19+22, NodeRect.Top+1,
          NodeRect.Right, NodeRect.Top+16),
          NodeRect.Left+Node.Level*19+22+Ord(doHighlight)*7, NodeRect.Top+1,
          Node.Text);

        SchemaTreeView.Canvas.Font.Style:=[];

        DefaultDraw:=False;
      end;
  end;}
end;

procedure TSchemataFrame.SchemaTreeViewExpanding(Sender: TObject;
  Node: TTreeNode; var AllowExpansion: Boolean);
begin
  if(ShowTablesOnSchemaExpansion)then
    if(Node.Data<>nil)then
      if(TObject(Node.Data) is TMYX_SCHEMA)then
        if(TMYX_SCHEMA(Node.Data).schema_tables=nil)then
        begin
          Node.DeleteChildren;
          RefreshTableNames(TMYX_SCHEMA(Node.Data));
        end;
end;


function SortSchemaList(Item1, Item2: Pointer): Integer;
begin
  SortSchemaList:=CompareStr(TMYX_SCHEMA(Item1).schema_name, TMYX_SCHEMA(Item2).schema_name);
end;

function StrMatchesSearchStr(Str: String; searchStr: String): Boolean;
begin
  StrMatchesSearchStr:=True;

  if(searchStr<>'')then
    StrMatchesSearchStr:=(myx_match_string(PChar(AnsiToUtf8(Str)),
      PChar(AnsiToUtf8(searchStr)), 0)<>0);
end;

function AddSchemaToTree(MySQLMajorVersion: integer; TreeView: TTreeView; CatalogNode: TTreeNode;
  Schema: TMYX_SCHEMA; searchStr: String; ShowSchemaAssets: Boolean): TTreeNode;
var SchemaNode: TTreeNode;
begin
  SchemaNode:=AddTreeViewChildNode(TreeView, CatalogNode,
    //TCatalog(Schema.GetCatalog).GetCatalogName+'.'+
    Schema.schema_name, 9,
    Schema);

  if(ShowSchemaAssets)then
  begin
    //Add SchemaSubNodes
    AddTreeViewChildNode(TreeView, SchemaNode, 'Tables', 10,
      TSchemaSubNode.Create(Schema, TSNTables));
    AddTreeViewChildNode(TreeView, SchemaNode, 'Indices', 12,
      TSchemaSubNode.Create(Schema, TSNIndices));

    //MySQL 5.0 (not now)
    if(MySQLMajorVersion>=5+100)then
    begin
      AddTreeViewChildNode(TreeView, SchemaNode, 'Views', 10,
        TSchemaSubNode.Create(Schema, TSNViews));
      AddTreeViewChildNode(TreeView, SchemaNode, 'Stored Procedures', 13,
        TSchemaSubNode.Create(Schema, TSNStoredProcedures));
      AddTreeViewChildNode(TreeView, SchemaNode, 'Events', 13,
        TSchemaSubNode.Create(Schema, TSNEvents));
    end;

    {AddTreeViewChildNode(TreeView, SchemaNode, 'Developer Docs', 18,
      TSchemaSubNode.Create(Schema, TSNDevDocs));}

    AddTreeViewChildNode(TreeView, SchemaNode, 'Users', 5,
      TSchemaSubNode.Create(Schema, TSNUsers));
  end;

  Result:=SchemaNode;
end;

procedure TSchemataFrame.RebuildSchemaTree(MySQLMajorVersion: integer; CatalogList: TMYX_CATALOGS; TreeView: TTreeView; Catalog: TMYX_CATALOG; searchStr: String);
var i, j, k: integer;
  CatalogNode, Node: TTreeNode;
  LastCatalogName, LastSchemaName, schema_name: string;
  found: boolean;
begin
  if(TreeView.Selected<>nil)then
    if(TreeView.Selected.Data<>nil)then
      if(TObject(TreeView.Selected.Data) is TMYX_SCHEMA)then
      begin
        LastCatalogName:=TMYX_SCHEMA(TreeView.Selected.Data).catalog_name;
        LastSchemaName:=TMYX_SCHEMA(TreeView.Selected.Data).schema_name;
      end;

  TreeView.Items.Clear;

  if(CatalogList=nil)then
    Exit;

  CatalogNode:=nil;

  if(Catalog=nil)then
  begin
    for i:=0 to CatalogList.Catalogs.Count-1 do
    begin
      {CatalogNode:=AddTreeViewChildNode(TreeView, nil,
        CatalogList.Catalogs[i].catalog_name, 2,
        CatalogList.Catalogs[i]);}

      for j:=0 to CatalogList.Catalogs[i].schemata.Count-1 do
        if(StrMatchesSearchStr(
          CatalogList.Catalogs[i].schemata[j].schema_name, searchStr))then
        begin
          Node:=AddSchemaToTree(MySQLMajorVersion, TreeView, CatalogNode,
            CatalogList.Catalogs[i].schemata[j], searchStr, ShowSchemaAssets);

          if(CatalogList.Catalogs[i].catalog_name=LastCatalogName)and
            (CatalogList.Catalogs[i].schemata[j].schema_name=LastSchemaName)then
            TreeView.Selected:=Node;
        end;

      for j:=0 to SchemasToAdd.Count-1 do
      begin
        if(CompareText(
          Copy(SchemasToAdd[j], 1, Pos('/', SchemasToAdd[j])-1),
          CatalogList.Catalogs[i].catalog_name)=0)then
        begin
          found:=False;
          schema_name:=Copy(SchemasToAdd[j], Pos('/', SchemasToAdd[j])+1,
            Length(SchemasToAdd[j]));

          for k:=0 to CatalogList.Catalogs[i].schemata.Count-1 do
            if(CompareText(schema_name,
              CatalogList.Catalogs[i].schemata[k].schema_name)=0)then
            begin
              found:=True;
              break;
            end;

          if(Not(found))and(StrMatchesSearchStr(schema_name, searchStr))then
          begin
            Node:=AddSchemaToTree(MySQLMajorVersion, TreeView, CatalogNode,
              TMYX_SCHEMA.create(schema_name, CatalogList.Catalogs[i].catalog_name, nil),
                searchStr, ShowSchemaAssets);

            if(CatalogList.Catalogs[i].catalog_name=LastCatalogName)and
              (schema_name=LastSchemaName)then
              TreeView.Selected:=Node;
          end;
        end;
      end;

      if(CatalogNode<>nil)then
        if(CatalogNode.HasChildren)then
          CatalogNode.Expand(False)
        else
          TreeView.Items.Delete(CatalogNode);
    end;
  end
  else
  begin
    CatalogNode:=AddTreeViewChildNode(TreeView, nil,
      Catalog.catalog_name, 2,
      Catalog);

    k:=0;
    for j:=0 to Catalog.schemata.Count-1 do
      if(StrMatchesSearchStr(Catalog.schemata[j].schema_name, searchStr))then
      begin
        Node:=AddSchemaToTree(MySQLMajorVersion, TreeView, CatalogNode,
          Catalog.schemata[j], searchStr, ShowSchemaAssets);
        inc(k);

        if(Catalog.catalog_name=LastCatalogName)and
          (Catalog.schemata[j].schema_name=LastSchemaName)then
          TreeView.Selected:=Node;
      end;

    if(k>0)then
      CatalogNode.Expand(False)
    else
      TreeView.Items.Delete(CatalogNode);
  end;
end;

//------------------------------------------------------------------------------

constructor TSchemaSubNode.Create(Schema: TMYX_SCHEMA; SchemaSubNodeType: TSchemaSubNodeType);
begin
  self.Schema:=Schema;
  self.SchemaSubNodeType:=SchemaSubNodeType;
end;

function TSchemaSubNode.GetSchema: TMYX_SCHEMA;
begin
  GetSchema:=Schema;
end;

function TSchemaSubNode.GetSchemaSubNodeType: TSchemaSubNodeType;
begin
  GetSchemaSubNodeType:=SchemaSubNodeType;
end;


procedure TSchemataFrame.SchemaTreeViewMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var Node: TTreeNode;
  ClientPoint: TPoint;
begin
  if(ssRight in Shift)then
  begin
    ClientPoint:=Point(X, Y);
    Node:=SchemaTreeView.GetNodeAt(ClientPoint.X, ClientPoint.Y);

    if(Node<>nil)then
      SchemaTreeView.Selected:=Node;
  end;
end;

procedure TSchemataFrame.FetchSchemaTables(Sender: TObject);
var SchemaTables: PMYX_SCHEMA_TABLES;
begin
  SchemaTables:=myx_get_schema_tables(TFetchDataThread(Sender).InstanceDM.PMySQL,
    PChar(''),
    PChar(AnsiToUtf8(TMYX_SCHEMA(TFetchDataThread(Sender).Target).schema_name)));
  if(SchemaTables=nil)then
    raise EMyxSQLError.Create('Could not fetch Schema Tables.',
      myx_mysql_errno(TFetchDataThread(Sender).InstanceDM.PMySQL),
      myx_mysql_error(TFetchDataThread(Sender).InstanceDM.PMySQL));

  TMYX_SCHEMA(TFetchDataThread(Sender).Target).schema_tables:=
    TMYX_SCHEMA_TABLES.create(SchemaTables);
  myx_free_schema_tables(SchemaTables);
end;

procedure TSchemataFrame.SchemaTablesFetched(Sender: TObject);
begin
  RefreshTableNames(TMYX_SCHEMA(TFetchDataThread(Sender).Target));
end;

procedure TSchemataFrame.RefreshTableNames(schema: TMYX_SCHEMA);
var i, j: integer;
  SchemaNode, TableNode: TTreeNode;
begin
  if(schema.schema_tables=nil)then
    InstanceDM.FetchData(KindOfData_SchemaTables,
      FetchSchemaTables, SchemaTablesFetched,
      schema, 'Fetching Tables ...')
  else
  begin
    SchemaTreeView.Items.BeginUpdate;
    try
      //Find schema node
      SchemaNode:=nil;
      for i:=0 to SchemaTreeView.Items.Count-1 do
        if(SchemaTreeView.Items[i].Data=
          schema)then
          SchemaNode:=SchemaTreeView.Items[i];

      //Attach tables to schema node
      if(SchemaNode<>nil)then
      begin
        SchemaNode.DeleteChildren;
        for i:=0 to schema.schema_tables.schema_tables.Count-1 do
        begin
          TableNode:=AddTreeViewChildNode(SchemaTreeView,
            SchemaNode,
            schema.schema_tables.schema_tables[i].table_name, 10,
            schema.schema_tables.schema_tables[i]);

          for j:=0 to schema.schema_tables.schema_tables[i].columns.Count-1 do
          begin
            AddTreeViewChildNode(SchemaTreeView,
              TableNode,
              schema.schema_tables.schema_tables[i].columns[j].column_name,
              11+(schema.schema_tables.schema_tables[i].columns[j].is_primary_key*13),
              schema.schema_tables.schema_tables[i].columns[j]);
          end;

          SchemaNode.Expand(False);
        end;
      end;
    finally
      SchemaTreeView.Items.EndUpdate;
    end;
  end;
end;

end.
